import json
import math
import os
import time
from pathlib import Path

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread
from PySide2.QtGui import QDoubleValidator, QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QFileDialog, QApplication
from axipy import Settings

from RasterGridTool import RasterSource
from RasterGridTool.Test.Grid.Contour.widgetCountourLevel import CountourLevelWidget
from RasterGridTool.Test.Grid.ui.process_tool.build_contour import CreateContourProcess


def getRasterTab():
    def isRaster(obj):
        if not isinstance(obj, axipy.Raster):
            return False
        return True
    rasters = list(filter( isRaster, axipy.da.data_manager.objects))
    return rasters
def existRasterTab(list_tab,path_base):
    for i,item in enumerate(list_tab):
        if item['path']==path_base:
            return i
    return None
def getListGridTabNoDuble(list_raster):
    list_grid=[]
    for rast in list_raster:
        path_name=rast.properties['fileName']
        path_name=str(Path(path_name))
        tem_rast=RasterSource()
        isGrid=tem_rast.OpenRead(path_name)
        if isGrid:
            min,max=tem_rast.MinMax
            if tem_rast.IsVrt:
                base_path=tem_rast.vrtBaseGrid
                path_vrt=str(Path(path_name).parent)
                full_base_path=os.path.join(path_vrt,base_path[0]['file'])
                index=existRasterTab(list_grid,full_base_path)
                if index is None:
                    list_grid.append({'raster':rast,'path':full_base_path,'min':min,'max':max})
                else:
                    list_grid.pop(index)
                    list_grid.append({'raster':rast,'path':full_base_path,'min':min,'max':max})
            else:
                index=existRasterTab(list_grid,path_name)
                if index is None:
                    list_grid.append({'raster':rast,'path':path_name,'min':min,'max':max})
        tem_rast.close()
    if len(list_grid)==0:
        return []
    return list_grid

def getListGrid():
    li_raster=getRasterTab()
    list_grid_uniqal=getListGridTabNoDuble(li_raster)
    return list_grid_uniqal

class CreateContour(QWidget):
    __def_count_interval=5
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        super(CreateContour, self).__init__(parent)
        self.__parentWin=parent
        self.load_ui()
        curent_path=str(Path(__file__).parent)
        path_icon=os.path.join(curent_path,"icons")
        path_add_png=os.path.join(path_icon,"saveas.png")
        self.window.pb_saveAs.setIcon(QIcon(path_add_png))
        self.window.pb_refresh_min.setIcon(QIcon(os.path.join(path_icon,"refresh.png")))
        self.window.pb_refresh_max.setIcon(QIcon(os.path.join(path_icon,"refresh.png")))
        self.__widgetLevels=CountourLevelWidget(None)
        self.window.vl_intervals.addWidget(self.__widgetLevels.window)
        self.window.pb_update_interval.clicked.connect(self.__update_interval)
        self.window.pb_save.clicked.connect(self.__save_intervals)
        self.window.pb_load.clicked.connect(self.__load_intervals)
        '''
        self.__testParm()
        
        self.__curent_step=(self.__curent_max-self.__curent_min)/self.__def_count_interval
        '''
        self.__min_validator=QDoubleValidator()
        #self.__min_validator.setTop(self.__curent_max)
        self.window.ln_min.setValidator(self.__min_validator)
        '''
        self.window.ln_min.setText(str(self.__curent_min))
        self.window.ln_max.setText(str(self.__curent_max))
        self.window.ln_step.setText(str(self.__curent_step))
        '''
        self.window.ln_min.textChanged.connect(self.__change_min)
        self.window.ln_max.textChanged.connect(self.__change_max)
        self.window.cmb_grids.currentIndexChanged.connect(self.__change_grid)
        self.window.pb_saveAs.clicked.connect(self.__change_out_file)
        self.window.pb_run.clicked.connect(self.__run)
        self.window.pb_cancel.clicked.connect(self.__cancel)
        self.window.pb_refresh_min.clicked.connect(self.__refresh_min)
        self.window.pb_refresh_max.clicked.connect(self.__refresh_max)
    def load_ui(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "ContourBuild.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window  = loader.load(ui_file,self.__parentWin)
        ui_file.close()
    def __testParm(self):
        self.__curent_min=100
        self.__curent_max=600
    def __update_interval(self):
        params_init={}
        params_init['min']=float(self.window.ln_min.text())
        params_init['max']=float(self.window.ln_max.text())
        step=float(self.window.ln_step.text())

        #params_init['count']=int((params_init['max']-params_init['min'])/step)
        params_init['step']=step
        self.__widgetLevels.updateIntervals(params_init)
        self.window.pb_run.setEnabled(True)
        self.window.pb_save.setEnabled(True)
    def __change_min(self):
       curent_min=float(self.window.ln_min.text())
       if curent_min>=self.__curent_max:
           self.window.ln_min.setText(str(self.__curent_min))
           return
       self.__curent_min=curent_min
       self.__update_count_intervals()
    def __change_max(self):
        try:
            curent_max=float(self.window.ln_max.text())
        except:
            return
        if curent_max<=self.__curent_min:
            self.window.ln_max.setText(str(self.__curent_max))
            return
        self.__curent_max=curent_max
        self.__update_count_intervals()
    def __update_count_intervals(self):
        count=math.ceil((self.__curent_max-self.__curent_min)/float(self.window.ln_step.text()))
        self.window.ln_count.setText(str(count))
    def updateGrids(self,clearPrev=True):
        self.__list_grid=getListGrid()
        if len(self.__list_grid)==0:
            return
        if clearPrev:
            self.window.cmb_grids.clear()
        for grid in self.__list_grid:
            name=grid['raster'].name
            self.window.cmb_grids.addItem(name)
        self.window.cmb_grids.setCurrentIndex(0)
        self.__change_grid(0)
    def __change_grid(self,index):
        #print("Select index:"+str(index))
        self.__curent_min=self.__list_grid[index]['min']
        self.__curent_max=self.__list_grid[index]['max']
        self.__curent_step=(self.__curent_max-self.__curent_min)/self.__def_count_interval
        self.window.ln_step.setText(str(self.__curent_step))
        self.window.ln_min.setText(str(self.__curent_min))
        self.window.ln_max.setText(str(self.__curent_max))
        path_source=self.__list_grid[index]['path']
        base_path=str(Path(path_source).parent)
        name=str(Path(path_source).stem)
        out_path_contour=os.path.join(base_path,name+"_contour.tab")
        self.window.ln_file_contour.setText(out_path_contour)
        self.window.ln_file_contour.setCursorPosition(0)
    def __change_out_file(self):
            ext_files = "MapInfo tab (*.tab)"
            name_file=self.window.ln_file_contour.text()
            name_load_file = QFileDialog.getSaveFileName(axipy.app.mainwindow.qt_object(), 'Сохранить в tab',name_file, ext_files)
            if not(name_load_file[0] ==''):
                new_file=name_load_file[0]
                self.window.ln_file_contour.setText(new_file)
                self.window.ln_file_contour.setCursorPosition(0)
    def __run(self):
        path_out=self.window.ln_file_contour.text()
        levels=self.__widgetLevels.getLevels()
        source_grid=self.__list_grid[self.window.cmb_grids.currentIndex()]['path']
        smothing=self.window.doubleSmothing.text()
        self.__process=CreateContourProcess(self)
        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.finished.connect(self.__end_process)
        self.__process.setParametrs(path_out,levels,{'path':source_grid,'band':1},float(smothing))
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        self.window.pb_cancel.setEnabled(True)
        self.__process.run()
    def __end_process(self,value):
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
        self.__process.updateProcess.disconnect()
        self.__process.finished.disconnect()
        #if value==100:
    def __cancel(self):
        self.cancelProcess.emit()
        self.__process.setCancel()
    def __update_progress(self,value):
        self.window.progress_cntour.setValue(value)
        #print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def setFormatprogressBar(self,str_format):
        self.window.progress_cntour.setFormat(str_format)
    def __save_intervals(self):
        ''' Сохранение интервалов в файл'''
        levels=self.__widgetLevels.getLevels()
        last_path=Settings.value(Settings.LastOpenPath)
        ext_files = "Интервалы  (*.levels)"
        name_file=os.path.join(last_path,"*.levels")
        name_load_file = QFileDialog.getSaveFileName(axipy.app.mainwindow.qt_object(), 'Сохранить интервалы',name_file, ext_files)
        if not(name_load_file[0] ==''):
            new_file=name_load_file[0]
            json_levels={'levels':levels}
            with open(new_file,"w", encoding='utf-8') as jsonfile:
                json.dump(json_levels,jsonfile,ensure_ascii=False)
    def __load_intervals(self):
        ''' Загрузка интервалов'''
        last_path=Settings.value(Settings.LastOpenPath)
        ext_files = "Интервалы  (*.levels)"
        name_file=os.path.join(last_path,"*.levels")
        name_load_file = QFileDialog.getOpenFileName(axipy.app.mainwindow.qt_object(), 'Загрузить интервалы',name_file, ext_files)
        if not(name_load_file[0] ==''):
            new_file=name_load_file[0]
            with open(new_file, encoding='utf-8') as fh:
                levels_data = json.load(fh)
                self.__widgetLevels.setIntervals(levels_data['levels'])
                self.window.pb_run.setEnabled(True)
                self.window.pb_save.setEnabled(True)
    def __refresh_min(self):
        self.window.ln_min.setText(str(self.__curent_min))
    def __refresh_max(self):
        self.window.ln_max.setText(str(self.__curent_max))
    @property
    def widget(self):
        return self.window

    