import math
import os

from PySide2.QtCore import QFile
from PySide2.QtGui import QDoubleValidator
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget

def valueToString(value,isLatlog=False):
    if isLatlog:
        patern_format="{value:.5f}"

    else:
        patern_format="{value:.2f}"
    return patern_format.format(value=value)
class DlgGridSize(QWidget):
    __def_max_col=100
    def __init__(self,parent=None):
        self.__parentWin=parent
        self.load_ui()
        self.__db_validator_step=QDoubleValidator()
        self.__db_validator_step.setBottom(0.1)
        self.window.ln_step.textChanged.connect(self.__change_step)

        self.window.ln_step.setValidator(self.__db_validator_step)
        self.window.ln_max_x.setValidator(QDoubleValidator())
        self.window.ln_max_y.setValidator(QDoubleValidator())
        self.window.ln_min_x.setValidator(QDoubleValidator())
        self.window.ln_min_y.setValidator(QDoubleValidator())
        self.window.ln_min_x.textChanged.connect(self.__change_xmin)
        self.window.ln_min_y.textChanged.connect(self.__change_ymin)

    def load_ui(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "widgetGridSize.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window  = loader.load(ui_file,self.__parentWin)
        ui_file.close()
    def setBound(self,xmin,ymin,xmax,ymax,isLatLong=False):
        self.__xmin=xmin
        self.__ymin=ymin
        self.__xmax=xmax
        self.__ymax=ymax
        self.__isLatLong=isLatLong
        if (self.__xmax-self.__xmin)>(self.__ymax-self.__ymin):
            self.__step=(self.__xmax-self.__xmin)/self.__def_max_col
        else:
            self.__step=(self.__ymax-self.__ymin)/self.__def_max_col
        self.__calcNxNy(True)
        self.__setValue()

    def __setValue(self):
        if self.__isLatLong:
            patern_format="{value:.5f}"
            self.__db_validator_step.setBottom(0.00001)
            self.__db_validator_step.setDecimals(6)
        else:
            patern_format="{value:.2f}"
            self.__db_validator_step.setBottom(0.1)
            self.__db_validator_step.setDecimals(2)

        self.window.ln_min_x.setText( patern_format.format(value=self.__xmin))
        self.window.ln_min_y.setText( patern_format.format(value=self.__ymin))
        self.window.ln_max_x.setText( patern_format.format(value=self.__xmax))
        self.window.ln_max_y.setText( patern_format.format(value=self.__ymax))
        if (self.__xmax-self.__xmin)>(self.__ymax-self.__ymin):
            self.__step=(self.__xmax-self.__xmin)/self.__def_max_col
        else:
            self.__step=(self.__ymax-self.__ymin)/self.__def_max_col
    def __calcNxNy(self,updateStep=True):
        width=math.ceil((self.__xmax-self.__xmin)/(self.__step))
        height=math.ceil((self.__ymax-self.__ymin)/(self.__step))
        self.window.ln_nx.setText(str(width))
        self.window.ln_ny.setText(str(height))
        if self.__isLatLong:
            patern_format="{value:.5f}"
        else:
            patern_format="{value:.2f}"
        if updateStep:
            self.window.ln_step.setText(patern_format.format(value=self.__step))
    def __change_step(self,str_value):
        if str_value=='':
            return
        if float(str_value)<=0:
            return
        self.__step=float(str_value)
        self.__calcNxNy(False)
    def __change_xmin(self,str_xmin):
        if float(str_xmin)>=self.__xmax:
            temp_value=self.__xmax-3*self.__step
            self.window.ln_min_x.setText(valueToString(temp_value,self.__isLatLong))
            self.__xmin=temp_value
        else:
            self.__xmin=float(str_xmin)
        self.__calcNxNy(False)
    def __change_ymin(self,str_ymin):
        if float(str_ymin)>=self.__ymax:
            temp_value=self.__ymax-3*self.__step
            self.window.ln_min_x.setText(valueToString(temp_value,self.__isLatLong))
            self.__ymin=temp_value
        else:
            self.__ymin=float(str_ymin)
        self.__calcNxNy(False)
    @property
    def Nx(self):
        return int(self.window.ln_nx.text())
    @property
    def Ny(self):
        return int(self.window.ln_ny.text())
    @property
    def bound(self):
        step=float(self.window.ln_step.text())
        xmin=float(self.window.ln_min_x.text())-3*step
        ymin=float(self.window.ln_min_y.text())-3*step
        xmax=float(self.window.ln_max_x.text())+3*step
        ymax=float(self.window.ln_max_y.text())+3*step
        return [xmin,ymin,xmax,ymax]





