import xml.etree.ElementTree as ET

from RasterGridTool.toolprocessing.grid_io.ColorTable import ItemColorMap, ColorMap, ColorTable


def lut_interval(list_lut_values,no_value):
    out_color=[]
    for item_lut in list_lut_values:
        values=item_lut.split(':')
        if values[0]==no_value:
            continue
        color_value={'value':float(values[0]),"color":int(values[1])}
        out_color.append(color_value)
    return out_color
class VrtRaterBand:
    def __init__(self,color:str,no_value:str,list_colr_value):
        self.__name_color=color
        self.__no_value=no_value
        self.__value_colors=list_colr_value
    @property
    def NameColor(self):
        return self.__name_color
    @property
    def size(self):
        return len(self.__value_colors)
    @property
    def MinValue(self):
        return self.__value_colors[0]['value']
    @property
    def MaxValue(self):
        return self.__value_colors[self.size-1]['value']

    def ValueItem(self,index):
        return self.__value_colors[index]['value'],self.__value_colors[index]['color']
    @staticmethod
    def factory(root_xml):
        tag_NoDataValue=findXmlTag(root_xml,"NoDataValue")
        str_NoDataValue=None
        if tag_NoDataValue is not None:
            str_NoDataValue=tag_NoDataValue.text
        tag_ColorInterp=findXmlTag(root_xml,"ColorInterp")
        str_color=None
        if tag_ColorInterp is not None:
            str_color=tag_ColorInterp.text
        if str_color is None:
            return None
        tag_ComplexSource=findXmlTag(root_xml,"ComplexSource")
        if tag_ComplexSource is None:
            return None
        tag_lut=findXmlTag(tag_ComplexSource,"LUT")
        if tag_lut is None:
            return None
        str_lut=tag_lut.text
        values_intrevals_color=str_lut.split(',')
        lut=lut_interval(values_intrevals_color,str_NoDataValue)
        if len(lut)==0:
            return None
        return VrtRaterBand(str_color,str_NoDataValue,lut)

def findXmlTag(root_xml,name_tag):

    for item in root_xml:
        if item.tag==name_tag:
            return item
    return None
def getLutInBand(root_xml):
    str_nodata=None
    str_color=None
    for item in root_xml:
        if item.tag=='NoDataValue':
            str_nodata=item.text
        if item.tag=='ColorInterp':
            str_color=item.text
        if item.tag=='ComplexSource':
            jkl=0
def getColorTable(root_xml):
    #teg_vrtRaster=findXmlTag(root_xml,'VRTRasterBand')
    list_Raster_band=[]
    for item in root_xml:
        if item.tag=='VRTRasterBand':
            band_item=VrtRaterBand.factory(item)
            if band_item is not None:
                list_Raster_band.append(band_item)
            jkl=0
    if len(list_Raster_band)!=3:
        return None
    isValid=True
    max_len=0
    color_index={}
    for i,item_Color in enumerate(list_Raster_band):
        cut_len=item_Color.size
        name_color=item_Color.NameColor
        if name_color.lower()=="red":
            color_index['r']=i
        if name_color.lower()=="green":
            color_index['g']=i
        if name_color.lower()=="blue":
            color_index['b']=i
        if i==0:
            max_len=cut_len
            continue
        if cut_len!=max_len:
            isValid=False
            break
    min_value=list_Raster_band[0].MinValue
    max_value=list_Raster_band[0].MaxValue
    val_proc=(max_value-min_value)/100.0
    itemItemColorMap=[]
    for i in range(list_Raster_band[0].size):
        val_r,col_r=list_Raster_band[color_index['r']].ValueItem(i)
        val_g,col_g=list_Raster_band[color_index['g']].ValueItem(i)
        val_b,col_b=list_Raster_band[color_index['b']].ValueItem(i)
        proc_value=(val_r-min_value)/val_proc
        itemItemColorMap.append(ItemColorMap(proc_value,int(col_r),int(col_g),int(col_b)))
    colorMap=ColorMap()
    colorMap.createFromItemColorMap(1,1,itemItemColorMap)
    return ColorTable(colorMap)
path_file_xml='E:\\Support\\GridTest\\Elevation_vrt.vrt'
tree = ET.parse(path_file_xml)
root=tree.getroot()
teg_vrtRaster=findXmlTag(root,'VRTRasterBand')
#VrtRaterBand.factory(teg_vrtRaster)
element = root[0]
color_tab=getColorTable(root)
w=200
h=100
image=color_tab.getQImageBaseClr(w,h)
path_save="e:/temp/Vrt_pal.png"
image.save(path_save)
print("End")