from concurrent.futures import ThreadPoolExecutor
from multiprocessing.pool import ThreadPool

import axipy
import vtk
from axipy import Table, Pnt, Geometry


def getPointFromPoint(geo):
    return [Pnt(geo.x,geo.y)]
    #return [(geo.x,geo.y,)]
def getPointFromPolygon(geo):
    points=[]
    points.extend(geo.points)
    if geo.type==axipy.GeometryType.LineString :
        return points
    del points[-1]
    for geo_hole in geo.holes:
        points.extend(geo_hole)
        del points[-1]
    return points
def getPointFromLine(geo):
    return [geo.begin,geo.end]
def getPointsCollection(geo:Geometry):
    points=[]
    for g in geo:
        pts_g=getPoints(g)
        if pts_g is not None:
            points.extend(pts_g)
    if len(points)==0:
        return None
    return points
def getPoints(geo:Geometry):
    points_source=None
    if geo.type == axipy.GeometryType.Point:
        return getPointFromPoint(geo)
    if geo.type == axipy.GeometryType.MultiPoint:
        points_source = getPointsCollection(geo)
    elif geo.type == axipy.GeometryType.Polygon:
        points_source = getPointFromPolygon(geo)
    elif geo.type == axipy.GeometryType.Line:
        points_source = getPointFromLine(geo)
    elif geo.type == axipy.GeometryType.LineString:
        points_source = getPointFromPolygon(geo)
    elif geo.type == axipy.GeometryType.LineString:
        points_source = getPointFromPolygon(geo)
    elif geo.type == axipy.GeometryType.MultiLineString:
        points_source = getPointsCollection(geo)
    elif geo.type == axipy.GeometryType.MultiPolygon:
        points_source = getPointsCollection(geo)
    elif geo.type == axipy.GeometryType.GeometryCollection:
        points_source = getPointsCollection(geo)
    return points_source
def getPointsZ(ft_virtual):
    z_value=ft_virtual['z']
    points=getPoints(ft_virtual['geometry'])
    return {'z':z_value,'points':points}
def getPointsZ1(ft_virtual):
    z_value=ft_virtual['z']
    points=getPoints(ft_virtual['geometry'])
    '''
    vtk_points=vtk.vtkPoints()
    for pnt in points:
        vtk_points.InsertNextPoint(pnt.x,pnt.y,z_value)
    '''
    ft_virtual['points']=points
    return
def getPointInPool(list_ft,max_pool=24):
    p = ThreadPool(max_pool)
    result = p.map(getPointsZ,list_ft)
    p.close()
    p.join()
    p.close()
    return result

def getPointsPoolExicutor(listft,max_pool=12):
    with ThreadPoolExecutor(max_workers=max_pool) as executor:
        executor.map(getPointsZ1, listft)
    return