import os
import time
from pathlib import Path

import axipy
import vtk
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread
from PySide2.QtGui import QColor
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication, QFileDialog, QColorDialog
from axipy import Table
from vtkmodules.vtkRenderingCore import vtkActor

from RasterGridTool.Vtk.ui.process.ViewVtkActors import SimpleViewActors
from RasterGridTool.Vtk.ui.process.createTin import CreateTin


def listNumericField(table:Table):
    shema_tab=table.schema
    list_numeric_fields=[]
    for att in shema_tab:
        if att.type_string=='int' or att.type_string=='double' or att.type_string=='decimal':
            list_numeric_fields.append(att.name)
    return list_numeric_fields
class FormVtkTin(QWidget):
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        super(FormVtkTin, self).__init__(parent)
        self.__parentWin = parent
        self.__window3d=None
        self.load_ui('FormVtkTin.ui')
        self.window.cmb_tables.currentIndexChanged.connect(self.__change_tab)
        self.__initListTab()
        #self.window.toolBox.setItemEnabled(2,False)
        self.window.pb_cancel.clicked.connect(self.__cancel)
        self.window.pb_run.clicked.connect(self.__run)
        self.window.pb_saveAs.clicked.connect(self.__selectSaveStl)
        self.window.chb_view3D.toggled.connect(self.__checkEnable)
        self.window.chb_wireframe.toggled.connect(self.__checkwireframe)
        self.window.pb_color_background.clicked.connect(self.__select_background)
        self.window.pb_color_tin.clicked.connect(self.__select_tin_color)
        #self.window.resizeEvent = self.resizeEventOn
        self.__destinationFolder=None
        self.window.pb_color_background.setStyleSheet("background-color:rgb(0,0,0)")
        self.window.pb_color_tin.setStyleSheet("background-color:rgb(255,255,255)")
        self.__background_color=QColor(0,0,0)
        self.__color_tin =QColor(255,255,255)
    def load_ui(self, name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window = loader.load(ui_file, self.__parentWin)

        ui_file.close()
    def __initListTab(self):
        '''
        self.window.cmb_tables.addItem("Test1")
        self.window.cmb_tables.addItem("Test2")
        '''
        tables=axipy.da.data_manager.tables
        for i,tab in enumerate(tables):
            if not tab.is_spatial:
                continue
            list_numeric_field=listNumericField(tab)
            if len(list_numeric_field)==0:
                continue
            self.window.cmb_tables.addItem(tab.name)
            if i==0:
                self.__update_listFields(list_numeric_field)
        self.window.cmb_tables.setCurrentIndex(0)
    def __change_tab(self):
        if self.window.cmb_tables.count()==0:
            return
        name_table=self.window.cmb_tables.currentText()
        list_numeric_field=listNumericField(axipy.da.data_manager.find(name_table))
        self.__update_listFields(list_numeric_field)
    def __change_field(self):
        name_field=self.window.cmb_field.currentText()

    def __update_listFields(self,list_field):
        self.window.cmb_field.clear()
        for field in list_field:
            self.window.cmb_field.addItem(field)
    def __cancel(self):
        if self.__process is not None:
            self.__process_status = False
            self.cancelProcess.emit()
            self.__process.setCancel()
    def __update_progress(self, value):
        self.window.pg_progress.setValue(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def __selectSaveStl(self):
        if self.__destinationFolder is None:
            self.__destinationFolder = os.path.expanduser('~/Documents')

        #if self.__dlgSelectDestinationeMws.exec_():
        default_file=str(Path(self.__destinationFolder).joinpath(self.window.cmb_tables.currentText()+".stl"))
        filenames = QFileDialog.getSaveFileName(parent=self.window, caption="Сохранить Tin ",dir=default_file, filter = "Stl (*.stl)")
        if not(filenames[0]==''):

            file_wor=filenames[0]
            self.window.ln_edit_stl_file.setText(file_wor)
            self.__destinationFolder=str(Path(file_wor).parent)
            self.__enableRun()
    def __enableRun(self):

        if self.window.ln_edit_stl_file.text() == '' or not Path(self.window.ln_edit_stl_file.text()).parent.exists():
            self.window.pb_run.setEnabled(False)
            return
        self.window.pb_run.setEnabled(True)
    def __checkEnable(self):
        self.window.gr_box_color.setEnabled(self.window.chb_view3D.isChecked())
    def __run(self):
        self.window.pb_run.setEnabled(False)
        self.window.pb_cancel.setEnabled(True)
        self.__process = CreateTin("Создание Tin", self)
        name_source_tab=self.window.cmb_tables.currentText()
        name_field_z=self.window.cmb_field.currentText()
        self.__process.setParams({'source': name_source_tab, 'z_field': name_field_z,"stl":self.window.ln_edit_stl_file.text()})

        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.finished.connect(self.__end_process)
        self.__process.updateFormatProgressBar.connect(self.__setFormatprogressBar)
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        self.__process.run()
    def __end_process(self, value):
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
        ''' View Result '''
        if self.window.chb_view3D.isChecked():
            meshMapper = vtk.vtkPolyDataMapper()
            meshMapper.SetInputConnection(self.__process.getPolydateTin())
            actor = vtkActor()
            actor.SetMapper(meshMapper)
            if self.window.chb_wireframe.isChecked():
                actor.GetProperty().SetRepresentationToWireframe()
            rgbf=self.__color_tin.getRgbF()
            actor.GetProperty().SetColor(rgbf[0], rgbf[1], rgbf[2])
            #import axioma
            #self.__window3d = SimpleViewActors([actor], axioma.app.mainWindow)
            self.__window3d = SimpleViewActors([actor],axipy.app.mainwindow.qt_object())
            self.__window3d.closeWindows.connect(self.__closeViwer3d)
            self.__window3d.setWindowTitle("Axioma 3d")
            self.__window3d.setBackground(self.__background_color.getRgbF())
            #from PyQt5 import QtCore
            #self.__window3d.setWindowModality(QtCore.Qt.ApplicationModal)
            self.__window3d.show()
            self.__window3d.iren.Initialize()
        if value == 100:
            pass
        else:
            '''
            if os.path.exists(self.__path_out_raster):
                os.remove(self.__path_out_raster)
            Notifications.push('Объединение растров', 'Было преррвано', Notifications.Information)
            '''
            pass
        self.__process=None
    def __closeViwer3d(self):
        self.__window3d=None

    def __setFormatprogressBar(self, str_format):
        self.window.pg_progress.setFormat(str_format)
        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __select_background(self):
        dialog = QColorDialog(self.window)
        dialog.setCurrentColor(self.__background_color)
        dialog.exec_()
        select_color=dialog.selectedColor()
        isValidColor=select_color.isValid()
        if isValidColor:
            self.__background_color=select_color
            rgbColor=select_color.getRgb()
            str_rgb='rgb({0}, {1}, {2})'.format(rgbColor[0], rgbColor[1], rgbColor[2])
            self.window.pb_color_background.setStyleSheet("background-color: %s;" % str_rgb)
            if self.__window3d is not None:
                self.__window3d.setBackground(self.__background_color.getRgbF(),True)
    def __select_tin_color(self):
        dialog = QColorDialog(self.window)
        dialog.setCurrentColor(self.__color_tin)
        dialog.exec_()
        select_color = dialog.selectedColor()
        isValidColor = select_color.isValid()
        if isValidColor:
            self.__color_tin = select_color
            rgbColor = select_color.getRgb()
            str_rgb = 'rgb({0}, {1}, {2})'.format(rgbColor[0], rgbColor[1], rgbColor[2])

            self.window.pb_color_tin.setStyleSheet("background-color: %s;" % str_rgb)
            if self.__window3d is not None:
                self.__window3d.updateActorColor( select_color.getRgbF())
    def __checkwireframe(self):
        ''' Включение выключение отображение TIn как каркас'''
        if self.__window3d is not None:
            self.__window3d.updateActorWireframe(self.window.chb_wireframe.isChecked())
    def resizeEventOn(self, event):
        jkl=0
    @property
    def widget(self):
        return self.window
    def show(self):
        self.window.show()

