import os
from pathlib import Path
import time
import axipy
import vtk
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread
from PySide2.QtGui import QColor
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QFileDialog, QApplication, QColorDialog
from vtkmodules.vtkRenderingCore import vtkActor

from RasterGridTool.toolprocessing.ui.contour.formCreateContour import getListGrid
from .process.GridToTin import GridToTinProcess
from .process.ViewVtkActors import SimpleViewActors


class FormGridToTin(QWidget):
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        super(FormGridToTin, self).__init__(parent)
        self.__dlg_SelectFiles=None
        self.__parentWin = parent
        self.load_ui('FromGridToTin3d.ui')
        self.window.pb_color_background_source.setStyleSheet("background-color:rgb(255,0,0)")
        self.window.pb_color_background_decimate.setStyleSheet("background-color:rgb(0,255,0)")
        self.__color_tin_source=QColor(255,0,0)
        self.__color_tin_decimate = QColor(0,255,0)
        self.window.pb_color_background_source.clicked.connect(self.__select_source_color)
        self.window.pb_color_background_decimate.clicked.connect(self.__select_decimate_color)
        self.window.tb_select_save_as.clicked.connect(self.__select_save_file)
        self.window.cmb_tab_grid.currentIndexChanged.connect(self.__change_grid)
        self.window.gpb_decimate.toggled.connect(self.__isEnableDecimate)
        self.window.gpb_source_tin.toggled.connect(self.__isVisibleSource)
        self.window.grb_decimate_tin.toggled.connect(self.__isVisibleDecimate)
        self.window.pb_cancel.clicked.connect(self.__cancel)
        self.window.pb_run.clicked.connect(self.__run)
        self.updateGrids()
    def load_ui(self, name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window = loader.load(ui_file, self.__parentWin)

        ui_file.close()
    def updateGrids(self,clearPrev=True):
        self.__list_grid=getListGrid()
        if len(self.__list_grid)==0:
            return
        if clearPrev:
            self.window.cmb_tab_grid.clear()
        for grid in self.__list_grid:
            name=grid['raster'].name
            self.window.cmb_tab_grid.addItem(name)
        self.window.cmb_tab_grid.setCurrentIndex(0)
        self.__change_grid(0)
    def __change_grid(self,index):
        name_select_grid = self.window.cmb_tab_grid.currentText()
        tab_select = axipy.data_manager.find(name_select_grid)
        file_tab = tab_select.properties['tabFile']
        path_tab = Path(file_tab)
        path_out_stl = str(path_tab.parent.joinpath(path_tab.stem + ".stl"))
        path_out_axistl = str(path_tab.parent.joinpath(path_tab.stem + ".axistl"))
        self.window.ln_out_stl.setText(path_out_axistl)
        self.window.ln_out_stl.setCursorPosition(0)
        self.window.pb_run.setEnabled(True)
    def __select_save_file(self):
        ext_list = "Tin Axioma (*.axistl);;Tin (.stl)"
        name_select_grid=self.window.cmb_tab_grid.currentText()
        tab_select=axipy.data_manager.find(name_select_grid)
        file_tab=tab_select.properties['tabFile']
        path_tab=Path(file_tab)
        path_out_stl=str(path_tab.parent.joinpath(path_tab.stem+".stl"))
        path_out_axistl = str(path_tab.parent.joinpath(path_tab.stem + ".axistl"))
        filenames = QFileDialog.getSaveFileName(axipy.app.mainwindow.qt_object(), "Сохранить Tin ", path_out_axistl,ext_list)
        if not (filenames[0] == ''):
            self.window.ln_out_stl.setText(filenames[0])
            self.window.ln_out_stl.setCursorPosition(0)
            self.window.pb_run.setEnabled(True)
    def __run(self):
        params = {}
        params['file_out'] = self.window.ln_out_stl.text()
        params['tab_grid']=self.window.cmb_tab_grid.currentText()
        if self.window.gr_3dView.isEnabled():
            pass
        params['decimate']=None
        if self.window.gpb_decimate.isEnabled():
            #params['decimate']=1.0-float(self.window.sp_decimate.value())/100.0
            params['decimate'] = float(self.window.sp_decimate.value()) / 100.0
        self.window.pb_run.setEnabled(False)
        self.window.pb_cancel.setEnabled(True)
        self.__process = GridToTinProcess("Создание Grid to Tin", self)
        self.__process.setParams( params)

        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.finished.connect(self.__end_process)
        self.__process.updateFormatProgressBar.connect(self.__setFormatprogressBar)
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        self.__process.run()
    def __cancel(self):
        if self.__process is not None:
            self.__process_status = False
            self.cancelProcess.emit()
            self.__process.setCancel()
    def __end_process(self, value):
        if self.window.gr_3dView.isChecked() and (not self.__process.isCancel):
            meshMapper = vtk.vtkPolyDataMapper()
            polyDataSource,colors_tin=self.__process.sourceTin
            if colors_tin is not None:
                polyDataSource.GetPointData().SetScalars(colors_tin)
            meshMapper.AddInputDataObject(polyDataSource)
            actor = vtkActor()
            actor.SetMapper(meshMapper)
            rgbf_source=self.__color_tin_source.getRgbF()
            actor.GetProperty().SetColor(rgbf_source[0], rgbf_source[1], rgbf_source[2])
            actor_decimate =None
            if self.__process.isDecimate:
                polyDataDecimate,colors_decimate=self.__process.resultPolyData
                meshMapperDecimate = vtk.vtkPolyDataMapper()
                meshMapperDecimate.AddInputDataObject(polyDataDecimate)
                actor_decimate=vtkActor()
                actor_decimate.SetMapper(meshMapperDecimate)
                rgbf_decimate = self.__color_tin_decimate.getRgbF()
                actor_decimate.GetProperty().SetColor(rgbf_decimate[0], rgbf_decimate[1], rgbf_decimate[2])
            import axioma
            self.__window3d = SimpleViewActors([actor,actor_decimate], axioma.app.mainWindow)
            self.__window3d.closeWindows.connect(self.__closeViwer3d)
            self.__window3d.setWindowTitle("Axioma 3d")
            #self.__window3d.setBackground(self.__background_color.getRgbF())
            # from PyQt5 import QtCore
            # self.__window3d.setWindowModality(QtCore.Qt.ApplicationModal)
            self.__window3d.show()
            self.__window3d.iren.Initialize()
        else:
            self.window.pb_cancel.setEnabled(False)
            self.window.pb_run.setEnabled(True)
    def __closeViwer3d(self):
        self.__window3d=None
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
    def __update_progress(self, value):
        self.window.process_progress.setValue(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def __select_source_color(self):
        new_color=self.__select_color(self.__color_tin_source)
        if new_color is None:
            return
        self.__color_tin_source=new_color
        str_rgb = 'rgb({0}, {1}, {2})'.format(self.__color_tin_source[0], self.__color_tin_source[1], self.__color_tin_source[2])
        self.window.pb_color_background_source.setStyleSheet("background-color: %s;" % str_rgb)
        if self.__window3d is not None:
            self.__window3d.updateActorColor(self.__color_tin_source.getRgbF(),0)
    def __select_decimate_color(self):
        new_color = self.__select_color(self.__color_tin_decimate)
        if new_color is None:
            return
        self.__color_tin_decimate = new_color
        str_rgb = 'rgb({0}, {1}, {2})'.format(self.__color_tin_decimate[0], self.__color_tin_decimate[1],
                                              self.__color_tin_decimate[2])
        self.window.pb_color_background_decimate.setStyleSheet("background-color: %s;" % str_rgb)
        if self.__window3d is not None:
            self.__window3d.updateActorColor(self.__color_tin_decimate.getRgbF(),1)
    def __setFormatprogressBar(self, str_format):
        self.window.process_progress.setFormat(str_format)
        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __isVisibleSource(self,state):
        if self.__window3d is not None:
            self.__window3d.setVisibility(state,0)
    def __isVisibleDecimate(self,state):
        if self.__window3d is not None:
            self.__window3d.setVisibility(state,1)
    def __isEnableDecimate(self,state):

        self.window.gpb_decimate.setEnabled(state)
    def __select_color(self,color):
        dialog = QColorDialog(self.window)
        dialog.setCurrentColor(color)
        dialog.exec_()
        select_color = dialog.selectedColor()
        isValidColor = select_color.isValid()
        if isValidColor:
            return select_color
        return None
    @property
    def widget(self):
        return self.window

    def show(self):
        self.window.show()