import os
import time
from pathlib import Path

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread
from PySide2.QtGui import QDoubleValidator, QValidator
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QFileDialog, QApplication
from axipy import Notifications, CoordSystem, Layer, Map, view_manager, selection_manager, Polygon

from RasterGridTool.Vtk.ui.process.VtkTin import VtkTin
from RasterGridTool.Vtk.ui.process.createGrid import TinToGridProcess
from RasterGridTool.Vtk.ui.process.helper import AxiStl
def coordToString(value,isLatLon):
    if isLatLon:
        str_value="{:.7f}".format(value)
        return str_value
    str_value = "{:.2f}".format(value)
    return str_value
class ValidatorStep(QDoubleValidator):
    __xmin=__xmax=__ymin=__ymax=None
    def validate(self, value, pos):
        text = value.strip().title()
        for null in ('None', 'Null', 'Nothing'):
            if text == null:
                return QValidator.Acceptable, text, pos
            if null.startswith(text):
                return QValidator.Intermediate, text, pos
        if text.isnumeric():
            if float(text)<=0.0:
                return QValidator.Intermediate, text, pos
            if self.__xmin is not None and self.__ymin is not None and self.__xmax is not None and self.__ymax is not None:
                nx=(self.__xmax-self.__xmin)/float(text)+1
                if nx<=1:
                    return QValidator.Intermediate, text, pos
                ny = (self.__ymax - self.__ymin) / float(text) + 1
                if ny<=1:
                    return QValidator.Intermediate, text, pos
        return super(ValidatorStep, self).validate(value, pos)
    def setBound(self,name,value):
        if name=='xmin':
            self.__xmin=value
            return
        if name == 'ymin':
            self.__ymin=value
            return
        if name=='xmax':
            self.__xmax=value
            return
        if name == 'ymax':
            self.__ymax=value
            return

class FormTinToGrid(QWidget):
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        super(FormTinToGrid, self).__init__(parent)
        self.__dlg_SelectFiles=None
        self.__parentWin = parent
        self.load_ui('FormTinToGrid.ui')

        self.window.ln_step.setValidator(QDoubleValidator())
        self.window.tb_select_tin.clicked.connect(self.__selectTin)
        self.window.tb_select_out.clicked.connect(self.__selectSaveGrid)
        self.window.ch_bound_sel.toggled.connect(self.__checkSelectBound)
        self.window.pb_run.clicked.connect(self.__run)
        self.window.ln_step.textChanged[str].connect(self.__changeTextStep)
        axipy.gui.selection_manager.changed.connect(self.__changeSlection)
    def load_ui(self, name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window = loader.load(ui_file, self.__parentWin)

        ui_file.close()
    def __selectSaveGrid(self):
        ext_list = "Gpkg (*.gpkg);;Tif (.tif)"
        path_source=self.window.ln_out_grid.text()
        fileNameGrid=QFileDialog.getSaveFileName(axipy.app.mainwindow.qt_object(),"Сохранить файл поверхность",path_source,ext_list)
        jkl=0
    def __selectTin(self):
        ext_list = "Tin Axioma (*.axistl);;Tin (.stl)"
        home_directory = str(Path.home())
        if self.__dlg_SelectFiles is None:
            self.__dlg_SelectFiles = QFileDialog(self, 'Выбрать Tin', home_directory, ext_list)
            name_select = self.__dlg_SelectFiles.getOpenFileNames(axipy.app.mainwindow.qt_object(), 'Выбрать Tin',
                                                                  home_directory, ext_list)
        else:
            # last_folder=self.__dlg_SelectFiles.directory()
            # filter=self.__dlg_SelectFiles.filter()
            name_select = self.__dlg_SelectFiles.getOpenFileNames(dir=self.__last_source_folder, filter=ext_list)

        if len(name_select[0]) == 0:
            return
        self.window.ln_edit_source_tin.setText(name_select[0][0])
        self.window.ln_edit_source_tin.setCursorPosition(0)
        if name_select[1]=='Tin Axioma (*.axistl)':
            ''' Выбрали axi_stl описатель stl'''
            self.__axiStl=AxiStl()
            self.__axiStl.read(name_select[0][0])
            self.window.ln_min_x.setText(str(self.__axiStl.Xmin))
            self.window.ln_min_y.setText(str(self.__axiStl.Ymin))
            self.window.ln_max_x.setText(str(self.__axiStl.Xmax))
            self.window.ln_max_y.setText(str(self.__axiStl.Ymax))
            self.__step,self.__nx,self.__ny=self.__calcDefStep(self.__axiStl.Xmin,self.__axiStl.Xmax,self.__axiStl.Ymin,self.__axiStl.Ymax)
            self.window.ln_step.setText(str(self.__step))
            self.window.ln_nx.setText(str(self.__nx))
            self.window.ln_ny.setText(str(self.__ny))
            self.__curent_cs=CoordSystem.from_prj(self.__axiStl.CsMiStr)
            self.window.ln_coordSys.setText(self.__curent_cs.name)
            self.window.ln_coordSys.setCursorPosition(0)
            self.window.gb_out_file.setEnabled(True)
            path_source=Path(name_select[0][0])
            path_out=path_source.parent.joinpath(path_source.stem+"_grid.gpkg")
            self.window.ln_out_grid.setText(str(path_out))
            self.window.ln_out_grid.setCursorPosition(0)
            self.window.pb_run.setEnabled(True)
            self.__changeSlection()
            '''
            self.__stepValidator.setBound('xmin',axiStl.Xmin)
            self.__stepValidator.setBound('xmax', axiStl.Xmax)
            self.__stepValidator.setBound('ymin', axiStl.Ymin)
            self.__stepValidator.setBound('ymax', axiStl.Ymax)
            '''
        self.__last_source_folder=str(Path(name_select[0][0]).parent)
    def __changeTextStep(self,value:str):
        if not value.isnumeric():
            return
        if float(value)<=0.0:
            Notifications.push('Предупреждение', 'Шаг не может быть <=0', Notifications.Warning)
            return
        xmin=float(self.window.ln_min_x.text())
        xmax=float(self.window.ln_max_x.text())
        ymin=float(self.window.ln_min_y.text())
        ymax=float(self.window.ln_max_y.text())
        nx=int((xmax-xmin)/float(value))+1
        ny = int((ymax - ymin) / float(value)) + 1
        if nx<=2 or ny <=2:
            Notifications.push('Предупреждение', 'Шаг очень большой', Notifications.Warning)
            return
        self.window.ln_nx.setText(str(nx))
        self.window.ln_ny.setText(str(ny))
    def __calcDefStep(self,xmin,xmax,ymin,ymax):
        def__max_col=100
        delta_x=xmax-xmin
        delta_y=ymax-ymin
        if delta_x>delta_y:
            step=delta_x/def__max_col
            nx=def__max_col
            ny=delta_y/step+1
        else:
            step = delta_y / def__max_col
            ny=def__max_col
            nx=delta_x/step+1
        return step,int(nx),int(ny)
    def __run(self):
        params={}
        params['stl']=self.__axiStl.StlFile
        params['cs']=self.__curent_cs
        params['xmin']=float(self.window.ln_min_x.text())
        params['xmax'] = float(self.window.ln_max_x.text())
        params['ymin'] = float(self.window.ln_min_y.text())
        params['ymax'] = float(self.window.ln_max_y.text())
        params['nx']=int(self.window.ln_nx.text())
        params['ny'] = int(self.window.ln_ny.text())
        params['step']=float(self.window.ln_step.text())
        params['ext']=3
        params['out_grid']=self.window.ln_out_grid.text()
        self.window.pb_run.setEnabled(False)
        self.window.pb_cancel.setEnabled(True)
        self.__process = TinToGridProcess("Создание GridFromTin", self)
        self.__process.setParams(params)
        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.finished.connect(self.__end_process)
        self.__process.updateFormatProgressBar.connect(self.__setFormatprogressBar)
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        self.__process.run()
    def __end_process(self, value):
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
        if not self.__process.isCancel:
            name_file_grd=self.__process.nameOutFileGrd
            name=str(Path(name_file_grd).stem)
            tab_this_name=axipy.da.data_manager.find(name)
            if tab_this_name is not None:
                tab_this_name.close()
            tab_grd=axipy.da.provider_manager.gdal.open(name_file_grd)
            grd_layer = Layer.create(tab_grd)
            map_ = Map([grd_layer])
            mapview = view_manager.create_mapview(map_)
        self.__process = None

    def __cancel(self):
        if self.__process is not None:
            self.__process_status = False
            self.cancelProcess.emit()
            self.__process.setCancel()

    def __update_progress(self, value):
        self.window.process_progress_bar.setValue(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def __setFormatprogressBar(self, str_format):
        self.window.process_progress_bar.setFormat(str_format)
        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __changeSlection(self):
        if axipy.gui.selection_manager.count==0:
            self.window.ch_bound_sel.setEnabled(False)
            return
        self.window.ch_bound_sel.setEnabled(True)
        bound_sel=None
        tab_sel=selection_manager.get_as_table()
        layer_sel = Layer.create(tab_sel)
        bound_sel = layer_sel.get_bounds()
        layer_sel=None
        bound_poly=Polygon.from_rect(bound_sel,tab_sel.coordsystem)
        self.__select_rect_bound=bound_poly
        self.__select_bound_cs=tab_sel.coordsystem


        #str_xmin=coordToString(rect_bound.xmin,self.__curent_cs.lat_lon)

    def __checkSelectBound(self):
        if not self.window.ch_bound_sel.isChecked():
            return

        if not ( self.__select_bound_cs==self.__curent_cs):
            self.__select_rect_bound=self.__select_rect_bound.reproject(self.__curent_cs).bounds
        str_min_x=coordToString(self.__select_rect_bound.xmin,self.__curent_cs.lat_lon)
        str_min_y = coordToString(self.__select_rect_bound.ymin, self.__curent_cs.lat_lon)
        str_max_x = coordToString(self.__select_rect_bound.xmax, self.__curent_cs.lat_lon)
        str_max_y = coordToString(self.__select_rect_bound.ymax, self.__curent_cs.lat_lon)
        self.window.ln_min_x.setText(str_min_x)
        self.window.ln_min_y.setText(str_min_y)
        self.window.ln_max_x.setText(str_max_x)
        self.window.ln_max_y.setText(str_max_y)
        self.__changeTextStep(self.window.ln_step.text())
    @property
    def widget(self):
        return self.window

    def show(self):
        self.window.show()