from pathlib import Path

import axipy
import vtk
from PySide2.QtCore import QObject, Signal
import time

from axipy import Notifications
from osgeo import gdal
from vtkmodules.vtkCommonDataModel import vtkPolyData
from vtkmodules.vtkIOGeometry import vtkSTLWriter

from RasterGridTool.Vtk.ui.process.gridToTin.rowsToTriangle import gridToTin
from RasterGridTool.Vtk.ui.process.helper import _update_alg, _writer_alg
from RasterGridTool.Vtk.ui.process.memGrid import MemGrid
from RasterGridTool.toolprocessing.grid_io.VrtGridColorNewPalette import RasterGrid


class GridToTinProcess(QObject):
    updateProcess = Signal(float)
    finished = Signal(int)

    updateFormatProgressBar = Signal(str)
    __old_val_progress=0
    def __init__(self, name_prosess, parent_class):
        super(GridToTinProcess, self).__init__()
        self.__parent = parent_class
        self.__name_process = name_prosess
    def setParams(self,params):
        self.__params=params
    def __process_callback(self, complete):
        # print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        cur_progress_int = int(complete * 100 + 0.5)
        if cur_progress_int > self.__old_val_progress:
            self.updateProcess.emit(complete * 100)
            self.__old_val_progress = cur_progress_int
        else:
            if cur_progress_int < self.__old_val_progress:
                self.__old_val_progress = cur_progress_int
                self.updateProcess.emit(complete * 100)
        # self.__parent.window.pr_bar.setValue(complete*100)
        # print(complete*100)
        return 1

        self.__cancelProcess=True

    def setCancel(self):
        self.__cancelProcess = True
    @property
    def isCancel(self):
        return self.__cancelProcess
    @property
    def isDecimate(self):
        if self.__params['decimate'] is None:
            return False
        return True
    @property
    def resultPolyData(self):
        return self.__polyData,self.__vtkColors
    @property
    def sourceTin(self):
        if self.isDecimate:
            return self.__polyData_source_tin,self.__vtkColors
        return self.__polyData, self.__vtkColors

    def run(self):
        self.__cancelProcess = False
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        polyDecimate=None
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess = False
        tab_grid_source=axipy.data_manager.find(self.__params['tab_grid'])
        source_grid=RasterGrid(tab_grid_source)
        colorTable=None
        if source_grid.existPalette:
            colorTable=source_grid.colorTable
        try:
            ds = gdal.Open(source_grid.pathRastre)

            memGrid = MemGrid()
            memGrid.Load(ds)
            self.updateFormatProgressBar.emit("Создание Tin .. %p%")
            polyData,self.__vtkColors=gridToTin(memGrid, colorTable,self.__process_callback)
            if polyData is not None:
                info_tin=" точек:"+str(polyData.GetNumberOfPoints())+" треугольников:"+str(polyData.GetNumberOfPolys())
                Notifications.push('Поверхность в Tin', info_tin, Notifications.Information)
            if polyData is not None and self.__params['decimate'] is not None:
                ''' Требуется децимация '''
                polyDecimate=vtk.vtkDecimatePro()
                polyDecimate.SetInputData(polyData)
                polyDecimate.SetTargetReduction(self.__params['decimate'])
                self.updateFormatProgressBar.emit("Упрощение Tin .. %p%")
                _update_alg(polyDecimate, self.__process_callback)


        except Exception as ex:
            self.__cancelProcess=True
        finally:
            ds=None
        if self.__cancelProcess:
            self.updateProcess.emit(100)
            self.finished.emit(100)
            return
        self.__polyData = vtkPolyData()
        self.__polyData_source_tin=vtkPolyData()
        if self.__params['decimate'] is not None:
            self.__polyData.ShallowCopy(polyDecimate.GetOutput())
            self.__polyData_source_tin.ShallowCopy(polyData)
            info_tin = " точек:" + str(self.__polyData.GetNumberOfPoints()) + " треугольников:" + str(
                self.__polyData.GetNumberOfPolys())
            Notifications.push('Упрощение Tin', info_tin, Notifications.Information)
        else:
            self.__polyData.ShallowCopy(polyData)
            self.__polyData_source_tin=None
        ''' сохранение в STL'''
        path_out_file=Path(self.__params['file_out'])
        path_stl_file=None
        if str(path_out_file.suffix).lower()=='.stl':
            path_stl_file=self.__params['file_out']
        else:
            path_stl_file=str(path_out_file.parent.joinpath(path_out_file.stem+".stl"))
        stlWriter = vtkSTLWriter()
        stlWriter.SetFileName(path_stl_file)
        stlWriter.AddInputDataObject(self.__polyData)
        stlWriter.SetFileTypeToBinary()
        self.updateFormatProgressBar.emit("Сохраение Tin в файл .. %p%")
        _writer_alg(stlWriter, self.__process_callback)
        self.updateProcess.emit(100)
        self.finished.emit(100)
        return
    def __isCancel(self):
        self.__cancelProcess=True