import time
from PySide2.QtCore import QObject, Signal

from RasterGridTool.Vtk.ui.process.CalcRowGrid import RowCalcGrid
from RasterGridTool.Vtk.ui.process.VtkTin import VtkTin
from RasterGridTool.Vtk.ui.process.helper import newZBound
from RasterGridTool.Vtk.ui.process.memGrid import MemGrid


class TinToGridProcess(QObject):
    updateProcess = Signal(float)
    finished = Signal(int)

    updateFormatProgressBar = Signal(str)
    __old_val_progress = 0
    def __init__(self, name_prosess, parent_class):
        super(TinToGridProcess, self).__init__()
        self.__parent = parent_class
        self.__name_process=name_prosess
    def __process_callback(self, complete):
        # print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        cur_progress_int = int(complete * 100 + 0.5)
        if cur_progress_int > self.__old_val_progress:
            self.updateProcess.emit(complete * 100)
            self.__old_val_progress = cur_progress_int
        else:
            if cur_progress_int < self.__old_val_progress:
                self.__old_val_progress = cur_progress_int
                self.updateProcess.emit(complete * 100)
        # self.__parent.window.pr_bar.setValue(complete*100)
        # print(complete*100)
        return 1
    def setParams(self,params:dict):
        self.__params=params
    def __isCancel(self):
        self.__cancelProcess=True

    def setCancel(self):
        self.__cancelProcess = True
    @property
    def isCancel(self):
        return self.__cancelProcess
    @property
    def nameOutFileGrd(self):
        return self.__params['out_grid']
    def __buildGrid(self):

        self.__cls_grid=MemGrid(self.__params['xmin'],self.__params['ymin'],self.__params['xmax'],self.__params['ymax'],self.__params['step'],3)
        self.__cls_grid.Init()
        bound_tin=self.__cls_vtkTin.bound
        newZmin,newZmax=newZBound(bound_tin[4],bound_tin[5])
        calcRow = RowCalcGrid(self.__cls_grid.params, newZmin,newZmax,self.__cls_vtkTin)
        ny=self.__cls_grid.params['ny']
        for j in range(ny):
            row_data = calcRow.calcRow(j)
            if self.__process_callback(j/ny)==0:
                return False

            self.__cls_grid.setRow(j, row_data)
        return True
    def run(self):
        self.__cancelProcess=False
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess = False
        self.updateFormatProgressBar.emit("Чтение stl .. %p%")
        self.__cls_vtkTin=VtkTin()
        path_source=self.__params['stl']
        self.__cls_vtkTin.load(path_source,self.__process_callback)
        self.updateFormatProgressBar.emit("Формирование поверхности .. %p%")
        isOkBuild=self.__buildGrid()
        if isOkBuild:
            self.updateFormatProgressBar.emit("Запись в файл .. %p%")
            cs_wkt=self.__params['cs'].wkt
            self.__cls_grid.WriteToFile(self.__params['out_grid'], cs_wkt)
        self.updateProcess.emit(100)
        self.finished.emit(100)
        return
