import time
from pathlib import Path

import axipy
import numpy as np
import vtk
from PySide2 import QtCore
from PySide2.QtCore import QObject, Signal
from axipy import Pnt, Geometry, Table, Notifications
from osgeo import ogr
from vtkmodules.vtkIOGeometry import vtkSTLWriter

from RasterGridTool.Vtk.ui.process.helper import _update_alg, writeAxiStl


def getPointFromPoint(geo):
    return [Pnt(geo.x,geo.y)]
def getPointFromPolygon(geo):
    points=[]
    points.extend(geo.points)
    if geo.type==axipy.GeometryType.LineString :
        return points
    del points[-1]
    for geo_hole in geo.holes:
        points.extend(geo_hole)
        del points[-1]
    return points
def getPointFromLine(geo):
    return [geo.begin,geo.end]
def getPointsCollection(geo:Geometry):
    points=[]
    for g in geo:
        pts_g=getPoints(g)
        if pts_g is not None:
            points.extend(pts_g)
    if len(points)==0:
        return None
    return points
def getPoints(geo:Geometry):
    points_source=None
    if geo.type == axipy.GeometryType.Point:
        return getPointFromPoint(geo)
    if geo.type == axipy.GeometryType.MultiPoint:
        points_source = getPointsCollection(geo)
    elif geo.type == axipy.GeometryType.Polygon:
        points_source = getPointFromPolygon(geo)
    elif geo.type == axipy.GeometryType.Line:
        points_source = getPointFromLine(geo)
    elif geo.type == axipy.GeometryType.LineString:
        points_source = getPointFromPolygon(geo)
    elif geo.type == axipy.GeometryType.LineString:
        points_source = getPointFromPolygon(geo)
    elif geo.type == axipy.GeometryType.MultiLineString:
        points_source = getPointsCollection(geo)
    elif geo.type == axipy.GeometryType.MultiPolygon:
        points_source = getPointsCollection(geo)
    elif geo.type == axipy.GeometryType.GeometryCollection:
        points_source = getPointsCollection(geo)
    return points_source


class CreateTin(QObject):
    updateProcess = Signal(float)
    finished = Signal(int)

    updateFormatProgressBar = Signal(str)
    __old_val_progress=0
    def __init__(self, name_prosess, parent_class):
        super(CreateTin, self).__init__()
        self.__parent = parent_class
        self.__name_process=name_prosess
    def __process_callback(self, complete):
        # print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        cur_progress_int=int(complete*100+0.5)
        if cur_progress_int>self.__old_val_progress:
            self.updateProcess.emit(complete * 100)
            self.__old_val_progress=cur_progress_int
        else:
            if cur_progress_int<self.__old_val_progress:
                self.__old_val_progress = cur_progress_int
                self.updateProcess.emit(complete * 100)
        #time.sleep(0.001)
        # self.__parent.window.pr_bar.setValue(complete*100)
        # print(complete*100)
        return 1
    def setParams(self,params:dict):
        self.__params=params
    def __isCancel(self):
        self.__cancelProcess=True

    def setCancel(self):
        self.__cancelProcess = True
    def run(self):
        self.__cancelProcess=False
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess=False
        name_table=self.__params['source']
        tab_source=axipy.data_manager.find(name_table)
        cs_table=tab_source.coordsystem
        field_z_name=self.__params['z_field']
        self.updateFormatProgressBar.emit("Чтение данных .. %p%")
        print("start read points")
        ts_row = time.time()
        points=self.__getVtkPoints(tab_source,field_z_name,self.__process_callback)
        #points = self.__getVtkPoints(tab_source, field_z_name, None)
        if points is None:
            pass
        if points.GetNumberOfPoints()<3:
            Notifications.push('Ошибка создания Tin', 'Число исходных точек меньше 3', Notifications.Critical)
            self.finished.emit(100)
            return
        te_row = time.time()
        delta_time = te_row - ts_row
        print(delta_time)
        print("end read points")
        aPolyData = vtk.vtkPolyData()
        aPolyData.SetPoints(points)
        boundary = vtk.vtkPolyData()
        boundary.SetPoints(aPolyData.GetPoints())
       # st = datetime.now()
        self.updateFormatProgressBar.emit("Создание Tin .. %p%")
        self.__delaunay = vtk.vtkDelaunay2D()
        self.__delaunay.SetInputData(aPolyData)
        _update_alg(self.__delaunay,self.__process_callback)
        if self.__cancelProcess:
            self.finished.emit(0)
            return
        self.updateFormatProgressBar.emit("Сохраение Tin в файл .. %p%")
        path_out=self.__params["stl"]
        ''' Сохранение Tin в Stl'''
        stlWriter = vtkSTLWriter()
        stlWriter.SetFileName(path_out)
        stlWriter.SetInputConnection(self.__delaunay.GetOutputPort())
        stlWriter.Write()

        cs_mi=cs_table.prj
        bounds_out=self.__delaunay.GetOutputDataObject(0).GetBounds()
        writeAxiStl(path_out,cs_mi,bounds_out)
        self.updateProcess.emit( 100)
        self.finished.emit(100)
        return
    def getPolydateTin(self):
        return self.__delaunay.GetOutputPort()
    def __getVtkPoints(self,table_source: Table, name_fild_z,call_back=None) -> vtk.vtkPoints:
        count_feature = table_source.count()
        points = vtk.vtkPoints()
        for i, ft in enumerate(table_source.items()):
            if call_back is not None:
                is_cancel = call_back(i /  count_feature)
                if is_cancel == 0:
                    # logging.debug(("send send cancel"))
                    break
            geo = ft.geometry
            if geo is None:
                continue
            value = ft[name_fild_z]
            points_source = None
            points_source = getPoints(geo)
            if points_source is None:
                continue
            for pnt in points_source:
                points.InsertNextPoint(pnt.x, pnt.y, value)
        if call_back is not None:
            call_back(100)
        return points
