from pathlib import Path

import numpy as np
from numpy import float32
from osgeo import gdal, osr


class MemGrid:
    def __init__(self,xmin=None,ymin=None,xmax=None,ymax=None,stepx=None,offsetXY=0,stepy=None):
        if xmin is None:
            return
        self.__xmin=xmin
        self.__ymin=ymin
        self.__xmax=xmax
        self.__ymax=ymax
        self.__stepX=stepx
        if stepy is None:
            self.__stepY=stepx
        else:
            self.__stepY=stepy
        if offsetXY>0:
            self.__xmin=xmin-offsetXY*self.__stepX
            self.__ymin=self.__ymin-offsetXY*self.__stepY
        self.__nx=int((self.__xmax-self.__xmin)/self.__stepX)+1
        self.__ny = int((self.__ymax - self.__ymin) / self.__stepY) + 1
    def Init(self,noDataValue=65535):
        self.__noDataValue=noDataValue
        try:
            self.__grid=np.full((self.__ny,self.__nx ), self.__noDataValue,dtype=np.float32)
        except:
            return False
        return True
    def Load(self,raster_ds,band=1):
        self.__raster_geo_transform=raster_ds.GetGeoTransform()
        self.__xmin=self.__raster_geo_transform[0]
        self.__ymax=self.__raster_geo_transform[3]
        self.__stepX=self.__raster_geo_transform[1]
        self.__stepY = self.__raster_geo_transform[5]
        self.__grid=np.array(raster_ds.GetRasterBand(band).ReadAsArray())
        size_array= self.__grid.shape
        statistic_raster=raster_ds.GetRasterBand(band).GetStatistics(True, True)
        self.__zmin=statistic_raster[0]
        self.__zmax=statistic_raster[1]
        self.__ny=size_array[0]
        self.__nx=size_array[1]
        self.__noDataValue=raster_ds.GetRasterBand(band).GetNoDataValue()
        return True
    def FromArray(self,xmin,ymax,step,array,noData):
        self.__xmin=xmin
        self.__ymax=ymax
        self.__stepX=step
        self.__stepY=step
        self.__noDataValue=noData
        self.__grid=array
        size_array = self.__grid.shape
        self.__ny = size_array[0]
        self.__nx = size_array[1]
        self.__raster_geo_transform=(self.__xmin,self.__stepX,0,self.__ymax,-self.__stepY,0)
        return True
    def getGeoXY(self,index_x,index_y):
        X_geo = self.__raster_geo_transform[0] + index_x * self.__raster_geo_transform[1] + index_y * self.__raster_geo_transform[2]
        Y_geo=self.__raster_geo_transform[3] + index_x * self.__raster_geo_transform[4] + index_y * self.__raster_geo_transform[5]
        return X_geo,Y_geo
    def Z(self,index_x,index_y):
        return self.__grid[index_y,index_x]
    @property
    def Zmin(self):
        return self.__zmin
    @property
    def Zmax(self):
        return self.__zmax
    @property
    def Nx(self):
        return self.__nx
    @property
    def Ny(self):
        return self.__ny
    def getRow(self,i):
        return self.__grid[i,:]
    @property
    def NoData(self):
        return self.__noDataValue
    @property
    def params(self):
        return {'xmin':self.__xmin,'xmax':self.__xmax,'ymin':self.__ymin,'ymax':self.__ymax,"stepx":self.__stepX,'stepy':self.__stepY,'nx':self.__nx,'ny':self.__ny,'nodata':self.__noDataValue}
    def setRow(self,id_row,row_value):
        ''' Занесение строки в матрицу'''
        if id_row>=self.__ny:
            return False
        self.__grid[self.__ny-id_row-1]=row_value
        return True
    def WriteToFile(self,path_out,cs_wkt):
        ext_out_file=Path(path_out).suffix
        name_driver = 'GTiff'
        if ext_out_file.lower()=='.tif':
            name_driver='GTiff'
        if ext_out_file.lower()=='.gpkg':
            name_driver='GPKG'
        driver = gdal.GetDriverByName(name_driver)
        outRaster = driver.Create(path_out, self.__nx, self.__ny, 1, gdal.GDT_Float32)
        if outRaster is None:
            return
        z_min=self.__grid.min()
        z_max= self.__grid.max()
        outRaster.SetGeoTransform((self.__xmin, self.__stepX, 0, self.__ymax, 0, -self.__stepY))
        outband = outRaster.GetRasterBand(1)
        outband.SetNoDataValue(self.__noDataValue)
        outband.WriteArray(self.__grid)
        outRasterSRS = osr.SpatialReference()
        outRasterSRS.ImportFromWkt(cs_wkt)
        outRaster.SetProjection(outRasterSRS.ExportToWkt())
        outband.FlushCache()
        outband=None
        outRaster=None


