# read file CLR
import os
import re

from PySide2.QtGui import QColor
import numpy as np
from PySide2.QtGui import QPixmap, QImage
from osgeo import gdalconst

from RasterGridTool.toolprocessing.grid_io.colour import ColorRgb


def valueToString(value,type_gdal):
    if type_gdal==gdalconst.GDT_Byte or type_gdal==gdalconst.GDT_Int16 or type_gdal==gdalconst.GDT_UInt16 or type_gdal==gdalconst.GDT_Int32 or type_gdal==gdalconst.GDT_UInt32:

        return '%i'%(value)
    return '%f'%(value)

def parserLineReg(line_inp):
    list_items=re.split(' |, |\*|\t',line_inp)
    list_out=[]
    for ln in list_items:
        tmp=ln.strip()
        if len(tmp)>0:
            list_out.append(tmp)
    return list_out
def parserLine(line_inp):

    list_items=line_inp.split(' ')
    list_out=[]
    for ln in list_items:
        tmp=ln.strip()
        if len(tmp)>0:
            list_out.append(tmp)
    return list_out
def factoryItemColor(value,color):
    return ItemColorMap(value,color.red(),color.green(),color.blue())
class ItemColorMap:
    __val=None
    __R=None
    __G=None
    __B=None
    def __init__(self,value,r,g,b):
        if isinstance(value,str):
            self.__val=float(value)
        else:
            self.__val=value
        self.__color=[r,g,b]

        self.__R=r
        self.__G=g
        self.__B=b
    @property
    def getR(self):
        return self.__R
    @property
    def getG(self):
        return self.__G
    @property
    def getB(self):
        return self.__B
    @property
    def R(self):
        return self.__R
    @property
    def G(self):
        return self.__G
    @property
    def B(self):
        return self.__B
    @property
    def value(self):
        return self.__val

    def ColorIndex(self,index):
        return self.__color[index]
    def setValue(self,new_value):
        if isinstance(new_value,str):
            self.__val=float(new_value)
        else:
            self.__val=new_value
    def setColor(self,color):
        self.__R=color.red()
        self.__B=color.blue()
        self.__G=color.green()
    def setValue(self,value):
        self.__val=value
    def colorValueByName(self,name_color:str):
        if name_color.lower()=='red':
            return self.__R
        if name_color.lower()=='blue':
            return self.__B
        if name_color.lower()=='green':
            return self.__G
        return None
    @property
    def ColorToString(self):
        out_str=str(round(self.value,2))+"   "+str(self.R)+"   "+str(self.G)+"   "+str(self.B)
        return out_str
    def clone(self):
        return ItemColorMap(self.value,self.__R,self.__G,self.__B)
def createDefColorMap(array_line=None):
    if array_line is None:
        list_def_color=[]
        list_def_color.append('ColorMap 1 1 ')
        list_def_color.append('0   200 215 133')
        list_def_color.append('5   171 217 177')
        list_def_color.append('10  124 196 120')
        list_def_color.append('15  117 193 120')
        list_def_color.append('20  175 204 166')
        list_def_color.append('25  219 208  78')
        list_def_color.append('30  241 207  14 ')
        list_def_color.append('35  242 167   0')
        list_def_color.append('40  192 159  13')
        list_def_color.append('45  211 192 112')
        list_def_color.append('50  240 219 161')
        list_def_color.append('55  252 236 192')
        list_def_color.append('60  248 250 234')
        list_def_color.append('65  229 254 250')
        list_def_color.append('70  219 255 253')
        list_def_color.append('75  214 251 252 ')
        list_def_color.append('80  183 244 247')
        list_def_color.append('85  115 224 241')
        list_def_color.append('90   29 188 239')
        list_def_color.append('95    0 137 245')
        list_def_color.append('100   0  80 250')
        return ColorMap(list_def_color)
    return ColorMap(array_line)
def createGrayColor():
    list_def_color=[]
    list_def_color.append('ColorMap 1 1 ')
    list_def_color.append('0   255 255 255')
    list_def_color.append('100 0 0 0')
    return ColorMap(list_def_color)
class ColorMap:
    __version=1
    __interpMethod=1
    __source_path=None
    __val_min=None
    __val_max=None
    def __init__(self,path_file=None):
        if path_file is None:
            return
        self.__source_path=path_file
        if isinstance(path_file,list):
            self.__linesToColorMap(path_file)
            return
        if path_file:
            self.__readFileCrl(path_file)

    def __readFileCrl(self,path_file):
        f_clr=open(path_file,'r')
        lines=f_clr.readlines()
        f_clr.close()
        if (lines is None ) or len(lines)==0:
            return
        self.__linesToColorMap(lines)
        return
        head_line=lines[0]
        keys=parserLine(head_line)
        if keys[0].lower()!='colormap':
            return
        self.__version=int(keys[1])
        self.__interpMethod=int(keys[2])
        count_line=len(lines)
        self.__colors=[]
        for i in range(1,count_line):
            line=lines[i]
            parserLineReg(line)
            values=parserLine(line)
            color_item=ItemColorMap(float(values[0]),int(values[1]),int(values[2]),int(values[3]))
            self.__colors.append(color_item)
            if self.__val_min:
                self.__val_min=min(self.__val_min,color_item.value)
            else:
                self.__val_min=color_item.value
            if self.__val_max:
                self.__val_max=max(self.__val_max,color_item.value)
            else:
                self.__val_max=color_item.value
    def __linesToColorMap(self,lines):
        if (lines is None ) or len(lines)==0:
            return
        head_line=lines[0]

        keys=parserLineReg(head_line)
        if keys[0].lower()!='colormap':
            return
        self.__version=int(keys[1])
        self.__interpMethod=int(keys[2])
        count_line=len(lines)
        self.__colors=[]
        for i in range(1,count_line):
            line=lines[i]
            values=parserLineReg(line)
           # values=parserLine(line)
            color_item=ItemColorMap(float(values[0]),int(values[1]),int(values[2]),int(values[3]))
            self.__colors.append(color_item)
            if self.__val_min is not None:
                self.__val_min=min(self.__val_min,color_item.value)
            else:
                self.__val_min=color_item.value
            if self.__val_max is not None:
                self.__val_max=max(self.__val_max,color_item.value)
            else:
                self.__val_max=color_item.value
    def createFromValue(self,min_value,min_color,max_value,max_color,count):
        self.__val_min=min_value
        self.__val_max=max_value

        self.__colors=[]
        if count==1:
            self.__colors.append(factoryItemColor(min_value,min_color))
            self.__colors.append(factoryItemColor(max_value,max_color))
            return
        step=(max_value-min_value)/count
        color_prev=min_color
        for i in range(1,count):
            val_i=min_value+step*i
            df_val=max_value-min_value+step*(i-1)
            delta_val=val_i-min_value+step*(i-1)
            val_ration=delta_val/df_val
            color_cur=self.__calcRGB(val_ration,color_prev,max_color)
            color_prev=color_cur
            self.__colors.append(factoryItemColor(val_i,color_cur))
        return
    def clacFromDefValue(self,min_value,max_value):
        dh=(max_value-min_value)/100.0
        list_out_color_value=[]
        for itemColor in self.__colors:
            cur_proc_value=itemColor.value
            cur_value=min_value+cur_proc_value*dh
            cur_item_value=itemColor.clone()
            cur_item_value.setValue(cur_value)
            list_out_color_value.append(cur_item_value)
        return list_out_color_value

    def __calcRGB(self,val_ration,color_i,color_j,coef=0.45):
        color_ind=int(coef+color_i+val_ration*(color_j-color_i))
        if color_ind>255:
            return 255
        if color_ind<0:
            return 0
        return color_ind
    def createFromItemColorMap(self,version:int,interpMethod:int,list_ItemColorMap):
        self.__version=version
        self.__interpMethod=interpMethod
        self.__colors=[]
        for color_item in list_ItemColorMap:
            self.__colors.append(color_item)
            if self.__val_min:
                self.__val_min=min(self.__val_min,color_item.value)
            else:
                self.__val_min=color_item.value
            if self.__val_max:
                self.__val_max=max(self.__val_max,color_item.value)
            else:
                self.__val_max=color_item.value



    @property
    def ColorItems(self):
        return self.__colors
    @property
    def ZminZmax(self):
        return self.__val_min,self.__val_max
    @property
    def count(self):
        if self.__colors:
            return len(self.__colors)
        return 0
    def WriteToFile(self,path_out_file):
        if self.__colors is None:
            return False
        fo = open(path_out_file, "w+")
        out_lines=[]
        out_lines.append("ColorMap 1 1\n")
        for color_item in self.__colors:
            str_color=color_item.ColorToString
            out_lines.append(str_color+"\n")
        fo.writelines(out_lines)
        fo.close()
        return True


def ColorMapRead(path_file):
    b_exist_file=os.path.exists(path_file)
    if not b_exist_file:
        return None
    lines_file=None


class ColorTable:
    __table_color=None
    __zmin=None
    __zmax=None
    def __init__(self,color_tab=None):
        if color_tab is None:
            self.__table_color=None
        self.__table_color=color_tab
    @property
    def size(self):
        if self.__table_color:
            return self.__table_color.count
    def LoadClr(self,path_clr_file):
        self.__table_color=ColorMap(path_clr_file)
        self.__zmin,self.__zmax=self.__table_color.ZminZmax
        return self.__table_color.count
    def initDefault(self):
        self.__table_color=createDefColorMap()
        return self.__table_color.count
    @staticmethod
    def createDefault():
        cls_color=ColorTable()
        cls_color.initDefault()
        return cls_color
    def initGray(self):
        self.__table_color=createGrayColor()
        return self.__table_color.count
    def setZminZmax(self,zmin,zmax):
        self.__zmin=zmin
        self.__zmax=zmax
        self.__color_tab=None
        self.__color_tab=[]
        colr_tab_zmin,colr_tab_zmax=self.__table_color.ZminZmax
        step_z_pr=(zmax-zmin)/(colr_tab_zmax-colr_tab_zmin)
        for base_colr_item in self.__table_color.ColorItems:
            cur_val_z=zmin+base_colr_item.value*step_z_pr
            self.__color_tab.append(ItemColorMap(cur_val_z,base_colr_item.R,base_colr_item.G,base_colr_item.B))
    def getValueInProcent(self,new_value):
        delta_z=new_value-self.__zmin
        one_proc=(self.__zmax-self.__zmin)/100.0
        pr_val=delta_z/one_proc
        return pr_val
    def SavePal(self,path_file):
        return self.__table_color.WriteToFile(path_file)
    def getQImageBaseClr(self,width,height):
        pix_proc=int(width/100+0.5)
        out_image= QImage(width, height, QImage.Format_ARGB32)
        count_color=self.__table_color.count
        for i in range(count_color-1):
            start_item=self.__table_color.ColorItems[i]
            end_item=self.__table_color.ColorItems[i+1]
            proc_start=self.__table_color.ColorItems[i].value
            proc_end=self.__table_color.ColorItems[i+1].value
            start_pix=int(proc_start*pix_proc+0.5)
            end_pix=int(proc_end*pix_proc+0.5)
            start_color=ColorRgb(start_item.R,start_item.G,start_item.B)
            end_color=ColorRgb(end_item.R,end_item.G,end_item.B)
            count_pixels=int(end_pix-start_pix)
            list_colors=list(start_color.range_to(end_color,count_pixels))
            #print("Start "+str(start_pix)+" end "+str(end_pix))
            for i,item_color in enumerate(list_colors):
                i_pix=start_pix+i
                #print(i_pix)
                color_rgb=QColor(item_color.get_red()*255,item_color.get_green()*255,item_color.get_blue()*255)
                for h_i in range(height):
                    out_image.setPixel(i_pix,h_i,color_rgb.rgb())

        return out_image
    @property
    def colorMap(self):
        return self.__table_color
    def getColorForZ(self,val_z):
        if val_z<=self.__zmin:
            return QColor(self.__color_tab[0].R,self.__color_tab[0].G,self.__color_tab[0].B)
        nc=len(self.__color_tab)
        if val_z>=self.__zmax:

            return QColor(self.__color_tab[nc-1].R,self.__color_tab[nc-1].G,self.__color_tab[nc-1].B)
        for i in range(0,nc-1):
            val_0=self.__color_tab[i].value
            val_1=self.__color_tab[i+1].value

            if val_z==val_0:
                return QColor(self.__color_tab[i].R,self.__color_tab[i].G,self.__color_tab[i].B)
            if val_z>val_0 and val_z<val_1:
                df_val=val_1-val_0
                delta_val=val_z-val_0
                val_ration=delta_val/df_val
                cl_r=self.__calcRGB(val_ration,self.__color_tab[i].R,self.__color_tab[i+1].R)
                cl_g=self.__calcRGB(val_ration,self.__color_tab[i].G,self.__color_tab[i+1].G)
                cl_b=self.__calcRGB(val_ration,self.__color_tab[i].B,self.__color_tab[i+1].B)
                return QColor(cl_r,cl_g,cl_b)
        return None
    def getColorArrayFromZ(self,source_z,out_type,coef=0.45):
        color_r=self.__colorArrayFromColorComp(source_z,out_type,0,coef)
        color_g=self.__colorArrayFromColorComp(source_z,out_type,1,coef)
        color_b=self.__colorArrayFromColorComp(source_z,out_type,2,coef)
        return [color_r,color_g,color_b]
    def __colorArrayFromColorComp(self,z_values,type_out,index_color,coef):
        nx,ny=z_values.shape
        cl_i=np.zeros((nx, ny), type_out)
        #color_item=self.__color_tab[0]
        #color_r=color_item.ColorIndex(index_color)
        cl_i=cl_i+self.__color_tab[0].ColorIndex(index_color)*(z_values<self.__zmin)
        nc=len(self.__color_tab)
        cur_z_min=z_values.min()
        cur_z_max=z_values.max()
        for i in range(0,nc-1):
            val_0=self.__color_tab[i].value
            val_1=self.__color_tab[i+1].value
           # if (cur_z_min>val_0) and (cur_z_min>val_1):
           #     continue
           # if (cur_z_max<val_0):
           #     continue
            df_val=val_1-val_0
            #delta_val=val_z-val_0
            color_i_r=self.__color_tab[i].ColorIndex(index_color)
            color_j_r=self.__color_tab[i+1].ColorIndex(index_color)
            delta_r=color_j_r-color_i_r

            cl_i=cl_i+(coef+color_i_r+((z_values-val_0)/df_val)*delta_r)*np.logical_and(z_values >=val_0,z_values<val_1)
            pass
        cl_i=cl_i+(self.__color_tab[nc-1].ColorIndex(index_color)*(z_values>=self.__zmax))
        return cl_i.astype(type_out)
    def getColorPalForGdal(self,delemited,noData=None):
        count=len(self.__color_tab)
        result_outStr=[]
        for i in range(count):
            ij=count-1-i
            color_item=self.__color_tab[ij]
            str_col_item=str(color_item.value)+delemited+str(color_item.R)+delemited+str(color_item.G)+delemited+str(color_item.B)+"\r\n'"
            result_outStr.append(str_col_item)
        if noData is not None:
            pass
        return result_outStr
    def listValueColor(self):
        result=[]
        for itemColor in self.__color_tab:
            item={}
            if isinstance(itemColor.value,str):
                itemColor.value=float(itemColor.value)
            item['value']=itemColor.value

            item['color']=QColor(itemColor.R,itemColor.G,itemColor.B)
            result.append(item)
        return result
    def updateValue(self,index,value):
        self.__color_tab[index].setValue(value)
    def updateColor(self,index,color):
        self.__color_tab[index].setColor(color)
    def deleteItem(self,index):
        item=self.__color_tab.pop(index)
        item=None
    def addColor(self,index,value,color):
        self.__color_tab.insert(index,ItemColorMap(value,color.red(),color.green(),color.blue()))
    def __calcRGB(self,val_ration,color_i,color_j,coef=0.45):
        color_ind=int(coef+color_i+val_ration*(color_j-color_i))
        if color_ind>255:
            return 255
        if color_ind<0:
            return 0
        return color_ind

def getLut(color_table,color_name,type_data,noValue):
    str_lut_values=''
    if noValue is not None:
        if noValue<color_table[0].value:
            str_lut_values=valueToString(noValue,type_data)+":"+valueToString(noValue,type_data)+","
    count=len(color_table)
    for i in range(count):
        item=color_table[i]
        str_item=valueToString(item.value,type_data)+':%i'%(item.colorValueByName(color_name))
        if i<(count-1):
            str_item=str_item+","
        str_lut_values=str_lut_values+str_item
    if noValue is not None:
        if noValue>color_table[count-1].value:
            str_lut_values=str_lut_values+','+valueToString(noValue,type_data)+":"+valueToString(noValue,type_data)
    return str_lut_values




