import os
from enum import Enum
from pathlib import Path

from osgeo import gdal, gdalconst

#from RasterGridTool.toolprocessing.grid_io.VrtGridColor import VrtXml


def gdalTypeToString(type_gdal):
    if type_gdal==gdalconst.GDT_Byte:
        return 'Byte'
    if type_gdal==gdalconst.GDT_Int16:
        return 'Int16'
    if type_gdal==gdalconst.GDT_UInt16:
        return 'UInt16'
    if type_gdal==gdalconst.GDT_Int32:
        return 'Int32'
    if type_gdal==gdalconst.GDT_UInt32:
        return 'UInt32'
    if type_gdal==gdalconst.GDT_Float32:
        return 'Float32'
    if type_gdal==gdalconst.GDT_Float62:
        return 'Float64'
    return None
def BandInfo(band):
    type=band.DataType
    #des=band.GetDescription()
    statistic=band.GetStatistics(False,True)
    #min_z=band.GetMinimum()
    #max_z=band.GetMaximum()
    min_z=statistic[0]
    max_z=statistic[1]
    if max_z is None or min_z is None:
        statistic=band.GetStatistics(True,True)
        band_info= {'type':type,'min':statistic[0],'max':statistic[1],'mean':statistic[2],'stddev':statistic[3]}
    else:
        band_info= {'type':type,'min':min_z,'max':max_z,'mean':None,'stddev':None}
    band_info['scale']=band.GetScale()
    band_info['offset']=band.GetOffset()
    band_info['id_band']=band.GetBand()
    return band_info


class RasterSource:
    __isGrid=None
    __base_band=None
    __def_base_band=1
    __vrt=None
    def __init__(self):
        self.__id_base_band=self.__def_base_band
    def OpenRead(self,path_file):
        if not os.path.exists(path_file):
            return False
        self.__path_file=path_file
        self.__ds=gdal.Open( self.__path_file)
        if self.__ds is None:
            return False
        self.__ds_drive_name=self.__ds.GetDriver().ShortName
        meta_=self.__ds.GetMetadata()
        if meta_ is not None:
            try:
                str_TIFFTAG_IMAGEDESCRIPTION=meta_['TIFFTAG_IMAGEDESCRIPTION']
                list_tag_value=str_TIFFTAG_IMAGEDESCRIPTION.split(',')
                for item in list_tag_value:
                    tag_value=item.split('=')
                    if tag_value[0].strip().lower()=='fieldtype':
                        if tag_value[1].strip().lower()=='continuous':
                            self.__isGrid=1
            except:
                pass
        #info=gdal.Info(self.__path_file)
        self.__count_band=self.__ds.RasterCount
        self.__bandInfo()
        if self.__isGrid is None or self.__isGrid==0:
            self.__isGridSource()
            if self.__ds_drive_name=="VRT":
                from RasterGridTool.toolprocessing.grid_io.VrtGridColor import VrtXml
                self.__vrt=VrtXml(self.__path_file)
        if self.__ds is not None:
            self.__ds=None
        return True
    @property
    def IsVrt(self):
        if self.__vrt is None:
            return False
        return True
    @property
    def vrtBaseGrid(self):
        return self.__vrt.baseRaster
    @property
    def driverName(self):
        return self.__ds_drive_name
    @property
    def MinMax(self):
        if self.__band_info is not None:
            return self.__min,self.__max
        return None,None
    @property
    def width(self):
        if self.__ds is None:
            return None
        return self.__ds.RasterXSize
    @property
    def height(self):
        if self.__ds is None:
            return None
        return self.__ds.RasterYSize

    @property
    def Projection(self):
        if self.__ds is None:
            return None
        return self.__ds.GetProjection()
    @property
    def GeoTransform(self):
        return self.__ds.GetGeoTransform()
    def setBaseBand(self,id_band):
        if self.__ds is None:
            self.__ds=gdal.Open( self.__path_file)
        self.__base_band=self.__ds.GetRasterBand(id_band)
        self.__id_base_band=id_band
        return self.__base_band
    @property
    def BaseBand(self):
        return self.__base_band
    @property
    def fileRaster(self):
        return self.__path_file
    @property
    def nameFile(self):
        return str(Path(self.__path_file).name)
    def __bandInfo(self):
        self.__band_info=[]
        for i in range(self.__count_band):
            band=self.__ds.GetRasterBand(i+1)
            self.__band_info.append(BandInfo(band))
            band=None
        return
    def __isGridSource(self):
        self.__isGrid=0
        for band_inf in self.__band_info:
            if (band_inf['type']==gdalconst.GDT_Byte):
                #or band_inf['type']==gdalconst.GDT_Int16 or band_inf['type']==gdalconst.GDT_Int32) and band_inf['scale'] is not None and band_inf['offset']:
                #self.__isGrid=self.__isGrid+1
                continue
            if band_inf['type']==gdalconst.GDT_UInt16 or band_inf['type']==gdalconst.GDT_Int16 or band_inf['type']==gdalconst.GDT_Int32 or band_inf['type']==gdalconst.GDT_Float32 or band_inf['type']==gdalconst.GDT_Float64:
                self.__isGrid=self.__isGrid+1
                self.__min=band_inf['min']
                self.__max=band_inf['max']
                self.__grid_band=band_inf['id_band']
                continue
        return
    @property
    def getIdBand(self):
        return self.__grid_band
    @property
    def isGrid(self):
        return self.__isGrid
    @property
    def Xoffset(self):
        return 0
    @property
    def Yoffset(self):
        return 0
    def close(self):
        if self.__ds is not None:
            del self.__ds
            self.__ds=None
    





