import io
import os.path
from pathlib import Path

import axipy
from axipy import provider_manager
from osgeo import gdalconst

from .ColorTable import getLut
#from .ColorTable import getLut, ItemColorMap, ColorMap, ColorTable
from .GdalSource import RasterSource, gdalTypeToString
from .utils import MiTabFileRaster
from ..ui.Palette.tools.ColorInterpolation import ItemColorMap
from ..ui.Palette.tools.ColorTable import ColorMap, ColorTable
from ..ui.addTools.mapTool import addRasterTabLayerUpTab, reloadRasterTab


def CreateBandVrt(base_raster_grid:RasterSource,id_band,name_color,table_color,typeBand="Byte"):
    out_band=[]

    band=base_raster_grid.BaseBand
    noData=band.GetNoDataValue()
    data_type=band.DataType
    str_data_type=gdalTypeToString(data_type)
    str_head_band='<VRTRasterBand dataType="%s" band="%i">'%(typeBand,id_band)
    out_band.append(str_head_band)
    if noData is not None:
        str_noData=""
        if data_type==gdalconst.GDT_Byte or data_type==gdalconst.GDT_Int16 or data_type==gdalconst.GDT_Int32 or data_type==gdalconst.GDT_UInt16  or  data_type==gdalconst.GDT_UInt32:
            str_noData='%i'%(noData)
        else:
            str_noData='%f'%(noData)
        out_band.append('<NoDataValue>'+str_noData+'</NoDataValue>')
    if name_color is not None:
        out_band.append('<ColorInterp>%s</ColorInterp>'%(name_color))
    out_band.append('<ComplexSource>')
    out_band.append('<SourceFilename relativeToVRT="1">%s</SourceFilename>'%(base_raster_grid.nameFile))
    out_band.append('<SourceBand>%i</SourceBand>'%(band.GetBand()))
    block_size=band.GetBlockSize()

    out_band.append('<SourceProperties RasterXSize="%i" RasterYSize="%i" DataType="%s" BlockXSize="%i" BlockYSize="%i"/>'%(base_raster_grid.width,base_raster_grid.height,str_data_type,block_size[0],block_size[1]))
    out_band.append('<SrcRect xOff="0" yOff="0" xSize="%i" ySize="%i"/>'%(base_raster_grid.width,base_raster_grid.height))
    out_band.append('<DstRect xOff="0" yOff="0" xSize="%i" ySize="%i"/>'%(base_raster_grid.width,base_raster_grid.height))
    if name_color is not None:
        str_lut=getLut(table_color,name_color,data_type,noData)
        out_band.append('<LUT>'+str_lut+'</LUT>')
    out_band.append('</ComplexSource>')
    out_band.append('</VRTRasterBand>')
    return out_band

class VrtColorGrid:
    def __init__(self,base_raster_grid:RasterSource):
        self.__base_grid=base_raster_grid
    def write(self,id_base_band,color_table,path_out):
        self.__base_grid.setBaseBand(id_base_band)
        list_colorInterp=["Red",'Blue','Green',None]

        out_string=[]
        out_string.append(self.__getHead())
        out_string.append(self.__getSrs())
        str_geo_tr=self.__geoTransform()
        if str_geo_tr is not None:
            out_string.append(str_geo_tr)
        min_value=self.__base_grid.BaseBand.GetMinimum()
        max_value=self.__base_grid.BaseBand.GetMaximum()
        for id_band_out in range(1,4):
            data_band_vtr=CreateBandVrt(self.__base_grid,id_band_out,list_colorInterp[id_band_out-1],color_table)
            out_string.extend(data_band_vtr)

        data_band_vtr=CreateBandVrt(self.__base_grid,4,None,color_table)
        out_string.extend(data_band_vtr)
        out_string.append('</VRTDataset>')
        with io.open(path_out,'w',encoding='utf8') as f:
            for str_out in out_string:
                f.write(str_out)
    def __getHead(self):
        head_str='<VRTDataset rasterXSize="%i" rasterYSize="%i">'%(self.__base_grid.width,self.__base_grid.height)
        return head_str
    def __getSrs(self):
        str_srs='<SRS>%s</SRS>'%(self.__base_grid.Projection)
        return str_srs
    def __geoTransform(self):
        data_geo_tr=self.__base_grid.GeoTransform
        if self.__base_grid.GeoTransform is None:
            return None

        #str_geoTransform='<GeoTransform> %f ,%f,%f,%f,%f,%f </GeoTransform>'%(data_geo_tr[0],data_geo_tr[1],data_geo_tr[2],data_geo_tr[3],data_geo_tr[4],data_geo_tr[5])
        str_geoTransform='<GeoTransform> %0.17E ,%0.17E,%0.17E,%0.17E,%0.17E,%0.17E </GeoTransform>'%(data_geo_tr[0],data_geo_tr[1],data_geo_tr[2],data_geo_tr[3],data_geo_tr[4],data_geo_tr[5])
        return str_geoTransform
import xml.etree.ElementTree as ET
def getColorTabVrt(path_vrt):
    tree = ET.parse(path_vrt)
    root=tree.getroot()
    return getColorTable(root)
def lut_interval(list_lut_values,no_value):
    out_color=[]
    for item_lut in list_lut_values:
        values=item_lut.split(':')
        if values[0]==no_value:
            continue
        color_value={'value':float(values[0]),"color":int(values[1])}
        out_color.append(color_value)
    return out_color
class VrtRaterBand:
    def __init__(self,color:str,no_value:str,list_colr_value,source_file=None,source_band=None):
        self.__name_color=color
        self.__no_value=no_value
        self.__value_colors=list_colr_value
        self.__sourth_file=source_file
        self.__source_band=source_band
    @property
    def NameColor(self):
        return self.__name_color
    @property
    def size(self):
        return len(self.__value_colors)
    @property
    def MinValue(self):
        return self.__value_colors[0]['value']
    @property
    def MaxValue(self):
        return self.__value_colors[self.size-1]['value']

    def ValueItem(self,index):
        return self.__value_colors[index]['value'],self.__value_colors[index]['color']
    @property
    def fileSource(self):
        return self.__sourth_file
    @property
    def bandSource(self):
        return self.__source_band
    @staticmethod
    def factory(root_xml):
        tag_NoDataValue=findXmlTag(root_xml,"NoDataValue")
        str_NoDataValue=None
        if tag_NoDataValue is not None:
            str_NoDataValue=tag_NoDataValue.text
        tag_ColorInterp=findXmlTag(root_xml,"ColorInterp")
        str_color=None
        if tag_ColorInterp is not None:
            str_color=tag_ColorInterp.text
        if str_color is None:
            return None
        tag_ComplexSource=findXmlTag(root_xml,"ComplexSource")
        if tag_ComplexSource is None:
            return None
        tag_sourceFilename=findXmlTag(tag_ComplexSource,'SourceFilename')
        name_source=tag_sourceFilename.text
        tag_SourceBand=findXmlTag(tag_ComplexSource,'SourceBand')
        band_source=tag_SourceBand.text
        tag_lut=findXmlTag(tag_ComplexSource,"LUT")
        if tag_lut is None:
            return None
        str_lut=tag_lut.text
        values_intrevals_color=str_lut.split(',')
        lut=lut_interval(values_intrevals_color,str_NoDataValue)
        if len(lut)==0:
            return None
        return VrtRaterBand(str_color,str_NoDataValue,lut,name_source,int(band_source))

def findXmlTag(root_xml,name_tag):

    for item in root_xml:
        if item.tag==name_tag:
            return item
    return None
def getLutInBand(root_xml):
    str_nodata=None
    str_color=None
    for item in root_xml:
        if item.tag=='NoDataValue':
            str_nodata=item.text
        if item.tag=='ColorInterp':
            str_color=item.text
        if item.tag=='ComplexSource':
            jkl=0
def getColorTable(root_xml):
    #teg_vrtRaster=findXmlTag(root_xml,'VRTRasterBand')
    list_Raster_band=[]
    for item in root_xml:
        if item.tag=='VRTRasterBand':
            band_item=VrtRaterBand.factory(item)
            if band_item is not None:
                list_Raster_band.append(band_item)
            jkl=0
    if len(list_Raster_band)!=3:
        return None
    isValid=True
    max_len=0
    color_index={}
    for i,item_Color in enumerate(list_Raster_band):
        cut_len=item_Color.size
        name_color=item_Color.NameColor
        if name_color.lower()=="red":
            color_index['r']=i
        if name_color.lower()=="green":
            color_index['g']=i
        if name_color.lower()=="blue":
            color_index['b']=i
        if i==0:
            max_len=cut_len
            continue
        if cut_len!=max_len:
            isValid=False
            break
    min_value=list_Raster_band[0].MinValue
    max_value=list_Raster_band[0].MaxValue
    val_proc=(max_value-min_value)/100.0
    itemItemColorMap=[]
    for i in range(list_Raster_band[0].size):
        val_r,col_r=list_Raster_band[color_index['r']].ValueItem(i)
        val_g,col_g=list_Raster_band[color_index['g']].ValueItem(i)
        val_b,col_b=list_Raster_band[color_index['b']].ValueItem(i)
        proc_value=(val_r-min_value)/val_proc
        itemItemColorMap.append(ItemColorMap(proc_value,int(col_r),int(col_g),int(col_b)))
    colorMap=ColorMap()
    colorMap.createFromItemColorMap(1,1,itemItemColorMap)
    return ColorTable(colorMap)

class VrtXml:
    def __init__(self,path_vrt):
        self.__path_vrt=path_vrt
        if path_vrt is not None:
            self._read()
    @property
    def countColrBand(self):
        return len(self.__list_Raster_band)
    @property
    def colorFile(self):
        return self.__path_vrt
    def _read(self):
        tree = ET.parse(self.__path_vrt)
        root=tree.getroot()
        self.__list_Raster_band=[]
        for item in root:
            if item.tag=='VRTRasterBand':
                band_item=VrtRaterBand.factory(item)
                if band_item is not None:
                    self.__list_Raster_band.append(band_item)
        return
    def colorMap(self):
        isValid=True
        max_len=0
        color_index={}
        for i,item_Color in enumerate(self.__list_Raster_band):
            cut_len=item_Color.size
            name_color=item_Color.NameColor
            if name_color.lower()=="red":
                color_index['r']=i
            if name_color.lower()=="green":
                color_index['g']=i
            if name_color.lower()=="blue":
                color_index['b']=i
            if i==0:
                max_len=cut_len
                continue
            if cut_len!=max_len:
                isValid=False
                break
        min_value=self.__list_Raster_band[0].MinValue
        max_value=self.__list_Raster_band[0].MaxValue
        val_proc=(max_value-min_value)/100.0
        itemItemColorMap=[]
        for i in range(self.__list_Raster_band[0].size):
            val_r,col_r=self.__list_Raster_band[color_index['r']].ValueItem(i)
            val_g,col_g=self.__list_Raster_band[color_index['g']].ValueItem(i)
            val_b,col_b=self.__list_Raster_band[color_index['b']].ValueItem(i)
            proc_value=(val_r-min_value)/val_proc
            itemItemColorMap.append(ItemColorMap(proc_value,int(col_r),int(col_g),int(col_b)))
        colorMap=ColorMap()
        colorMap.createFromItemColorMap(1,1,itemItemColorMap)
        return ColorTable(colorMap)
    @property
    def baseRaster(self):
        list_raster=[]
        for item in self.__list_Raster_band:
            name_source=item.fileSource
            band=item.bandSource
            source_data={"file":name_source,'band':band}
            if source_data in list_raster:
                continue
            list_raster.append(source_data)
        return list_raster
    def updateColor(self,color_tab:ColorTable,out_path_new=None)->bool:
        out_path=out_path_new
        if out_path is None:
            out_path=self.__path_vrt
        base_folder=Path(self.__path_vrt).parent
        base_raster_info=self.baseRaster
        if len(base_raster_info)==1:
            base_raster_file=base_raster_info[0]['file']
            base_band=base_raster_info[0]['band']
        else:
            return False
        full_path_base_raster=os.path.join(base_folder,base_raster_file)
        gridSource=RasterSource()
        try:
            gridSource.OpenRead(full_path_base_raster)
            gridSource.setBaseBand(base_band)
            list_colorInterp=["Red",'Blue','Green',None]

            out_string=[]
            string_head= head_str='<VRTDataset rasterXSize="%i" rasterYSize="%i">'%(gridSource.width,gridSource.height)
            out_string.append(string_head)
            str_srs='<SRS>%s</SRS>'%(gridSource.Projection)
            out_string.append(str_srs)
            data_geo_tr=gridSource.GeoTransform
            if gridSource.GeoTransform is None:
                return False

        #str_geoTransform='<GeoTransform> %f ,%f,%f,%f,%f,%f </GeoTransform>'%(data_geo_tr[0],data_geo_tr[1],data_geo_tr[2],data_geo_tr[3],data_geo_tr[4],data_geo_tr[5])
            str_geo_tr='<GeoTransform> %0.17E ,%0.17E,%0.17E,%0.17E,%0.17E,%0.17E </GeoTransform>'%(data_geo_tr[0],data_geo_tr[1],data_geo_tr[2],data_geo_tr[3],data_geo_tr[4],data_geo_tr[5])

            if str_geo_tr is not None:
                out_string.append(str_geo_tr)
            min_value=gridSource.BaseBand.GetMinimum()
            max_value=gridSource.BaseBand.GetMaximum()
            list_color=color_tab.clacFromDefValue(min_value,max_value)
            for id_band_out in range(1,4):
                data_band_vtr=CreateBandVrt(gridSource,id_band_out,list_colorInterp[id_band_out-1],list_color)
                out_string.extend(data_band_vtr)
            band=gridSource.BaseBand

            data_type=band.DataType
            str_data_type=gdalTypeToString(data_type)
            data_band_vtr=CreateBandVrt(gridSource,4,None,color_tab,str_data_type)
            out_string.extend(data_band_vtr)
            out_string.append('</VRTDataset>')
            with io.open(out_path,'w',encoding='utf8') as f:
                for str_out in out_string:
                    f.write(str_out)
        except Exception as ex:
            pass
        finally:
            gridSource.close()
        return True
    @staticmethod
    def createVrtColor(raster_source:RasterSource,color_table,bandId=1,name_vrt=None):
        path_source=raster_source.fileRaster
        base_folder=Path(path_source).parent
        name_raster=Path(path_source).name
        name_without_ext=Path(path_source).stem
        out_vrt_name=name_vrt
        if name_vrt is None:
            out_vrt_name=name_without_ext+"_color.vrt"
        full_out_path_vrt=os.path.join(base_folder,out_vrt_name)
        raster_source.setBaseBand(bandId)
        list_colorInterp=["Red",'Blue','Green',None]

        out_string=[]
        string_head= head_str='<VRTDataset rasterXSize="%i" rasterYSize="%i">'%(raster_source.width,raster_source.height)
        out_string.append(string_head)
        str_srs='<SRS>%s</SRS>'%(raster_source.Projection)
        out_string.append(str_srs)
        data_geo_tr=raster_source.GeoTransform
        if raster_source.GeoTransform is None:
            return False

            #str_geoTransform='<GeoTransform> %f ,%f,%f,%f,%f,%f </GeoTransform>'%(data_geo_tr[0],data_geo_tr[1],data_geo_tr[2],data_geo_tr[3],data_geo_tr[4],data_geo_tr[5])
        str_geo_tr='<GeoTransform> %0.17E ,%0.17E,%0.17E,%0.17E,%0.17E,%0.17E </GeoTransform>'%(data_geo_tr[0],data_geo_tr[1],data_geo_tr[2],data_geo_tr[3],data_geo_tr[4],data_geo_tr[5])

        if str_geo_tr is not None:
            out_string.append(str_geo_tr)
        min_value=raster_source.BaseBand.GetMinimum()
        max_value=raster_source.BaseBand.GetMaximum()
        list_color=color_table.colorMap.clacFromDefValue(min_value,max_value)
        for id_band_out in range(1,4):
            data_band_vtr=CreateBandVrt(raster_source,id_band_out,list_colorInterp[id_band_out-1],list_color)
            out_string.extend(data_band_vtr)
        band=raster_source.BaseBand

        data_type=band.DataType
        str_data_type=gdalTypeToString(data_type)
        data_band_vtr=CreateBandVrt(raster_source,4,None,color_table,str_data_type)
        out_string.extend(data_band_vtr)
        out_string.append('</VRTDataset>')
        try:
            with io.open(full_out_path_vrt,'w',encoding='utf8') as f:
                for str_out in out_string:
                    f.write(str_out)
        except:
            return None

        return out_vrt_name

class RasterGrid:
    __raster=None
    __colorTable=None
    __def_active_band=1
    __value_min=None
    __value_max=None
    __no_value=None
    __vrt_file=None
    def __init__(self,tab_raster:axipy.da.Raster):
        self.__vrt_file=None
        if isinstance(tab_raster,axipy.da.Raster):
            self.__raster_path = tab_raster.properties['fileName']
            try:
                self.__path_tab = tab_raster.properties['tabFile']
            except:
                pass
        else:
            self.__path_tab=tab_raster.properties['tabFile']
            self.__raster_path=tab_raster.properties['tabFileData']['linkFile']
        self.__raster=RasterSource()
        self.__raster.OpenRead(self.__raster_path)
        self.__prepareColor()
        self.__raster.close()
        self.__tab_raster=tab_raster
    @property
    def axiRasterTab(self):
        return self.__tab_raster
    @property
    def isGrid(self):
        if self.__raster.isGrid is None:
            return False
        if self.__raster.isGrid>0:
            return True
        return False
    @property
    def existPalette(self):
        return self.__existPalette
    @property
    def path(self):
        return self.__path_tab
    @property
    def pathRastre(self):
        return self.__raster_path
    @property
    def colorTable(self):
        return self.__colorTable
    @property
    def colorGridTab(self):
        if self.__vrt_file is None:
            return None
        return self.__path_tab
    @property
    def baseRaster(self):
        if self.__vrt_file is None:
            return None
        return self.__vrt_file
    @property
    def baseGrid(self):

        return None
    def updateColorTable(self,color_table,new_path_tab=None,path_vrt=None):
        if self.__existPalette:
            if new_path_tab is None and path_vrt is None and self.__vrt_file is not None:
                ''' Изменения в существуещем файле'''
                isOk=self.__vrt_file.updateColor(color_table)
                reloadRasterTab(self.__tab_raster)
                return

        if self.__raster.isGrid:
            path_out_tab_color=new_path_tab
            if new_path_tab is None:
                path_base=Path(self.__path_tab).parent
                name_source=Path(self.__path_tab).stem
                path_out_tab_color=os.path.join(path_base,name_source+"_vrt.tab")
            self.__raster.OpenRead(self.__raster_path)
            try:
                name_vrt=VrtXml.createVrtColor(self.__raster,color_table,1,None)
            except Exception as ex:
                pass
            finally:
                self.__raster.close()
            cls_tab=MiTabFileRaster.createClass(self.__path_tab)
            cls_tab.setNewRaster(name_vrt)
            path_base=Path(self.__path_tab).parent
            name_source=Path(self.__path_tab).stem
            path_vrt_tab=os.path.join(path_base,name_source+"_vrt.tab")
            cls_tab.saveAs(path_vrt_tab)
            tab_vrt=provider_manager.openfile(path_vrt_tab)
            addRasterTabLayerUpTab(self.__tab_raster,tab_vrt)
            self.__raster.close()


    def __prepareColor(self):
        self.__existPalette=False
        name_drv=self.__raster.driverName
        if name_drv=="VRT":
            self.__vrt_file=VrtXml(self.__raster_path)
            if self.__vrt_file.countColrBand<3:
                self.__vrt_file=None
            #self.__colorTable=getColorTabVrt(self.__raster_path)
            self.__colorTable=self.__vrt_file.colorMap()
            if self.__colorTable is not None:
                self.__existPalette=True







