import inspect
import os

from PySide2.QtCore import QFile
from PySide2.QtGui import QPainter, QPixmap, QIcon, QColor
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QDialog, QListWidgetItem, QFileDialog

from RasterGridTool.toolprocessing.ui.Palette.DlgChangeColorValue import DlgChangeColorValue
#from GridTools.ColorizeGrid.DlgChangeColorValue import DlgChangeColorValue
from RasterGridTool.toolprocessing.ui.Palette.tools.ColorInterpolation import ItemColorMap
from RasterGridTool.toolprocessing.ui.Palette.tools.ColorTable import ColorTable, ColorMap


#from AxiomaRaster.gridtools.ColorTable import ColorTable, createDefColorMap, ColorMap, ItemColorMap


class PaintWidget(QWidget):
    def setColorTab(self,color_table:ColorTable):

        self.__colorTab=color_table
    def updateColor(self,color,index):
        if self.__colorTab is not None:
            self.__colorTab.colorMap[index].setColor(color)
            self.__colorTab.updateColors()
    def updateValue(self,value,index):
        if self.__colorTab is not None:
            self.__colorTab.colorMap[index].setValue(value)
    def addColorItem(self,index,color,value):
        if self.__colorMap is not None:
            self.__colorMap.ColorItems.insert(index+1,ItemColorMap(value,color.red(),color.green(),color.blue()))
    def deleteColorItem(self,index):
        if self.__colorMap is not None:
            item=self.__colorMap.ColorItems.pop(index)
            item=None

    def paintEvent(self, event):
        #print("update")
        qp = QPainter(self)

      #qp.setPen(QColor.black)
        size = self.size()


        height=size.height()
        width=size.width()
        dz=self.__colorTab.Zmax-self.__colorTab.Zmin
        pixel_value=dz/height
        old_color=QColor()
        for ih in range(height):
            cur_value = self.__colorTab.Zmin + ih * pixel_value
            #curent_color=self.__colorTab.getColor(float(cur_value))
            if ih>=296:
                jkl=0
            curent_color=self.__colorTab.getColorColorAide(float(cur_value))
            if curent_color.red()==0 and curent_color.blue()==0 and curent_color.green()==0:
                jkl=0
            qp.setPen(curent_color)
            if not(old_color==curent_color):
                old_color=curent_color
           # ih_reverse=height-ih-1
            qp.drawLine(0,ih,width,ih)
        pass

class DlgColorGridGdal(QDialog):
    __DefColorMap=None
    __dlgChangeColorValue=None
    __isOkDlg=False
    def __init__(self,zmin_grid,zmax_grid, parent=None):
        super().__init__(parent)
        self.__parent=parent
        cwd = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
        self.__ui = self.load_ui("DlgColorGridNew.ui")
        self.__ui.pb_LoadColor.clicked.connect(self.__loadClr)
        self.__ui.pb_add.clicked.connect(self.__AddValue)

        self.__zmin=zmin_grid
        self.__zmax=zmax_grid
        self.__colorTab=ColorTable()
        self.__colorTab.setZminZmax(self.__zmin,self.__zmax)
        self.m = PaintWidget(self)
        self.m.setColorTab(self.__colorTab)
        self.m.move(0,0)
        self.width=self.__ui.verticalLayoutWidget.width
        self.height==self.__ui.verticalLayoutWidget.height

        self.__ui.verticalLayout.addWidget(self.m)
        self.__fillListColor()
        self.__ui.listViewColorInterval.itemDoubleClicked.connect(self.__color_item_click)
        self.__ui.listViewColorInterval.clicked.connect(self.__color_item_sel)
        self.__ui.pb_del.clicked.connect(self.__del_item_sel)
        self.__ui.pb_create_pal.clicked.connect(self.__create_new_pal)
        self.__ui.pb_Save.clicked.connect(self.__save_pal)
        self.__ui.pb_ok.clicked.connect(self.__OkButtom)
        self.__ui.pb_Cancel.clicked.connect(self.__CancelButtom)
        #self.m.resize(self.width,self.height)
    def load_ui(self,name_ui):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_ui)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.__ui  = loader.load(ui_file,self.__parent)
        ui_file.close()
        return self.__ui
    @property
    def colorTable(self):
        return self.__colorTab
    @property
    def isOk(self):
        return self.__isOkDlg
    def ColorTab(self):
        return self.__colorTab
    def __OkButtom(self):
        self.__isOkDlg=True
        self.__ui.close()
    def __CancelButtom(self):
        self.__isOkDlg=False
        self.__ui.close()
    def __fillListColor(self):
        listColorValue=self.__colorTab.colorMap
        for colorItem in listColorValue:

            item_lst=self.__createItemList(colorItem.Color,colorItem.value)
            self.__ui.listViewColorInterval.addItem(item_lst)
        self.__ui.listViewColorInterval.setCurrentRow(0)
    def __createItemList(self,color_item,value):
        pixmap = QPixmap(20, 20)
        pixmap.fill(color_item)
        icon=QIcon()
        icon.addPixmap(pixmap)
        return QListWidgetItem(icon,str(value))
    def __createIcon(self,color):
        pixmap = QPixmap(20, 20)
        pixmap.fill(color)
        icon=QIcon()
        icon.addPixmap(pixmap)
        return icon
    def __loadClr(self):
        pathBase=os.path.realpath(__file__)
        pathFolderDefColorPallet=os.path.join(os.path.dirname(pathBase), "ColorPalettes")
        ext_file='*.'+'clr'
        name_load_file= QFileDialog.getOpenFileName(self, 'Загрузить палитру', pathFolderDefColorPallet, ext_file)
        if name_load_file is None:
            return
        self.__DefColorMap=None
        self.__DefColorMap=ColorMap(name_load_file[0])
        #self.__colorTab=None
        zmin=self.__colorTab.Zmin
        zmax=self.__colorTab.Zmax
        self.__colorTab=ColorTable(self.__DefColorMap,zmin,zmax)

        #self.__colorTab.setZminZmax(self.__zmin,self.__zmax)
        self.__ui.listViewColorInterval.clear()
        self.__fillListColor()
        self.m.setColorTab(self.__colorTab)
        self.m.update()
        #self.m.setColorTab(ColorMap(name_load_file[0]))
    def LoadPalette(self,path_file_clr):
        self.__DefColorMap = None
        self.__DefColorMap = ColorMap(path_file_clr)
        # self.__colorTab=None
        zmin = self.__colorTab.Zmin
        zmax = self.__colorTab.Zmax
        self.__colorTab = ColorTable(self.__DefColorMap, zmin, zmax)

        # self.__colorTab.setZminZmax(self.__zmin,self.__zmax)
        self.__ui.listViewColorInterval.clear()
        self.__fillListColor()
        self.m.setColorTab(self.__colorTab)
        self.m.update()
    def setColorTable(self,color_table):
        self.__colorTab=color_table
        self.__ui.listViewColorInterval.clear()
        self.__fillListColor()
        self.m.setColorTab(self.__colorTab)
        self.m.update()
    def __color_item_click(self,item):
        #sel_index=item.row()
        row = self.__ui.listViewColorInterval.currentRow()
        sel_index=row
        listColorValue=self.__colorTab.colorMap
        color=listColorValue[sel_index].Color
        value=listColorValue[sel_index].value
        if self.__dlgChangeColorValue is None:
            self.__dlgChangeColorValue=DlgChangeColorValue(None,None,self)
            pass
        low_val=str(listColorValue[sel_index].value)
        hig_val=str(listColorValue[sel_index].value)
        bIsEnableRecalc=False
        if (sel_index>0) and (sel_index<(len(listColorValue)-1)):
            low_val=str(listColorValue[sel_index-1].value)
            hig_val=str(listColorValue[sel_index+1].value)
        if sel_index==(len(listColorValue)-1):
            low_val=str(listColorValue[sel_index-1].value)
            bIsEnableRecalc=True
        if sel_index==0:
            hig_val=str(listColorValue[sel_index+1].value)
            bIsEnableRecalc=True

        self.__dlgChangeColorValue.setColorAndValue(color,str(value),low_val,hig_val,bIsEnableRecalc)
        self.__dlgChangeColorValue.exec_()
        bIsOk=self.__dlgChangeColorValue.IsOk
        if not bIsOk:
            return
        new_color,is_change_color=self.__dlgChangeColorValue.Color
        new_value,is_change_value=self.__dlgChangeColorValue.Value
        bRecalcProc=False
        if bIsEnableRecalc:
            bRecalcProc=self.__dlgChangeColorValue.bRecalProc
        if is_change_color and (not is_change_value):
            # изменен только цвет
            self.__updateColor(is_change_color,sel_index,new_color)
            #new_icon=self.__createIcon(new_color)
            #item_sel=self.__ui.listViewColorInterval.item(sel_index)
            #self.__ui.listViewColorInterval.item(sel_index).setIcon(new_icon)
            #self.m.updateColor(new_color,sel_index)
        if is_change_value:
            flt_value=float(new_value)
            #pr_val=self.__colorTab.getValueInProcent(flt_value)
            count_values=len(listColorValue)
            if sel_index==0:
                next_value=listColorValue[sel_index].value
                if flt_value<next_value:
                    # update value index 0
                    # пока не расширяем динамический дипозон
                    #self.m.updateValue(0.0,sel_index)
                    #self.__ui.listViewColorInterval.item(sel_index).setText(str(flt_value))
                    if not bRecalcProc:
                        self.__updateValue(sel_index,flt_value)
                        self.__updateColor(is_change_color,sel_index,new_color)
                    else:
                        temp_zmin,temp_zmax=self.__colorTab.ZminZmax
                        self.__colorTab.setZminZmax(flt_value,temp_zmax)
                        self.__ui.listViewColorInterval.clear()
                        self.__fillListColor()
                    return
                else:
                    self.__resortColor(sel_index,new_color,new_value)
            if sel_index==(count_values-1):
                pre_value=listColorValue[sel_index-1]['value']
                if flt_value>pre_value:

                    if not bRecalcProc:
                        self.__updateValue(sel_index,flt_value)
                        self.__updateColor(is_change_color,sel_index,new_color)
                        self.__resortColor(sel_index,new_color,new_value)
                    else:
                        temp_zmin,temp_zmax=self.__colorTab.ZminZmax
                        self.__colorTab.setZminZmax(temp_zmin,flt_value)
                        self.__ui.listViewColorInterval.clear()
                        self.__fillListColor()

                return
            pre_value=listColorValue[sel_index-1].value
            next_value=listColorValue[sel_index+1].value
            if flt_value>pre_value and flt_value<next_value:
                #pr_val=self.__colorTab.getValueInProcent(flt_value)
                self.m.updateValue(flt_value,sel_index)
                #self.__ui.listViewColorInterval.item(sel_index).setText(str(flt_value))
                self.__updateValue(sel_index,flt_value)
                self.__updateColor(is_change_color,sel_index,new_color)
            else:
                self.__resortColor(sel_index,new_color,new_value)
        pass
    def __resortColor(self,sel_index,color,value):
        pass
    def __updateValue(self,index,value):
        self.__ui.listViewColorInterval.item(index).setText(str(value))
        self.__colorTab.colorMap[index].setValue(value)
    def __updateColor(self,is_update,index,new_color):
        if is_update:
            # изменен цвет
            new_icon=self.__createIcon(new_color)
            item_sel=self.__ui.listViewColorInterval.item(index)
            self.__ui.listViewColorInterval.item(index).setIcon(new_icon)
            self.m.updateColor(new_color,index)
            self.__colorTab.colorMap[index].setColor(new_color)
            self.m.update()
    def __AddValue(self):
        listColorValue=self.__colorTab.colorMap
        sel_index = self.__ui.listViewColorInterval.currentRow()
        if self.__dlgChangeColorValue is None:
            self.__dlgChangeColorValue=DlgChangeColorValue(None,None,self)
            pass
        color=listColorValue[sel_index].Color
        str_value=listColorValue[sel_index].value
        low_val=str(listColorValue[sel_index].value)
        hig_val=str(listColorValue[sel_index].value)
        if (sel_index>0) and (sel_index<(len(listColorValue)-1)):
            low_val=str(listColorValue[sel_index].value)
            hig_val=str(listColorValue[sel_index+1].value)
            new_value=(listColorValue[sel_index+1].value-listColorValue[sel_index].value)/2.0+listColorValue[sel_index].value
            color=self.__colorTab.getColorColorAide(new_value)
        if sel_index==(len(listColorValue)-1):
            low_val=str(listColorValue[sel_index-1].value)
            new_value=(listColorValue[sel_index].value-listColorValue[sel_index-1].value)/2.0+listColorValue[sel_index-1].value
            color=self.__colorTab.getColorColorAide(new_value)
        if sel_index==0:
            hig_val=str(listColorValue[sel_index+1].value)
            new_value=(listColorValue[sel_index+1].value-listColorValue[sel_index].value)/2.0+listColorValue[sel_index].value
            color=self.__colorTab.getColorColorAide(new_value)
        str_value=str(new_value)

        self.__dlgChangeColorValue.setColorAndValue(color,str_value,low_val,hig_val,True)
        self.__dlgChangeColorValue.exec()
        bIsOk=self.__dlgChangeColorValue.IsOk
        if not bIsOk:
            return
        new_color,is_change_color=self.__dlgChangeColorValue.Color
        new_value,is_change_value=self.__dlgChangeColorValue.Value
        new_icon_color=self.__createIcon(new_color)
        new_item=QListWidgetItem(new_icon_color,new_value)
        if sel_index==(len(listColorValue)-1):
            pass
        else:
            self.__ui.listViewColorInterval.insertItem(sel_index+1,new_item)
            self.__colorTab.addInterval(sel_index+1,new_value,new_color)
            #pr_value=self.__colorTab.getValueInProcent(float(new_value))
            #self.m.addColorItem(sel_index,new_color,new_value)
            self.m.update()
    def __color_item_sel(self):
        sel_index = self.__ui.listViewColorInterval.currentRow()
        if sel_index==(self.__ui.listViewColorInterval.count()-1):
            self.__ui.pb_add.setEnabled(False)
        else:
            self.__ui.pb_add.setEnabled(True)
    def __del_item_sel(self):
        sel_index = self.__ui.listViewColorInterval.currentRow()
        item=self.__ui.listViewColorInterval.takeItem(sel_index)
        item=None
        item=self.__colorTab.deleteItem(sel_index)
        item=None
        #self.m.deleteColorItem(sel_index)
        self.m.update()
    def __save_pal(self):
        ext_file='*.clr'
        ext_file = "Абсалютная (.clr);;Относительная (*.clr)"
        pathBase=os.path.realpath(__file__)
        pathFolderDefColorPallet=os.path.join(os.path.dirname(pathBase), "ColorPalettes")
        name_save_file= QFileDialog.getSaveFileName(self, 'Сохранить палитру', pathFolderDefColorPallet, ext_file)
        if not(name_save_file[0]==''):
            if name_save_file[1]=='Абсалютная (.clr)':
                self.__colorTab.savePalette(name_save_file[0],3)
            else:
                is_ok=self.__colorTab.savePalette(name_save_file[0])
    def __create_new_pal(self):
        listColorValue=self.__colorTab.colorMap
        color_start=listColorValue[0].Color
        count=len(listColorValue)
        color_end=listColorValue[count-1].Color
        #dlg_new_pal=DlgNewPalette(color_start,self.__zmin,color_end,self.__zmax,self)
        #dlg_new_pal.exec()
        #is_ok_dlg=dlg_new_pal.isOk
        dlg_new_pal=None
        is_ok_dlg=False
        if is_ok_dlg:
            min_value,min_color,max_value,max_color,count=dlg_new_pal.params
            #new_ColorMap=ColorMap(None)
            new_ColorMap=None
            new_ColorMap.createFromValue(0.0,min_color,100.0,max_color,count)
            self.m.setColorTab(new_ColorMap)
            self.m.update()
            self.__zmin=min_value
            self.__zmax=max_value
            self.__colorTab=None
            self.__colorTab=ColorTable(new_ColorMap)
            self.__colorTab.setZminZmax(self.__zmin,self.__zmax)
            self.__ui.listViewColorInterval.clear()
            self.__fillListColor()
            pass
    def show(self):
        self.__ui.show()
    def exec_(self):
        self.show()
        self.__ui.exec_()

