import colorsys
from math import fabs

from PySide2.QtGui import QColor
from osgeo import gdalconst


def valueToString(value,type_gdal):
    if type_gdal==gdalconst.GDT_Byte or type_gdal==gdalconst.GDT_Int16 or type_gdal==gdalconst.GDT_UInt16 or type_gdal==gdalconst.GDT_Int32 or type_gdal==gdalconst.GDT_UInt32:

        return '%i'%(value)
    return '%f'%(value)
def getLut(color_table,color_name,type_data,noValue):
    str_lut_values=''
    if noValue is not None:
        if noValue<color_table[0].value:
            str_lut_values=valueToString(noValue,type_data)+":"+valueToString(noValue,type_data)+","
    count=len(color_table)
    for i in range(count):
        item=color_table[i]
        str_item=valueToString(item.value,type_data)+':%i'%(item.colorValueByName(color_name))
        if i<(count-1):
            str_item=str_item+","
        str_lut_values=str_lut_values+str_item
    if noValue is not None:
        if noValue>color_table[count-1].value:
            str_lut_values=str_lut_values+','+valueToString(noValue,type_data)+":"+valueToString(noValue,type_data)
    return str_lut_values
class ItemColorMap:
    __val=None
    __R=None
    __G=None
    __B=None
    def __init__(self,value,r,g,b):
        if isinstance(value,str):
            self.__val=float(value)
        else:
            self.__val=value
        self.__color=[r,g,b]

        self.__R=r
        self.__G=g
        self.__B=b
    @property
    def getR(self):
        return self.__R
    @property
    def getG(self):
        return self.__G
    @property
    def getB(self):
        return self.__B
    @property
    def R(self):
        return self.__R
    @property
    def G(self):
        return self.__G
    @property
    def B(self):
        return self.__B
    @property
    def value(self):
        return self.__val
    def setValue(self,value):
        self.__val=value
    def clone(self):
        return ItemColorMap(self.value,self.__R,self.__G,self.__B)
    @property
    def Color(self):
        return QColor.fromRgb(self.__R,self.__G,self.__B)
    def setColor(self,color):
        self.__R=color.red()
        self.__G=color.green()
        self.__B=color.blue()
    def colorValueByName(self,name_color:str):
        if name_color.lower()=='red':
            return self.__R
        if name_color.lower()=='blue':
            return self.__B
        if name_color.lower()=='green':
            return self.__G
        return None
def changeValueTuple(source,index,value):
    temp_list=list(source)
    temp_list[index]=value
    return tuple(temp_list)
def LerpHSV(color_start, color_end, value_t):
    start_color=color_start
    end_color=color_end
    ''' Hue
    interpolation
    '''
    h=0

    d = start_color[0] - end_color[0]
    value=value_t
    '''
    if (a.h > b.h)
    {
        // Swap (a.h, b.h)
        var h3 = b.h2;
        b.h = a.h;
        a.h = h3;
        d = -d;
        t = 1 - t;
    }
    '''
    if start_color[0]>end_color[0]:
        h3=end_color[0]
        end_color=changeValueTuple(end_color,0,start_color[0])


        #start_color[0]=h3
        start_color=changeValueTuple(start_color,0,h3)
        d=-d
        value=1-value_t
    if d>0.5:
        ''' 180 deg'''
        start_color[0]=start_color[0]+1.0
        ''' ( a.h + t * (b.h - a.h) ) % 1 '''
        h=( start_color[0] + value * (end_color[0] - start_color[0]) ) % 1
    else:
        ''' h = a.h + t * d '''
        h=start_color[0]+fabs(value*d)
    s=start_color[1]+value*(end_color[1]-start_color[1])
    v=start_color[2]+value*(end_color[2]-start_color[2])
    return (h,s,v)

class ColorInterpolation:
    __max=100
    def __init__(self,color_start:ItemColorMap,color_end:ItemColorMap):
        self.__start_color=color_start
        self.__end_color=color_end
        self.__color_hsv_start=colorsys.rgb_to_hsv(color_start.R/255,color_start.G/255,color_start.B/255)
        self.__color_hsv_end = colorsys.rgb_to_hsv(color_end.R / 255, color_end.G / 255, color_end.B / 255)
    def interpolationForIndex(self,count):
        step=1/count
        list_colors=[]
        for i in range(count):
            color_hsv=LerpHSV(self.__color_hsv_start,self.__color_hsv_end,i*step)
            color_rgb=colorsys.hsv_to_rgb(color_hsv)
            list_colors.append(color_rgb)
        return list_colors
    def getColorRgbByValue(self,value):
        ''' Value в диапозоне от 0 до 1'''
        norm_value=(value-self.__start_color.value)/(self.__end_color.value-self.__start_color.value)
        #color_hsv = LerpHSV(self.__color_hsv_start, self.__color_hsv_end, value)
        color_hsv = LerpHSV(self.__color_hsv_start, self.__color_hsv_end,norm_value)
        color_rgb = colorsys.hsv_to_rgb(color_hsv[0],color_hsv[1],color_hsv[2])

        return color_rgb
