import re

from PySide2.QtGui import QColor, QImage
from coloraide import Color

from RasterGridTool.toolprocessing.ui.Palette.tools.ColorInterpolation import ColorInterpolation, ItemColorMap


def parserLineReg(line_inp):
    list_items=re.split(' |, |\*|\t',line_inp)
    list_out=[]
    for ln in list_items:
        tmp=ln.strip()
        if len(tmp)>0:
            list_out.append(tmp)
    return list_out
def createDefColorMap(array_line=None):
    if array_line is None:
        list_def_color=[]
        list_def_color.append('ColorMap 1 1 ')
        list_def_color.append('0   200 215 133')
        list_def_color.append('5   171 217 177')
        list_def_color.append('10  124 196 120')
        list_def_color.append('15  117 193 120')
        list_def_color.append('20  175 204 166')
        list_def_color.append('25  219 208  78')
        list_def_color.append('30  241 207  14 ')
        list_def_color.append('35  242 167   0')
        list_def_color.append('40  192 159  13')
        list_def_color.append('45  211 192 112')
        list_def_color.append('50  240 219 161')
        list_def_color.append('55  252 236 192')
        list_def_color.append('60  248 250 234')
        list_def_color.append('65  229 254 250')
        list_def_color.append('70  219 255 253')
        list_def_color.append('75  214 251 252 ')
        list_def_color.append('80  183 244 247')
        list_def_color.append('85  115 224 241')
        list_def_color.append('90   29 188 239')
        list_def_color.append('95    0 137 245')
        list_def_color.append('100   0  80 250')
        return ColorMap(list_def_color)
    return ColorMap(array_line)
def createGrayColor():
    list_def_color=[]
    list_def_color.append('ColorMap 1 1 ')
    list_def_color.append('0   255 255 255')
    list_def_color.append('100 0 0 0')
    return ColorMap(list_def_color)

class ColorMap:
    __version=1
    __interpMethod=1
    __source_path=None
    __val_min=None
    __val_max=None
    def __init__(self,path_file=None):
        if path_file is None:
            return

        self.__source_path=path_file
        if isinstance(path_file,list):
            self.__linesToColorMap(path_file)
            return
        if path_file:
            self.__readFileCrl(path_file)
    @property
    def InterpMethod(self):
        return self.__interpMethod
    def __readFileCrl(self,path_file):
        f_clr=open(path_file,'r')
        lines=f_clr.readlines()
        f_clr.close()
        if (lines is None ) or len(lines)==0:
            return
        self.__linesToColorMap(lines)
        return
        head_line=lines[0]
        keys=parserLine(head_line)
        if keys[0].lower()!='colormap':
            return
        self.__version=int(keys[1])
        self.__interpMethod=int(keys[2])
        count_line=len(lines)
        self.__colors=[]
        for i in range(1,count_line):
            line=lines[i]
            parserLineReg(line)
            values=parserLine(line)
            color_item=ItemColorMap(float(values[0]),int(values[1]),int(values[2]),int(values[3]))
            self.__colors.append(color_item)
            if self.__val_min:
                self.__val_min=min(self.__val_min,color_item.value)
            else:
                self.__val_min=color_item.value
            if self.__val_max:
                self.__val_max=max(self.__val_max,color_item.value)
            else:
                self.__val_max=color_item.value
        self.__countColors=len(self.__colors)
    def createFromItemColorMap(self,type,listItemsColor):
        self.__interpMethod=type
        self.__colors=[]
        for item_color in listItemsColor:
            self.__colors.append(item_color.clone())
        self.__countColors = len(self.__colors)
    def __linesToColorMap(self,lines):
        if (lines is None ) or len(lines)==0:
            return
        head_line=lines[0]

        keys=parserLineReg(head_line)
        if keys[0].lower()!='colormap':
            return
        self.__version=int(keys[1])
        self.__interpMethod=int(keys[2])
        count_line=len(lines)
        self.__colors=[]
        for i in range(1,count_line):
            line=lines[i]
            values=parserLineReg(line)
           # values=parserLine(line)
            color_item=ItemColorMap(float(values[0]),int(values[1]),int(values[2]),int(values[3]))
            self.__colors.append(color_item)
            if self.__val_min is not None:
                self.__val_min=min(self.__val_min,color_item.value)
            else:
                self.__val_min=color_item.value
            if self.__val_max is not None:
                self.__val_max=max(self.__val_max,color_item.value)
            else:
                self.__val_max=color_item.value
        self.__countColors = len(self.__colors)
    @property
    def ItemsColor(self):
        return self.__colors

    @property
    def getMinColorItem(self):
        return self.__colors[0]
    @property
    def getMaxColorItem(self):
        return self.__colors[self.__countColors-1]
    @property
    def values(self):
        return self.__colors
    @property
    def Count(self):
        return self.__countColors

class ColorTable:
    __map_color = None
    __zmin = None
    __zmax = None
    def __init__(self,color_map=None,zmin=None,zmax=None):
        if color_map is None:
            self.__map_color=createDefColorMap()
            return
        self.__map_color=color_map
        if zmin is not None:
            self.__zmin=zmin
        if zmax is not None:
            self.__zmax=zmax
        self.__prepareTable()
        self.__createColorInterpolation()
    def setZminZmax(self,zmin,zmax):
        self.__zmin=zmin
        self.__zmax=zmax
        self.__prepareTable()
        self.__createColorInterpolation()
    def initGray(self):
        self.__map_color=createGrayColor()
        #return self.__table_color.Count
    def __prepareTable(self):
        ''' curent Map Color '''
        #delta=self.__zmax-self.__zmin
        type_MapColor=self.__map_color.InterpMethod
        ''' Если type_MapColor =1  то относительная палитра'''
        if type_MapColor==1:
            delta_source_map=self.__map_color.getMaxColorItem.value-self.__map_color.getMinColorItem.value
            if self.__zmax is None and self.__zmin is None:
                delta_real_z=delta_source_map
                self.__zmin=self.__map_color.getMinColorItem.value
                self.__zmax=self.__map_color.getMaxColorItem.value
            else:
                delta_real_z=self.__zmax-self.__zmin
            dz=delta_real_z/delta_source_map
            count_interval=self.__map_color.Count
            self.__local_map_color_abs=[]
            for item_color in self.__map_color.ItemsColor:
                curent_abs_value=self.__zmin+item_color.value*dz
                abs_color_item=item_color.clone()
                abs_color_item.setValue(curent_abs_value)
                self.__local_map_color_abs.append(abs_color_item)
            return
        self.__cretateTableAbs(self.__zmin,self.__zmax)
    def __cretateTableAbs(self,zmin,zmax):
        self.__local_map_color_abs=[]
        zmin_clr=self.__map_color.getMinColorItem.value
        zmax_clr=self.__map_color.getMaxColorItem.value
        for item in self.__map_color.ItemsColor:
            self.__local_map_color_abs.append(item)
        if zmin<zmin_clr :
            new_min_color=self.__map_color.ItemsColor[0].clone()
            new_min_color.setValue(zmin)
            self.__local_map_color_abs.insert(0,new_min_color)


        if zmax>zmax_clr:

            new_max_color = self.__map_color.ItemsColor[len(self.__map_color.ItemsColor)-1].clone()
            new_max_color.setValue(zmax)
            self.__local_map_color_abs.append(new_max_color)


    @property
    def baseColorMap(self):
        return self.__map_color
    @property
    def colorMap(self):
        return self.__local_map_color_abs
    @property
    def Zmin(self):
        return self.__zmin
    @property
    def Zmax(self):
        return self.__zmax
    def __createColorInterpolation(self):
        list_aidsColor=[]
        format_color_rgb = "rgb({r} {g} {b} /1)"
        for item_color in self.__local_map_color_abs:

            str_color = format_color_rgb.format(r=item_color.R,
                                                      g=item_color.G,
                                                      b=item_color.B)
            list_aidsColor.append(Color(str_color))
        #list_aidsColor.append(Color(str_color))
        #self.__color_iterpolation=Color.interpolate(list_aidsColor,method='bspline')
        self.__color_iterpolation = Color.interpolate(list_aidsColor)
    def updateColors(self):
        self.__createColorInterpolation()
    def deleteItem(self,index):
        del self.__local_map_color_abs[index]
        self.updateColors()
    def getColor(self,value):
        ''' поиск интервала '''
        start_index=-1
        count_intervals=len(self.__local_map_color_abs)
        for i in range(count_intervals-1):
            if value>=self.__local_map_color_abs[i].value and value<self.__local_map_color_abs[i+1].value:
                start_index=i
                break
        if start_index<0:
            start_index=count_intervals-2
        cls_color=ColorInterpolation(self.__local_map_color_abs[start_index],self.__local_map_color_abs[start_index+1])
        #norm_value=(value-self.__zmin)/(self.__zmax-self.__zmin)
        color_rgb_by_value=cls_color.getColorRgbByValue(value)
        color=QColor.fromRgb(int(color_rgb_by_value[0]*255+0.5),int(color_rgb_by_value[1]*255+0.5),int(color_rgb_by_value[2]*255+0.5))
        #color.setRgb(color_rgb_by_value[0],color_rgb_by_value[1],color_rgb_by_value[2])
        return color
    def addInterval(self,index,value,color):
        new_item=ItemColorMap(value,color.red(),color.green(),color.blue())
        self.__local_map_color_abs.insert(index,new_item)
        self.updateColors()
    def getColorColorAideOld(self,value):
        start_index = -1
        count_intervals = len(self.__local_map_color_abs)
        for i in range(count_intervals - 1):
            if value >= self.__local_map_color_abs[i].value and value < self.__local_map_color_abs[i + 1].value:
                start_index = i
                break
        if start_index < 0:
            start_index = count_intervals - 2
        norm_value = (value - self.__zmin) / (self.__zmax - self.__zmin)
        format_color_rgb="rgb({r} {g} {b} /1)"
        str_color_start=format_color_rgb.format(r=self.__local_map_color_abs[start_index].R,g=self.__local_map_color_abs[start_index].G,b=self.__local_map_color_abs[start_index].B)
        color_aid_start=Color(str_color_start)
        str_color_end = format_color_rgb.format(r=self.__local_map_color_abs[start_index+1].R,
                                                  g=self.__local_map_color_abs[start_index+1].G,
                                                  b=self.__local_map_color_abs[start_index+1].B)
        color_aid_end = Color(str_color_end)
        color_iterpolation=Color.interpolate([color_aid_start,color_aid_end])
        norm_value=(value-self.__local_map_color_abs[start_index].value)/(self.__local_map_color_abs[start_index+1].value-self.__local_map_color_abs[start_index].value)
        color_rgb_by_value=color_iterpolation(norm_value)
        dict_color=color_rgb_by_value.to_dict()
        r=int(dict_color['coords'][0]*255+0.5)
        g = int(dict_color['coords'][1]*255 + 0.5)
        b = int(dict_color['coords'][2]*255 + 0.5)
        return QColor.fromRgb(r,g,b)

    def getColorColorAide(self,value):
        norm_value=(value-self.__zmin)/(self.__zmax-self.__zmin)
        color_rgb_by_value = self.__color_iterpolation(norm_value).convert("srgb")
        #srgb=color_rgb_by_value.convert("srgb")
        #dict_color = color_rgb_by_value.to_dict()
        #dict_color = srgb.to_dict()
        #str_color=color_rgb_by_value.to_string()
        #list_rgb=color_rgb_by_value.to_dict()['coords']
        r = int(color_rgb_by_value.to_dict()['coords'][0] * 255.0 + 0.5)
        #rf=color_rgb_by_value.to_dict()['coords'][0] * 255.0 + 0.5
        #r=int(rf)
        '''
        if r<0:
            ri=int(color_rgb_by_value.to_dict()['coords'][0] * 255)
            jkl=0
        '''
        if r<0:
            r=0
        if r>255:
            r=255
        g = int(color_rgb_by_value.to_dict()['coords'][1] * 255.0 + 0.5)
        if g<0:
            g=0
        if g>255:
            g=255

        b = int(color_rgb_by_value.to_dict()['coords'][2] * 255.0 + 0.5)
        if b<0:
            b=0
        if b>255:
            b=255
        #format_color_rgb = "value {v} rgb({r} {g} {b} /1)"
        #print(format_color_rgb.format(v=value,r=r,g=g,b=b))
        return QColor.fromRgb(r, g, b)
    def savePalette(self,path_out,type=1):
        ''' type =1 относительная палитра ( в процентах)'''
        if type==1:
            self.__saveClrRelative(path_out)
            return
        if type==3:
            self.__saveClrAbsolute(path_out)
    def __saveClrAbsolute(self,path_out):
        lines_out = []
        line_head = "ColorMap 1 {type}".format(type=3)
        lines_out.append(line_head + "\n")
        str_format = "{value} {r} {g} {b}"
        for i, item in enumerate(self.__local_map_color_abs):
            line_curent = str_format.format(value=item.value, r=item.R, g=item.G, b=item.B)
            lines_out.append(line_curent + "\n")
        with open(path_out, "w") as file:
            file.writelines(lines_out)
    def __saveClrRelative(self,path_out):
        zmin=zmax=None
        for item in self.__local_map_color_abs:
            if zmax is None and zmin is None:
                zmin=zmax=item.value
                continue
            zmin=min(zmin,item.value)
            zmax=max(zmax,item.value)
        delta=zmax-zmin
        lines_out=[]
        line_head="ColorMap 1 {type}".format(type=1)
        lines_out.append(line_head+"\n")
        str_format="{proc:.2} {r} {g} {b}"
        for i,item in enumerate(self.__local_map_color_abs):
            proc_value=((item.value-zmin)/delta)*100.0
            line_curent=str_format.format(proc=proc_value,r=item.R,g=item.G,b=item.B)
            lines_out.append(line_curent+"\n")
        with open(path_out, "w") as file:
            file.writelines(lines_out)
    def getQImageBaseClr(self,width,height):

        out_image= QImage(width, height, QImage.Format_ARGB32)
        dz = self.Zmax - self.Zmin
        pixel_value = dz / width
        for iw in range(width):
            cur_value = self.Zmin + iw * pixel_value

            curent_color=self.getColorColorAide(float(cur_value))
            for h_i in range(height):
                #out_image.setPixel(iw, h_i, curent_color)
                out_image.setPixelColor(iw,h_i,curent_color)


        return out_image
    def clacFromDefValue(self,min_value,max_value):
        typePalette=self.__map_color.InterpMethod
        itemsColors=self.__map_color.ItemsColor
        dh=(max_value-min_value)/100.0
        list_out_color_value=[]
        for itemColor in itemsColors:
            cur_proc_value=itemColor.value
            cur_value=min_value+cur_proc_value*dh
            cur_item_value=itemColor.clone()
            cur_item_value.setValue(cur_value)
            list_out_color_value.append(cur_item_value)
        return list_out_color_value










