import os
from pathlib import Path

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication
from axipy import provider_manager, TabFile, Layer
from axipy.app import Notifications
from osgeo import gdal


from RasterGridTool.toolprocessing.Utils.Observer import factoryObserverTabGrid, isDataForGridTab, isGridTab
from RasterGridTool.toolprocessing.ui.SlopeAspect.WidgetAspect import AspectWidget
from RasterGridTool.toolprocessing.ui.SlopeAspect.WidgetSlope import SlopeWidget
from RasterGridTool.toolprocessing.ui.SlopeAspect.tool.aspectSlopeTool import AspectSlope
from RasterGridTool.toolprocessing.ui.addTools.utils import RasterIsOpen
from RasterGridTool.toolprocessing.ui.viewshade.tools.helper_raster import getClassAxiGrid


class DlgAspectSlope(QWidget):
    cancelProcess = QtCore.Signal()
    def __init__(self,type,parent=None):
        super(DlgAspectSlope, self).__init__(parent)
        self.__type_calc=type
        self.__parentWin=parent
        self.__loadUI()
        if type==0:
            self.__params_widget=AspectWidget(self)
            self.__name_method="aspect"
        else:
            self.__params_widget=SlopeWidget(self)
            self.__name_method="углы наклона"
        self.window.vrt_params.addWidget(self.__params_widget.window)
        axipy.da.data_manager.removed.connect(self.__removeTab)
        axipy.da.data_manager.added.connect(self.__addTable)
        self.window.cmb_source.currentIndexChanged.connect(self.__change_grid_tab)
        self.window.pb_run.clicked.connect(self.__run)
        self.window.pb_cancel.clicked.connect(self.__cancel)
        self.__process=None
    def __loadUI(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "AspectSlope.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window = loader.load(ui_file, self.__parentWin)
        ui_file.close()
    def __addTable(self,name:str):
        data_obj=axipy.data_manager.find(name)
        if not isGridTab(data_obj):
            return
        try:
            index=self.window.cmb_source.findText(name)
            if index<0:
                self.window.cmb_source.addItem(name)
        except:
            pass
    def __removeTab(self,tab_name):
        data_obj=axipy.data_manager.find(tab_name)
        if not isGridTab(data_obj):
            return
        try:
            index=self.window.cmb_source.findText(tab_name)
            if index>=0:
                self.window.cmb_source.removeItem(index)
        except:
            pass
    def updatTable(self,tabs_for_grid_s=None):
        tabs_for_grid=tabs_for_grid_s
        if tabs_for_grid_s is None:
            tabs_for_grid=tabs_for_grid = list(filter(isGridTab, axipy.da.data_manager.objects))
        #tabs_for_grid = list(filter(isGridTab, axipy.da.data_manager.objects))
        if len(tabs_for_grid)==0:
            self.window.cmb_source.clear()
            self.window.pb_run.setEnabled(False)
            return
        self.window.cmb_source.clear()
        for tab in tabs_for_grid:
            self.window.cmb_source.addItem(tab.name)
    def __change_grid_tab(self,index):
        name_grid=self.window.cmb_source.currentText()
        tab_grid=axipy.data_manager.find(name_grid)
        if tab_grid is None:
            return
        path_grid=tab_grid.properties['fileName']
        path_source=Path(path_grid)
        folder=str(path_source.parent)
        name_file=str(path_source.stem)
        prefix="aspect"

        if self.__type_calc==1:
            prefix='slope'
        out_path=os.path.join(folder,name_file+"_"+prefix+".tif")
        self.window.out_tab_file.setText(out_path)
        self.window.out_tab_file.setCursorPosition(0)
        self.window.pb_run.setEnabled(True)
        jkl=0
    def __run(self):
        source_tab=self.window.cmb_source.currentText()
        axiTable=getClassAxiGrid(source_tab)
        self.window.pb_run.setEnabled(False)
        self.window.pb_cancel.setEnabled(True)
        path_dest=self.window.out_tab_file.text()
        tab_source=axipy.data_manager.find(source_tab)

        path_source=tab_source.properties['fileName']
        tabExist=RasterIsOpen(path_dest)
        if tabExist is not None:
            nameClose=tabExist.name
            tabExist.close()
            Notifications.push('Вычисление aspect(slope)', 'Закрыта т.к создается новый файл с таким же именем '+nameClose, Notifications.Warning)
        self.window.progres_bar.setFormat("Вычисление "+self.__name_method+" .. %p%")
        demOptions=None
        params=self.__params_widget.getParams()
        params['band']=axiTable.IdBand
        if self.__process is None:
            self.__process=AspectSlope("Вычисление aspect/slop",self)
            self.__process.updateProcess.connect(self.__update_progress)
            self.__process.finished.connect(self.__end_process)

        self.__process.setParametrs(path_source,self.window.out_tab_file.text(),self.__type_calc,params)
        self.__process.run()
    def __update_progress(self, value):
        self.window.progres_bar.setValue(value)
        # print(value)
        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __end_process(self, value):
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
        self.window.progres_bar.setValue(0)
        if not self.__process.isCancel:
        #if value == 100:

            path_out=self.__process.outPath
            tab_grid=provider_manager.gdal.open(path_out)
            tab = TabFile()
            out_file_name=os.path.join(str(Path(path_out).parent),str(Path(path_out).stem)+".tab")
            tab.generate_tab(tab_grid, out_file_name)
            layer =Layer.create(tab_grid)
            axipy.app.mainwindow.add_layer_interactive(layer)
        else:
            path_out=self.__process.outPath
            if os.path.isfile(path_out):
                os.remove(path_out)
    def __cancel(self):
        self.cancelProcess.emit()
        #self.__processGrid.setCancel()
    def setFormatprogressBar(self,str_format):
        self.window.pr_bar.setFormat(str_format)
    @property
    def widget(self):
        return self.window
