import os
import time
from pathlib import Path

import axipy
from PySide2.QtCore import QFile, Signal
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QFileDialog, QApplication
from axipy import data_manager, Notifications

from RasterGridTool.toolprocessing.Utils.helper_observer import existOneNumberField
from RasterGridTool.toolprocessing.ui.Stamp.toolprocessing.stamp_process import StampProcess


class StampForm(QWidget):
    cancelProcess = Signal()
    name_observer = 'GridAxi'

    def __init__(self, parent=None):
        self.__parent = parent
        super(StampForm, self).__init__(parent)
        self.__parentWin = parent
        self.load_ui('stamp_1.ui')
        curent_path = str(Path(__file__).parent)
        path_icon = os.path.join(curent_path, "icons")
        path_add_png = os.path.join(path_icon, "saveas.png")
        self.ui.pb_saveAs.setIcon(QIcon(path_add_png))
        self.ui.cmb_vectot_tab.currentIndexChanged.connect(self.__change_table)
        self.ui.cmb_rasters.currentIndexChanged.connect(self.__change_raster)
        self.ui.cmb_vector_tabfield.currentIndexChanged.connect(self.__change_method)
        self.ui.radio_replace.toggled.connect(self.__change_method)
        self.ui.pb_saveAs.clicked.connect(self.__selectSaveGrid)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.pb_cancel.clicked.connect(self.__cancel)
        # self.__initliatTab()

    def load_ui(self, name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.ui = loader.load(ui_file, self.__parentWin)

        ui_file.close()

    def __initliatTab(self):
        self.ui.cmb_vectot_tab.clear()
        for tab in data_manager.tables:
            existNuberField = existOneNumberField(tab)
            if not existNuberField:
                continue
            self.ui.cmb_vectot_tab.addItem(tab.name)
        self.ui.cmb_vectot_tab.setCurrentIndex(0)

    def __initGridData(self):

        grid_observer = axipy.ObserverManager.get(self.name_observer)
        list_grid = grid_observer.value
        if not list_grid:
            return
        self.ui.cmb_rasters.clear()
        for name in list(list_grid.keys()):
            self.ui.cmb_rasters.addItem(name)
        self.ui.cmb_rasters.setCurrentIndex(0)
    def __getRasterPath(self,name_raster):
        grid_observer = axipy.ObserverManager.get(self.name_observer)
        list_grid = grid_observer.value
        if not list_grid:
            return
        axiRaster=list_grid[name_raster]
        return axiRaster.pathRaster
    def __getAxiRaster(self,name_raster):
        grid_observer = axipy.ObserverManager.get(self.name_observer)
        list_grid = grid_observer.value
        if not list_grid:
            return None
        axiRaster = list_grid[name_raster]
        return axiRaster
    def __change_table(self):
        if self.ui.cmb_vectot_tab.count()==0:
            return
        name_sel_tab=self.ui.cmb_vectot_tab.currentText()

        self.ui.cmb_vector_tabfield.clear()
        tab_sel=data_manager.find(name_sel_tab)
        for att in tab_sel.schema:
            #self.ui.cmb_index_field.addItem(att.name)
            if att.type_string == "int" or att.type_string == 'double':
                self.ui.cmb_vector_tabfield.addItem(att.name)
        #self.ui.cmb_index_field.setCurrentIndex(0)
        self.ui.cmb_vector_tabfield.setCurrentIndex(0)
        #self.__updateOutPath()
    def __change_method(self):
        self.__updateOutPath()
    def __change_raster(self):
        self.__updateOutPath()
        pass
    def __selectSaveGrid(self):
        ext_list = "Gpkg (*.gpkg);;Tif (.tif)"
        path_source=self.ui.ln_out_raster.text()
        ext_raster=str(Path(path_source).suffix)
        if ext_raster.lower()=='.tif':
            ext_list = "Tif (.tif);;Gpkg (*.gpkg)"
        fileNameGrid=QFileDialog.getSaveFileName(axipy.app.mainwindow.qt_object(),"Сохранить файл поверхность",path_source,ext_list)
        if fileNameGrid[0]=='':
            return
        self.ui.ln_out_raster.setText(fileNameGrid[0])
        self.ui.ln_out_raster.setCursorPosition(0)
    def __run(self):
        params={}
        params['raster']=self.__getAxiRaster(self.ui.cmb_rasters.currentText())
        params['source_vector'] =self.ui.cmb_vectot_tab.currentText()
        params['source_vector_field'] =self.ui.cmb_vector_tabfield.currentText()
        params['method']='REPLACE'
        if self.ui.radio_sum.isChecked():
            params['method'] = 'ADD'
        params['result_file']=self.ui.ln_out_raster.text()
        self.__process=StampProcess(params,self)
        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.finished.connect(self.__end_process)
        self.__process.updateFormatProgressBar.connect(self.__setFormatprogressBar)
        self.__process.run()
    def __cancel(self):
        pass
    def __setFormatprogressBar(self, str_format_main):
        self.ui.prog_bar.setFormat(str_format_main)

        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __update_progress(self,value_main):

        self.ui.prog_bar.setValue(value_main)

            # print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def __end_process(self, value):
        self.ui.pb_cancel.setEnabled(False)
        self.ui.pb_run.setEnabled(True)
        pathOutRaster=self.__process.pathOutRaster()
        #name_out_tab=self.__process.nameOutTab
        tab_out=axipy.provider_manager.gdal.open(pathOutRaster)
        if tab_out is not None:
            Notifications.push('Штамп полигонов', 'Создан растр '+pathOutRaster, Notifications.Information)
        self.__process.updateProcess.disconnect()
        self.__process.finished.disconnect()
        self.__process.updateFormatProgressBar.disconnect()
        self.__process=None
    def __updateOutPath(self):

        name_field_stamp=self.ui.cmb_vector_tabfield.currentText()
        type_stamp="st_replace"
        if self.ui.radio_sum.isChecked():
            type_stamp = "st_add"
        type_stamp=type_stamp+"_"+name_field_stamp
        name_sel_tab_raster = self.ui.cmb_rasters.currentText()
        if name_sel_tab_raster=='':
            return
        path_raster=self.__getRasterPath(name_sel_tab_raster)

        raster_path=Path(path_raster)
        ext_file=str(raster_path.suffix)
        if ext_file.lower()==".vrt":
            ext_file=".tif"
        folder_path = raster_path.parent
        path_out_tab = str(folder_path.joinpath(name_sel_tab_raster + "_"+type_stamp+str(ext_file)))
        self.ui.ln_out_raster.setText(path_out_tab)
        self.ui.ln_out_raster.setCursorPosition(0)
    @property
    def widget(self):
        return self.ui

    def updateGrid(self):
        self.__initGridData()

    def updateSource(self):
        self.__initliatTab()

    def show(self):
        # self.ui.showMaximized()
        # print("Show Form")
        self.__initliatTab()
        self.__initGridData()
        self.ui.show()

