import time
from pathlib import Path

from PySide2.QtCore import QObject, Signal
from axipy import Notifications, data_manager
from osgeo import gdal, ogr


class StampProcess(QObject):
    updateProcess = Signal(float)
    updateFormatProgressBar = Signal(str)
    finished = Signal(int)
    def __init__(self,params,parent_cls):

        super(StampProcess, self).__init__()
        self.__parent_cls=parent_cls
        self.__params=params
        self.__old_progress=0
    def __process_callback(self, complete, message=None, user_data=None):
        # print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(self.__old_progress+complete * 50)
        time.sleep(0.001)
        # self.__parent.window.pr_bar.setValue(complete*100)
        # print(complete*100)
        return 1
    def __isCancel(self):
        self.__cancelProcess=True

    def setCancel(self):
        self.__cancelProcess = True
    def pathOutRaster(self):
        return self.__path_out_raster
    def run(self):
        self.__path_out_raster=None
        axi_raster=self.__params['raster']
        path_source_raster=axi_raster.pathRaster
        idBandGrid = axi_raster.IdBand
        if axi_raster.isVrt:
            path_source_raster=axi_raster.rasterSource.vrtBaseGrid[0]['file']
            path=Path(path_source_raster)
            folder=path.parent
            if str(folder)=='.':
                path_source_raster=str(Path(axi_raster.pathRaster).parent.joinpath(path_source_raster))
            idBandGrid=axi_raster.rasterSource.vrtBaseGrid[0]['band']
        source_tab=self.__params['source_vector']
        field_for_stamp=self.__params['source_vector_field']
        method_stamp=self.__params['method']
        path_result_raster=self.__params['result_file']
        self.__cancelProcess = False
        try:
            self.__parent_cls.cancelProcess.disconnect()
        except:
            pass
        self.__parent_cls.cancelProcess.connect(self.__isCancel)
        self.updateFormatProgressBar.emit("Штамп вектор - растр .. %p%")
        ''' Копирование исходного растра(грида)'''
        ds_source=gdal.Open(path_source_raster)

        if ds_source is None:
            self.__cancelProcess=True
            self.updateProcess.emit(0)
            self.finished.emit(100)
            Notifications.push('Штамп', 'Ошибка открытия ' + axi_raster.pathRaster, Notifications.Critical)
            return
        outputData = ds_source.GetDriver().CreateCopy(path_result_raster, ds_source, 0,callback =self.__process_callback)
        if self.__cancelProcess:
            ds_source = None
            outputData = None
            self.finished.emit(100)
            return
        ds_source=None
        self.__old_progress=50
        table=data_manager.find(source_tab)
        path_tab=table.properties['fileName']
        vector_ds=ogr.Open(path_tab)
        layer=vector_ds.GetLayer()
        opt=[]
        opt.append("ATTRIBUTE="+field_for_stamp)
        opt.append("ALL_TOUCHED=FALSE")
        opt.append("MERGE_ALG=" + method_stamp)
        gdal.RasterizeLayer(outputData, [idBandGrid], layer,
                            options=opt,callback =self.__process_callback)
        layer=None
        vector_ds=None
        outputData=None
        self.__path_out_raster=path_result_raster
        self.updateProcess.emit(100)
        self.finished.emit(100)
        return


