import os
import time
from pathlib import Path

import axipy
from PySide2.QtCore import QFile, Signal, QThread
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication, QFileDialog
from axipy import data_manager, Notifications

from RasterGridTool.toolprocessing.ui.ZoneStatistic.tools.cls_process import ProcessStatisticZone


class FormStatistic(QWidget):
    cancelProcess = Signal()

    name_observer = 'GridAxi'
    def __init__(self,parent=None):

        super(FormStatistic, self).__init__(parent)
        self.__parentWin=parent
        self.load_ui('formStatistic_1.ui')
        curent_path = str(Path(__file__).parent)
        path_icon = os.path.join(curent_path, "icons")
        path_add_png = os.path.join(path_icon, "saveas.png")
        self.ui.pb_saveAs.setIcon(QIcon(path_add_png))
        self.ui.pb_saveAs.clicked.connect(self.__SaveTab)
        self.ui.cmb_vectot_tab.currentIndexChanged.connect(self.__updateOutPath)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.pb_cancel.clicked.connect(self.__cancel)
    def load_ui(self, name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.ui = loader.load(ui_file, self.__parentWin)

        ui_file.close()
    def __initliatTab(self):
        self.ui.cmb_vectot_tab.clear()
        for tab in data_manager.tables:
            if not tab.is_spatial:
                continue
            self.ui.cmb_vectot_tab.addItem(tab.name)
        self.ui.cmb_vectot_tab.setCurrentIndex(0)
    def __initGridData(self):

        grid_observer=axipy.ObserverManager.get(self.name_observer)
        list_grid=grid_observer.value
        if not list_grid:
            return
        self.ui.cmb_rasters.clear()
        for name in list(list_grid.keys()):
            self.ui.cmb_rasters.addItem(name)
        self.ui.cmb_rasters.setCurrentIndex(0)

    def __updateOutPath(self):
        name_sel_tab = self.ui.cmb_vectot_tab.currentText()
        tab_sel = data_manager.find(name_sel_tab)
        if tab_sel is None:
            return 
        if isinstance(tab_sel,axipy.QueryTable) or isinstance(tab_sel,axipy.SelectionTable):
            path_folder=Path.home()
        else:
            file_name = tab_sel.properties['fileName']
            path_folder = Path(file_name).parent
        name_out=tab_sel.name+"__statistic_polygons.tab"
        out_path=str(path_folder.joinpath(name_out))
        self.ui.ln_out_tab.setText(out_path)
        self.ui.ln_out_tab.setCursorPosition(0)

    def __end_process(self, value):
        self.ui.pb_cancel.setEnabled(False)
        self.ui.pb_run.setEnabled(True)
        name_out_tab=self.__process.nameOutTab
        tab_out=data_manager.find(name_out_tab)
        if tab_out is not None:
            Notifications.push('Статистика полигонов', 'Создана таблица '+name_out_tab, Notifications.Information)
        self.__process.updateProcess.disconnect()
        self.__process.finished.disconnect()
        self.__process.updateFormatProgressBar.disconnect()
        self.__process=None
    def __run(self):
        params = {}
        params['tab_source']=self.ui.cmb_vectot_tab.currentText()
        name_raster = self.ui.cmb_rasters.currentText()
        grid_observer = axipy.ObserverManager.get(self.name_observer)
        list_grid = grid_observer.value
        axiGrid = list_grid[name_raster]
        params['grid']=axiGrid
        params['out_tab']=self.ui.ln_out_tab.text()
        self.__process=ProcessStatisticZone(params,self)
        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.updateFormatProgressBar.connect(self.__setFormatprogressBar)
        self.__process.finished.connect(self.__end_process)
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        self.ui.pb_cancel.setEnabled(True)
        self.ui.pb_run.setEnabled(False)
        self.__process.run()
    def __setFormatprogressBar(self, str_format_main):
        self.ui.prog_bar.setFormat(str_format_main)

        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __update_progress(self,value_main):

        self.ui.prog_bar.setValue(value_main)

            # print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def setFormatprogressBar(self, str_format_main):
        self.ui.prog_bar.setFormat(str_format_main)

        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __cancel(self):
        self.__process_status=False
        self.cancelProcess.emit()
        self.__process.setCancel()
    def __SaveTab(self):
        ext_list = "MapInfo Tab (*.tab)"
        path_source=self.ui.ln_out_tab.text()

        fileNameGrid=QFileDialog.getSaveFileName(axipy.app.mainwindow.qt_object(),"Сохранить MapInfo Tab",path_source,ext_list)
        if fileNameGrid[0]=='':
            return
        self.ui.ln_out_tab.setText(fileNameGrid[0])
        self.ui.ln_out_tab.setCursorPosition(0)
    @property
    def widget(self):
        return self.ui
    def updateGrid(self):
        self.__initGridData()
    def updateSource(self):
        self.__initliatTab()
    def show(self):
        # self.ui.showMaximized()
        # print("Show Form")
        self.__initliatTab()
        self.__initGridData()
        self.ui.show()