import time

from PySide2.QtCore import QObject, Signal
from axipy import Table, Attribute, provider_manager, data_manager, Polygon, MultiPolygon, Feature

from RasterGridTool.toolprocessing.ui.ZoneStatistic.tools.clsZoneStatistics import CalcZoneStatistics
def convertToFeature(data_ft):
    geometry=data_ft['+geometry']
    style=data_ft['+style']
    del data_ft['+geometry']
    del data_ft['+style']
    ft=Feature(data_ft,geometry,style)
    return ft
class TableCache:
    def __init__(self,out_tab,size_cache=100):
        self.__table=out_tab
        self.__size_cache=size_cache
        self.__cache=[]
    def insert(self,ft_insert):
        if isinstance(ft_insert,list):
            for data_ft in ft_insert:
                self.__cache.append(convertToFeature(data_ft))

        else:
            self.__cache.append(convertToFeature(ft_insert))
        if len(self.__cache)>self.__size_cache:
            self.__table.insert(self.__cache)
            self.__cache.clear()
    def end(self):
        if len(self.__cache) > 0:
            self.__table.insert(self.__cache)
            self.__cache.clear()

def existAttributeByName(schema,name_att):
    if not (schema.index_by_name(name_att) >= 0):
        return False
    return True
def createOutTab(tab_source:Table,path_new_tab):
    dest_schema = tab_source.schema
    if not existAttributeByName(dest_schema,'value_average'):
        dest_schema.append(Attribute.float('value_average'))
    if not existAttributeByName(dest_schema, 'value_mean'):
        dest_schema.append(Attribute.float('value_mean'))
    if not existAttributeByName(dest_schema, 'value_median'):
        dest_schema.append(Attribute.float('value_median'))
    if not existAttributeByName(dest_schema, 'value_min'):
        dest_schema.append(Attribute.float('value_min'))
    if not existAttributeByName(dest_schema, 'value_max'):
        dest_schema.append(Attribute.float('value_max'))
    if not existAttributeByName(dest_schema, 'value_count'):
        dest_schema.append(Attribute.integer('value_count'))

    dest_tab = provider_manager.tab.create_open(path_new_tab, dest_schema)
    return dest_tab

class ProcessStatisticZone(QObject):
    updateProcess = Signal(float)
    updateFormatProgressBar = Signal(str)
    finished = Signal(int)
    max_task=8
    def __init__(self,params,parent_cls):

        super(ProcessStatisticZone, self).__init__()
        self.__parent_cls=parent_cls
        self.__params=params

    def __process_callback(self, complete, message=None, user_data=None):
        # print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(complete * 100)
        time.sleep(0.001)
        # self.__parent.window.pr_bar.setValue(complete*100)
        # print(complete*100)
        return 1
    def __isCancel(self):
        self.__cancelProcess=True

    def setCancel(self):
        self.__cancelProcess = True
    @property
    def nameOutTab(self):
        return self.__name_out_tab
    def run(self):
        self.__cancelProcess = False
        try:
            self.__parent_cls.cancelProcess.disconnect()
        except:
            pass
        try:
            self.__parent_cls.cancelProcess.connect(self.__isCancel)
            self.__cancelProcess = False
            name_source_tab=self.__params['tab_source']
            tab_source=data_manager.find(name_source_tab)
            raster_source=self.__params['grid']
            path_raster=raster_source.pathRaster
            path_out_file=self.__params['out_tab']
            out_tab=createOutTab(tab_source,path_out_file)
            self.__name_out_tab=out_tab.name
            cls_statistics=CalcZoneStatistics(path_raster,raster_source.IdBand)
            cache_out_tab=TableCache(out_tab)
            data_polygons=[]
            data_no_polygons=[]
            self.updateFormatProgressBar.emit("Расчет статистики полигонов .. %p%")
            count=tab_source.count()
            for i,ft in enumerate(tab_source.items()):
                is_ok = self.__process_callback(i / count)
                if is_ok == 0:
                    break
                if not (isinstance(ft.geometry,Polygon) or isinstance(ft.geometry,MultiPolygon)):
                    cache_out_tab.insert(dict(ft.items()))
                    continue
                data_polygons.append(dict(ft.items()))
                if len(data_polygons)>self.max_task:
                    list_result=cls_statistics.calcStatisticsList(data_polygons)
                    cache_out_tab.insert(list_result)
                    data_polygons.clear()
            if len(data_polygons) > 0:
                list_result = cls_statistics.calcStatisticsList(data_polygons)
                cache_out_tab.insert(list_result)
            cache_out_tab.end()
            cls_statistics.closeDs()
        except:
            pass
        self.updateProcess.emit(100)
        self.finished.emit(100)
    def run_pool(self):
        name_source_tab=self.__params['tab_source']
        tab_source=data_manager.find(name_source_tab)
        raster_source=self.__params['grid']
        path_raster=raster_source.pathRaster
        path_out_file=self.__params['out_tab']
        out_tab=createOutTab(tab_source,path_out_file)
        cls_statistics=CalcZoneStatistics(path_raster)
        cache_out_tab=TableCache(out_tab)
        data_polygons=[]
        data_no_polygons=[]
        for i,ft in enumerate(tab_source.items()):
            if not (isinstance(ft.geometry,Polygon) or isinstance(ft.geometry,MultiPolygon)):
                cache_out_tab.insert(dict(ft.items()))
                continue
            data_polygons.append(dict(ft.items()))
            if len(data_polygons)>self.max_task:
                list_result=cls_statistics.calcStatisticsListPool(data_polygons)
                cache_out_tab.insert(list_result)
                data_polygons.clear()
        if len(data_polygons) > 0:
            list_result = cls_statistics.calcStatisticsListPool(data_polygons)
            cache_out_tab.insert(list_result)
        cache_out_tab.end()
