import os
import time
from pathlib import Path

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QDialog, QFileDialog, QApplication
from axipy import ChooseCoordSystemDialog, provider_manager, TabFile, SelectionTable, data_manager, CoordSystem

from RasterGridTool.toolprocessing.ui.clipRaster.tool.axiomaToGdal import getCoordSysCosmeticTab, \
    createTempShpFromRaster
from RasterGridTool.toolprocessing.ui.clipRaster.tool.rasterTool import TypeInterpoly, RasterProcessClip, \
    findTabRasterInPath


class FromClipRaster(QWidget):
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        super(FromClipRaster, self).__init__(parent)
        self.__parentWin=parent
        self.__loadUI()
        self.__out_file_raster=None
        self.window.cmb_raster_tab.currentIndexChanged.connect(self.__change_table)
        self.window.pb_change_out_cs.clicked.connect(self.__change_out_cs)
        self.updateTables()
        self.__change_selection()
        self.window.pb__select_out_file.clicked.connect(self.__change_out_file)
        ''' '''
        axipy.gui.selection_manager.changed.connect(self.__change_selection)
        axipy.da.data_manager.updated.connect(self.updateSource)
        self.window.pb_run.clicked.connect(self.__run)
        self.window.pb_cancel.clicked.connect(self.__cancel)

    def __loadUI(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "DlgClipRaster.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window  = loader.load(ui_file,self.__parentWin)
        ui_file.close()
    def updateSource(self):
        self.updateTables()
    def updateTables(self,clare=True):
        all_obj=axipy.da.data_manager.all_objects
        try:
            self.window.cmb_raster_tab.clear()

            for obj_data in all_obj:
                if  isinstance(obj_data, axipy.Raster):
                    self.window.cmb_raster_tab.addItem(obj_data.name)
            if self.window.cmb_raster_tab.count()>1:
                index=self.window.cmb_rej_clip.findText("Raster")
                if index<0:
                    self.window.cmb_rej_clip.addItem("Raster")
        except:
            return
    def __change_table(self,index):
        name_tab=self.window.cmb_raster_tab.currentText()
        curent_table=axipy.da.data_manager.find(name_tab)
        if curent_table is None:
            return
        path_raster=curent_table.properties['fileName']
        try:
            path_tab=curent_table.properties['tabFile']
        except:
            path_tab=path_raster
        base_path=str(Path(path_tab).parent)
        name_tab=Path(path_tab).stem
        ext_raster_file=Path(path_raster).suffix
        if not (ext_raster_file==".gpkg" or ext_raster_file==".tif" or ext_raster_file==".png" or ext_raster_file==".jpg"):
            ext_raster_file=".tif"
        if ext_raster_file==".gpkg" or ext_raster_file==".tif":
            self.window.gr_box_out_options.setEnabled(True)
        out_file_tab=os.path.join(base_path,name_tab+"_clip"+ext_raster_file)
        self.window.out_tab_file.setText(out_file_tab)
        self.window.out_tab_file.setCursorPosition(0)
        self.__out_file_raster=out_file_tab
        self.__out_cs=curent_table.coordsystem
        self.__out_cs_mi=curent_table.coordsystem.prj
        self.window.ln_coordSys.setText(self.__out_cs.name)
        self.window.ln_coordSys.setCursorPosition(0)
        self.__process=RasterProcessClip("Отсечение",self)
        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.finished.connect(self.__end_process)
    def __change_out_cs(self):
        dialog_cs = ChooseCoordSystemDialog(self.__out_cs)
        if dialog_cs.exec() == QDialog.Accepted:
            self.__out_cs = dialog_cs.chosenCoordSystem()
            self.__out_cs_mi=self.__out_cs.prj
            self.window.ln_coordSys.setText(self.__out_cs.name)
            self.window.ln_coordSys.setCursorPosition(0)
    def __change_selection(self):
        '''
        Изменение selection
        Если selection >0 то добавлем в области отсечения selection
        '''
        try:
            index=self.window.cmb_rej_clip.findText("Selection")
            if axipy.gui.selection_manager.count>0:

                if index<0:
                    self.window.cmb_rej_clip.addItem("Selection")
                    self.window.gr_box_obj_clip.setEnabled(True)
            else:
                if index>=0:
                    self.window.cmb_rej_clip.removeItem(index)
                if self.window.cmb_rej_clip.count()==0:
                    self.window.gr_box_obj_clip.setEnabled(False)
        except:
            pass
    def __change_out_file(self):
        name_out_file=self.window.out_tab_file.text()
        ext_list="GPKG (*.gpkg);;GeoTiff (*.tif);;PNG (*.png);;JPEG (*.jpg)"
        name_load_file = QFileDialog.getSaveFileName(axipy.app.mainwindow.qt_object(), 'Файл результата',name_out_file, ext_list)
        if not(name_load_file[0] ==''):
            self.__out_file_raster=name_load_file[0]
            if Path(self.__out_file_raster).suffix==".gpkg" or Path(self.__out_file_raster).suffix==".tif":
                self.window.gr_box_out_options.setEnabled(True)
            else:
                self.window.gr_box_out_options.setEnabled(False)
            self.window.out_tab_file.setText(self.__out_file_raster)
    def __path_bound_clip(self):
        ''' Путь на временный файл с границами отсечения'''
        if self.window.gr_box_obj_clip.isEnabled():
            ''' Область отсечения вектор из selection или bound растра'''
            name_source_raster = self.window.cmb_raster_tab.currentText()
            tab_raster=data_manager.find(name_source_raster)
            if self.window.cmb_rej_clip.currentText()=="Selection":
                #table_base=axipy.gui.selection_manager.table
                table_base=axipy.data_manager.selection
                name_table=table_base.name
                '''
                if isinstance(table_base,SelectionTable):
                    jkl=0
                if name_table.find("cosmetic")==0 and table_base.is_temporary:
                    cs_base=getCoordSysCosmeticTab(name_table)
                else:
                    cs_base=table_base.coordsystem
                '''
                cs_base=tab_raster.coordsystem
                type_tab=table_base.is_temporary
                from .tool.vectorTool import TempVector
                temp_clip=TempVector()
                temp_clip.create(cs_base)
                temp_clip.copy(table_base.items())
                if temp_clip.count==0:
                    temp_clip.close()
                    name_source_raster=self.window.cmb_raster_tab.currentText()
                    return createTempShpFromRaster(name_source_raster)
                return temp_clip
            else:
                pass

        else:
            ''' Отсечение по границам исходного растра'''
            name_source_raster=self.window.cmb_raster_tab.currentText()
            return createTempShpFromRaster(name_source_raster)
    def __run(self):
        ''' Выполняем операцию'''
        ''' Ajhvbhetv clip poly'''
        self.__out_file_raster= self.window.out_tab_file.text()
        cls_temp_clip=self.__path_bound_clip()
        table_raster=self.window.cmb_raster_tab.currentText()
        typeAlgoritm=TypeInterpoly(self.window.cm_alg_interpol.currentIndex())

        cs_out=self.__out_cs

        #cs_out=CoordSystem.from_prj(self.__out_cs_mi)
        isOpenRaster,name_tab=findTabRasterInPath(self.__out_file_raster)
        if isOpenRaster:
            tab_exist=axipy.da.data_manager.find(name_tab)
            tab_exist.close()
        ext_out=Path(self.__out_file_raster).suffix
        out_tiles=False
        out_compress=False
        if Path(self.__out_file_raster).suffix==".gpkg" or Path(self.__out_file_raster).suffix==".tif":
            if self.window.gr_box_out_options.isEnabled():
                out_tiles=self.window.chb_tiles.isChecked()
                out_compress=self.window.chb_compress.isChecked()
        self.__params_clip={}
        self.__params_clip['clip_region']=cls_temp_clip
        self.__params_clip["source_tab"]=table_raster
        self.__params_clip['method']=typeAlgoritm
        self.__params_clip['out_cs']=cs_out
        self.__params_clip['out_cs_mi'] = self.__out_cs_mi
        self.__params_clip['out_file']=self.__out_file_raster
        self.__params_clip['out_opthions']={}
        self.__params_clip['out_opthions']['tiles']=out_tiles
        self.__params_clip['out_opthions']['compression']=out_compress
        self.__process.setParams(self.__params_clip)
        self.window.pb_cancel.setEnabled(True)
        self.window.pb_run.setEnabled(False)
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        self.__process.run()
    def __update_progress(self,value):
        self.window.progres_bar.setValue(value)
        #print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def __end_process(self,value):
        self.window.progres_bar.setValue(0)

        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)

    def __cancel(self):
        self.cancelProcess.emit()
        self.__process.setCancel()
    def setFormatProgressBar(self,text_format):
        self.window.progres_bar.setFormat(text_format)
    @property
    def widget(self):
        return self.window