import os

from enum import Enum
from pathlib import Path

import axipy
from PySide2.QtCore import QObject, Signal
import time

from axipy import TabFile, provider_manager, GCP, Layer, Raster, CoordSystem
from axipy.app import mainwindow, Notifications
from axipy.da.raster import register
from osgeo import gdal, osr

from RasterGridTool.toolprocessing.ui.clipRaster.tool.axiomaToGdal import AxiGpcToGdal, AxiTransformToGeoTransform

def findTabRasterInPath(path_raster):
    list_tab=axipy.da.data_manager.objects
    path_obj_sourthe=Path(path_raster)
    for tab in list_tab:
        if isinstance(tab,Raster):
            path_source_raster=tab.properties['fileName']
            path_obj=Path(path_source_raster)
            if str(path_obj_sourthe)==str(path_obj):
                return True,tab.name
    return False,None
def setNodataDataSource(ds,value_nodata):
    count_band = ds.RasterCount
    for i in range(count_band):
        type_data = ds.GetRasterBand(i + 1).DataType
        if type_data == gdal.GDT_Byte:
            ds.GetRasterBand(i + 1).SetNoDataValue(value_nodata)
            if i+1==4:
                ds.GetRasterBand(i + 1).SetNoDataValue(0)

def createRasterRegistrationTab(ds_source,path_out_file,cs_out):
    cs_mi = "CoordSys " + cs_out.prj
    unit_name = '"' + cs_out.unit.name + '"'
    list_out_str=[]
    list_out_str.append('!table')
    list_out_str.append('!version 800')
    list_out_str.append('!charset WindowsCyrillic')
    list_out_str.append('')
    list_out_str.append("Definition Table")
    list_files=ds_source.GetFileList()
    name_raster=Path(list_files[0]).name
    list_out_str.append('   File "'+name_raster+'"')
    list_out_str.append('Type RASTER')
    geo_transform=ds_source.GetGeoTransform()
    x0,y0=calcPointGeoTransform( geo_transform,0,0)
    xmax,ymax=calcPointGeoTransform( geo_transform,ds_source.RasterXSize,ds_source.RasterYSize)
    str_1=" ("+"{:.8f}".format(x0)+","+"{:.8f}".format(y0)+') (0,0) Label "",'
    list_out_str.append(str_1)
    str_2 = " (" + "{:.8f}".format(xmax) + "," + "{:.8f}".format(y0) + ') ('+str(ds_source.RasterXSize)+',0) Label "",'
    list_out_str.append(str_2)
    str_3 = " (" + "{:.8f}".format(x0) + "," + "{:.8f}".format(ymax) + ') (' + str(
        0) + ','+str(ds_source.RasterYSize)+') Label ""'
    list_out_str.append(str_3)
    list_out_str.append("   "+cs_mi)
    list_out_str.append('   Units ' + unit_name)
    for i,str_item in enumerate(list_out_str):
        list_out_str[i]=list_out_str[i]+'\n'

    with open(path_out_file, mode='w', encoding='cp1251') as file:
        file.writelines(list_out_str)
    return
def calcPointGeoTransform(gdal_geo_transform,xpi,ypi):
    x=gdal_geo_transform[0] + xpi * gdal_geo_transform[1] + ypi * gdal_geo_transform[2]
    y=gdal_geo_transform[3] + xpi * gdal_geo_transform[4] + ypi * gdal_geo_transform[5]
    return x,y
def createAxiGpc(width,height,geo_transform_gdal):
    ''' Формирование точек привязки'''
    x0,y0=calcPointGeoTransform(geo_transform_gdal,0,0)
    pt1= GCP((0,0),(x0,y0))
    x1,y1=calcPointGeoTransform(geo_transform_gdal,width,0)
    pt2=GCP((width,0),(x1,y1))
    x2,y2=calcPointGeoTransform(geo_transform_gdal,width,height)
    pt3=GCP((width,height),(x2,y2))
    x3,y3=calcPointGeoTransform(geo_transform_gdal,0,height)
    pt4=GCP((0,height),(x3,y3))
    return [pt1,pt2,pt3,pt4]

def getOutFormatByExtantion(ext_file):
    if ext_file.lower()==".png":
        return "PNG"
    if ext_file.lower()==".jpg":
        return "JPG"
    if ext_file.lower()==".gpkg":
        return "GPKG"
    if ext_file.lower()==".tif":
        return "GTiff"
    return "GTiff"
class TypeInterpoly(Enum):
    near=0
    average=1
    bilinear=2
    cubic=3
    cubicspline=4
class RasterProcessClip(QObject):
    updateProcess = Signal(float)
    finished = Signal(int)
    def __init__(self,name_prosess,parent_class):
        self.__name_process=name_prosess
        self.__erorr_message=None
        super(RasterProcessClip, self).__init__()
        self.__parent=parent_class
    def __process_callback(self,complete, message=None, user_data=None):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(complete*100)
        time.sleep(0.001)
        #self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def setParams(self,params):
        self.__params=params
    def setCancel(self):
        self.__cancelProcess=True
    def __process_callback(self,complete, message=None, user_data=None):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(complete*100)
        time.sleep(0.001)
        #self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def run(self):
        self.__status_process=False
        self.__tab=[0]
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.setFormatProgressBar(self.__name_process+".. %p%")
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess=False
        axi_tab_raster_name=self.__params["source_tab"]
        axi_tab_raster=axipy.da.data_manager.find(axi_tab_raster_name)
        source_raster=axi_tab_raster.properties['fileName']
        ''' Create temp vrt '''
        path_out_raster=self.__params['out_file']
        if Path(path_out_raster).exists():
            ''' Растр существует '''
            Path(path_out_raster).unlink(missing_ok=False)
        path_out_vrt="/vsimem/"+Path(path_out_raster).stem+".vrt"
        #path_out_vrt="e:/tmp/"+Path(path_out_raster).stem+".vrt"

        tr=axi_tab_raster.device_to_scene_transform
        gpc=axi_tab_raster.get_gcps()
        gdal_gpc=AxiGpcToGdal(gpc)
        gdal_geo_transform=AxiTransformToGeoTransform(tr)
        cs_proj=axi_tab_raster.coordsystem.proj
        cs_proj = axi_tab_raster.coordsystem.wkt
        cs_source_wkt=axi_tab_raster.coordsystem.wkt
        gdal_SRS_source = osr.SpatialReference()
        #gdal_SRS_source.ImportFromProj4(cs_proj)
        gdal_SRS_source.ImportFromWkt(cs_source_wkt)
        ds_source=gdal.Open(source_raster)
        #ds_source.SetProjection(gdal_SRS_source.ExportToWkt())
        count_band=ds_source.RasterCount
        noDataValue=None
        #if count_band==1:
        raster_source_cs_wkt=ds_source.GetProjection()
        try:
            axi_cs_raster=CoordSystem.from_wkt(raster_source_cs_wkt)
        except:
            axi_cs_raster=axi_tab_raster.coordsystem
        noDataValue=ds_source.GetRasterBand(1).GetNoDataValue()

        cs_out=self.__params['out_cs']
        #cs_proj4=cs_out.proj
        cs_wkt=cs_out.wkt

        ds_curent = gdal.Translate(path_out_vrt, ds_source, format='VRT', outputSRS=cs_source_wkt,
                                   GCPs=gdal_gpc)

        if count_band==1 and noDataValue is not None:
            try:
                ds_curent.GetRasterBand(1).SetNoDataValue(noDataValue)
            except:
                print("Error set noDataValue"+str(noDataValue))
        else:
            setNodataDataSource(ds_curent,255)
        cls_clip_spatial=self.__params['clip_region']
        clip_spatial=None
        bound_clip=None
        if cls_clip_spatial is not None:
            clip_spatial=cls_clip_spatial.path
            bound_clip=cls_clip_spatial.getBound(axi_cs_raster)
        bound_clip=None
        out_format=getOutFormatByExtantion(Path(path_out_raster).suffix)
        options_create=[]
        if out_format=="GPKG":
            options_create.append('TILE_FORMAT=PNG')
        if out_format=="JPG" or  out_format=="PNG":
            options_create.append('TFW=YES')
        #options_create.append('COMPRESS=LZW')
        out_gpc=None
        out_geotransform=None
        try:
            if bound_clip is None:
                ds_out=gdal.Warp(path_out_raster,ds_curent,format = out_format,dstSRS =cs_wkt,creationOptions = options_create,tps = True,dstNodata =noDataValue,cropToCutline = True,cutlineDSName = clip_spatial,callback=self.__process_callback)
            else:
                bound_warp=(bound_clip.xmin,bound_clip.ymin,bound_clip.xmax,bound_clip.ymax)
                ds_out = gdal.Warp(path_out_raster, ds_curent, format=out_format, dstSRS=cs_wkt,
                                   creationOptions=options_create, tps=True, dstNodata=noDataValue, cropToCutline=True,
                                   cutlineDSName=clip_spatial,outputBounds=bound_warp, callback=self.__process_callback)
            if self.__cancelProcess:
                ds_curent=None
                ds_out=None
                ds_source=None
                gdal.Unlink(path_out_vrt)
                if cls_clip_spatial is not None:
                    cls_clip_spatial.close()
                    self.finished.emit(100)
                    return
            if ds_out is not None:

                if count_band==1 and noDataValue is not None:
                    try:
                        ds_out.GetRasterBand(1).SetNoDataValue(noDataValue)
                    except:
                        print("Error set noDataValue" + str(noDataValue))

                ds_out.FlushCache()
                out_geotransform = ds_out.GetGeoTransform()
                out_gpc = ds_out.GetGCPs()
                width_out = ds_out.RasterXSize
                hight_out = ds_out.RasterYSize
        except Exception as ex:
            print(ex)
        #finally:
        ds_curent=None

        ds_source=None
        gdal.Unlink(path_out_vrt)
        if ds_out is None:

            Notifications.push('Отсечение растра', 'Ошибка при выполнении операции . Обратитесь к разработчику', Notifications.Critical)
            self.updateProcess.emit(100)
            self.finished.emit(100)
            return
        if cls_clip_spatial is not None:
            cls_clip_spatial.close()
        out_file_name=os.path.join(str(Path(path_out_raster).parent),str(Path(path_out_raster).stem)+".tab")
        out_format=getOutFormatByExtantion(Path(path_out_raster).suffix)
        if out_format=="JPG" or out_format=="PNG":
            ''' формируем точки привязки'''
            gcps=createAxiGpc(width_out,hight_out,out_geotransform)
            register(path_out_raster,gcps,cs_out)
            tab_res=provider_manager.openfile(out_file_name)
            layer=Layer.create(tab_res)
            mainwindow.add_layer_interactive(layer)
        else:
            #out_file_name=os.path.join(str(Path(path_out_raster).parent),str(Path(path_out_raster).stem)+".tab")
            pass
            '''
            tab_out_raster=provider_manager.gdal.open(path_out_raster)

            tab = TabFile()
            tab.generate_tab(tab_out_raster, out_file_name)
            tab_out_raster.close()
            '''
            path_dest=Path(path_out_raster)
            folder=path_dest.parent
            name=str(path_dest.stem)
            path_out_tab=str(folder.joinpath(name+".tab"))
            cs_mi="CoordSys "+axi_cs_raster.prj
            unit_name='"'+axi_cs_raster.unit.name+'"'
            createRasterRegistrationTab(ds_out, path_out_tab, axi_cs_raster)

            ds_out.FlushCache()
            ds_out = None
            tab_raster_new=provider_manager.openfile(out_file_name)

            layer=Layer.create(tab_raster_new)
            mainwindow.add_layer_interactive(layer)

            #provider_manager.openfile(out_file_name)
        self.updateProcess.emit(100)
        self.finished.emit(100)
    def __isCancel(self):
        self.__cancelProcess=True
