import os
import tempfile

import axipy
from axipy import *
from osgeo import gdal, ogr

from .axiomaToGdal  import OgrVector


class TempVector:
    def __init__(self,name=None):
        if name is None:
            self.__name=next(tempfile._get_candidate_names())
        else:
            self.__name=name
    def create(self,cs):
        self.__coordsystem=cs
        self.__path_mem_shp=os.path.join("/vsimem/",self.__name+".shp")
        #self.__path_mem_shp=os.path.join("e:/tmp/",self.__name+".shp")
        self.__shp=OgrVector(self.__path_mem_shp,cs.proj)

    def copy(self, object_geometry):
        self.__count_polygon = 0
        self.__bound=None
        if isinstance(object_geometry, Geometry):
            str_type = str(object_geometry)
            is_polygon = object_geometry.type == axipy.GeometryType.Polygon
            if object_geometry.type == axipy.GeometryType.Polygon or object_geometry.type == axipy.GeometryType.MultiPolygon:
                isValid = object_geometry.is_valid
                self.__count_polygon = self.__count_polygon + 1
                self.__shp.addGeometry(object_geometry)
        else:
            for ft in object_geometry:
                str_type = str(ft.geometry.type)
                geometry_source = ft.geometry
                if ft.geometry.coordsystem!=self.__coordsystem:
                    geometry_source=ft.geometry.reproject(self.__coordsystem)

                if isinstance(ft.geometry,Rectangle):
                    geometry_source=ft.geometry.to_polygon()
                    str_type=geometry_source.type
                axi_geometry = geometry_source
                if isinstance(axi_geometry,Polygon) or isinstance(axi_geometry,MultiPolygon) :
                #if str_type == "GeometryType.Polygon" or str_type == "GeometryType.MultiPolygon" or str_type == 'GeometryType.GeometryCollection':
                    if self.__bound is None:
                        self.__bound=axi_geometry.bounds
                    else:
                        self.__bound.merge(axi_geometry.bounds)
                    isValid = geometry_source.is_valid
                    if not isValid:
                        geo_gdal = ogr.CreateGeometryFromWkb(geometry_source.wkb)
                        geom_gdal_valid = geo_gdal.MakeValid()
                        axi_geometry = Geometry.from_wkb(geom_gdal_valid.ExportToWkb(), ft.geometry.coordsystem)
                        geo_gdal = None
                        geom_gdal_valid = None
                        str_type = str(axi_geometry.type)
                    list_geometry = []
                    if str_type == 'GeometryType.GeometryCollection':
                        for geo in axi_geometry:
                            str_type_geo = str(geo.type)
                            if str_type_geo == "GeometryType.Polygon" or str_type_geo == "GeometryType.MultiPolygon":
                                list_geometry.append(geo)
                    else:
                        list_geometry.append(axi_geometry)
                    self.__count_polygon = self.__count_polygon + len(list_geometry)

                    self.__shp.addGeometry(list_geometry)
            self.__shp.close()
    def getBound(self,cs=None):
        if cs is None:
            return self.__bound
        geo_bound=Rectangle(self.__bound,cs=self.__coordsystem)
        geo_out_cs=geo_bound.reproject(cs)
        return geo_out_cs.bounds
    @property
    def count(self):
        return self.__count_polygon
    @property
    def path(self):
        return self.__path_mem_shp
    def close(self):
        if self.__shp is not None:
            pass
            gdal.Unlink(self.__path_mem_shp)