import tempfile
from pathlib import Path

from axipy import Raster, provider_manager
from osgeo import osr, gdal

from RasterGridTool.toolprocessing.ui.clipRaster.tool.axiomaToGdal import AxiGpcToGdal
from RasterGridTool.toolprocessing.ui.clipRaster.tool.rasterTool import setNodataDataSource


class RasterFromFile:
    def __init__(self,path_raster):
        self.__path_source=path_raster
        self.__name=Path(self.__path_source).name
        self.__isRaster=True
        try:
            name_temp=tempfile.NamedTemporaryFile()
            tab_temp= provider_manager.open_hidden({'src':self.__path_source,'name':name_temp})
            self.__cs=tab_temp.coordsystem
            self.__isRaster=isinstance(tab_temp,Raster)
            tab_temp.close()
        except:
            self.__cs=None
    @property
    def name(self):
        return self.__name
    @property
    def path(self):
        return self.__path_source
    @property
    def coordsystem(self):
        return self.__cs
    @property
    def isRaster(self):
        return self.__isRaster

class RasterAxioma:
    def __init__(self,table_raster):
        self.__table=table_raster
        self.__path_source=table_raster.properties['fileName']
    @property
    def name(self):
        return self.__table.name
    @property
    def path(self):
        return self.__path_source
    @property
    def coordsystem(self):
        return self.__table.coordsystem
    @property
    def isRaster(self):
        return isinstance(self.__table,Raster)
    @property
    def table(self):
        return self.__table
class prepareRasterAxi:
    def __init__(self):
        self.__axiRaster=[]
        self.__temp_files=[]

        pass
    def add(self,tab_add):
        #self.__axiRaster.append(tab)
        if isinstance(tab_add,RasterAxioma):
            tab=tab_add.table
        else:
            tab=tab_add
        source_raster=tab.properties['fileName']
        path_out_vrt="/vsimem/"+Path(source_raster).stem+".vrt"
        #path_out_vrt=str(Path(r"E:\Temp\temp_raster").joinpath(Path(source_raster).stem+".vrt"))
        self.__temp_files.append(path_out_vrt)
        gpc=tab.get_gcps()
        gdal_gpc=AxiGpcToGdal(gpc)
        cs_source_wkt=tab.coordsystem.wkt
        gdal_SRS_source = osr.SpatialReference()

        gdal_SRS_source.ImportFromWkt(cs_source_wkt)
        ds_source=gdal.Open(source_raster)
        ds_source.SetProjection(gdal_SRS_source.ExportToWkt())
        ds_source.SetGCPs(gdal_gpc, cs_source_wkt)
        setNodataDataSource(ds_source, 255)
        '''
        ds_source.SetGCPs(gdal_gpc,cs_source_wkt)
        setNodataDataSource(ds_source, 255)
        ds_curent = gdal.Translate(path_out_vrt, ds_source, format='VRT', outputSRS=cs_source_wkt,
                                   GCPs=gdal_gpc,rgbExpand="rgba")
        '''
        ds_curent = gdal.Translate(path_out_vrt, ds_source, format='VRT', outputSRS=cs_source_wkt,
                                   GCPs=gdal_gpc)
        ds_source=None
        ds_curent=None
    @property
    def getVrtFiles(self):
        return self.__temp_files
    @property
    def DsVrt(self):
        self.__list_ds_temp=[]
        for vrt in self.__temp_files:
            self.__list_ds_temp.append(gdal.Open(vrt))
        return self.__list_ds_temp
    def clearTemp(self):
        for ds_temp in self.__list_ds_temp:
            ds_temp=None
    def close(self):
        for temp_file in self.__temp_files:
            gdal.Unlink(temp_file)
        '''
        for tab in self.__axiRaster:
            tab.close()
        '''