import math
from pathlib import Path

from PySide2.QtCore import QObject, Signal
import time

from axipy import provider_manager
from osgeo import gdal


from RasterGridTool.toolprocessing.ui.clipRaster.tool.rasterTool import getOutFormatByExtantion, TypeInterpoly
from RasterGridTool.toolprocessing.ui.combineRaster.tool.addTool import RasterFromFile, prepareRasterAxi


def div_round_up(a, b):
    """Return ceil(a/b)."""
    return int(math.ceil(float(a) / b))
def calcLevels(width,height,size_tile=256):
    levels=[]
    nOvrFactor = 1
    while(div_round_up(width,nOvrFactor)>size_tile or div_round_up(height,nOvrFactor)>size_tile):
        nOvrFactor=nOvrFactor*2
        levels.append(nOvrFactor)
    return levels
class RasterProcessCombine(QObject):
    updateProcess = Signal(float)
    updateFormatProgressBar=Signal(str)
    finished = Signal(int)
    def __init__(self,name_prosess,parent_class):
        self.__name_process=name_prosess

        super(RasterProcessCombine, self).__init__()
        self.__parent=parent_class
        self.__erorr_message=None
    def setParametrs(self,tableRaster,out_cs_wkt,method_interp_raster,out_path_raster,out_options):
        self.__source_tables=tableRaster
        self.__out_cs_wkt=out_cs_wkt
        self.__method_interpol=method_interp_raster
        self.__out_path=out_path_raster
        self.__out_options=out_options
    def __process_callback(self,complete, message=None, user_data=None):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(complete*100)
        time.sleep(0.001)
        #self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def setCancel(self):
        self.__cancelProcess=True
    def __isCancel(self):
        self.__cancelProcess=True
    def __prepareSource(self,list_source,fn_call_back=None):
        count=len(list_source)
        temp_vrt_source=prepareRasterAxi()
        ret_val=1
        for i,source_tab in enumerate(list_source):
            if fn_call_back is not None:
                ret_val=fn_call_back(i/count)
                if ret_val==0:
                    break

            if isinstance(source_tab,RasterFromFile):
                temp_axi_raster_tab=provider_manager.open_hidden({'src':source_tab.path})
                temp_vrt_source.add(temp_axi_raster_tab)
                temp_axi_raster_tab.close()
            else:
                temp_vrt_source.add(source_tab)
        if ret_val==0:
            return None
        return temp_vrt_source
    def run(self):
        #print("Start combine")
        self.__status_process=False
        self.__tab=[0]
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        ''' формируем список vrt для исходных растров'''
        call_back_fun=None
        if len(self.__source_tables)>5:
            call_back_fun=self.__process_callback
            self.updateFormatProgressBar.emit("Подготовка исходных растров"+".. %p%")
            #self.__parent.__setFormatProgressBar("Подготовка исходных растров"+".. %p%")
        conteiner_vrt=self.__prepareSource(self.__source_tables,call_back_fun)
        #self.__parent.setFormatProgressBar(self.__name_process+".. %p%")
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess=False

        out_format=getOutFormatByExtantion(Path(self.__out_path).suffix)
        options_create=[]
        if out_format=="GPKG":
            if self.__out_options['tile']:
                options_create.append('TILE_FORMAT=PNG')
            '''    
            if self.__out_options['compress']:
                options_create.append('ZLEVEL=6')
            else:
                options_create.append('ZLEVEL=1')
            '''
            if self.__out_options['compress']:
                options_create.append('COMPRESS=LZW')
        if out_format=="GTiff":
            if self.__out_options['tile']:
                options_create.append('TILED=YES')
            if self.__out_options['compress']:
                options_create.append('COMPRESS=LZW')
        if out_format=="JPG" or "PNG":
            options_create.append('TFW=YES')
        try:
            alg_resampling=gdal.GRA_NearestNeighbour
            if self.__method_interpol==TypeInterpoly.average:
                alg_resampling=gdal.GRA_Average
            if self.__method_interpol==TypeInterpoly.cubic:
                alg_resampling=gdal.GRA_Cubic
            if self.__method_interpol==TypeInterpoly.bilinear:
                alg_resampling=gdal.GRA_Bilinear
            if self.__method_interpol==TypeInterpoly.cubicspline:
                alg_resampling=gdal.GRA_CubicSpline


            #ds_out=gdal.Warp(self.__out_path,conteiner_vrt.getVrtFiles,format = out_format,dstSRS =self.__out_cs_wkt,resampleAlg=alg_resampling,creationOptions=options_create,callback=self.__process_callback)
            ds_out = gdal.Warp(self.__out_path, conteiner_vrt.DsVrt, format=out_format, dstSRS=self.__out_cs_wkt,
                               resampleAlg=alg_resampling, creationOptions=options_create,
                               callback=self.__process_callback)
            if ds_out is not None:
                ds_out.FlushCache()
            else:
                print("Not create out Image")
            if ds_out is not  None and self.__out_options['tile']:
                count_over=ds_out.GetRasterBand(1).GetOverviewCount()

                levels=calcLevels(ds_out.RasterXSize,ds_out.RasterYSize)
                if count_over!=len(levels):
                    self.updateFormatProgressBar.emit("Построение пирамиды "+".. %p%")
                    self.updateProcess.emit(0)
                    ds_out.BuildOverviews("AVERAGE",overviewlist=levels,callback=self.__process_callback)

        except Exception as ex:
            self.__cancelProcess=True
            print(ex)
        conteiner_vrt.clearTemp()
        ds_out=None
        conteiner_vrt.close()
        if self.__cancelProcess:
            self.finished.emit(0)
            return
        self.finished.emit(100)
        return

    def runTest(self):
        print("Start combine")
        self.__status_process = False
        self.__tab = [0]

        ''' формируем список vrt для исходных растров'''
        call_back_fun = None

        conteiner_vrt = self.__prepareSource(self.__source_tables, call_back_fun)
        # self.__parent.setFormatProgressBar(self.__name_process+".. %p%")
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess = False

        out_format = getOutFormatByExtantion(Path(self.__out_path).suffix)
        options_create = []
        if out_format == "GPKG":
            if self.__out_options['tile']:
                options_create.append('TILE_FORMAT=PNG')
            '''    
            if self.__out_options['compress']:
                options_create.append('ZLEVEL=6')
            else:
                options_create.append('ZLEVEL=1')
            '''
            if self.__out_options['compress']:
                options_create.append('COMPRESS=LZW')
        if out_format == "GTiff":
            if self.__out_options['tile']:
                options_create.append('TILED=YES')
            if self.__out_options['compress']:
                options_create.append('COMPRESS=LZW')
        if out_format == "JPG" or "PNG":
            options_create.append('TFW=YES')
        try:
            alg_resampling = gdal.GRA_NearestNeighbour
            if self.__method_interpol == TypeInterpoly.average:
                alg_resampling = gdal.GRA_Average
            if self.__method_interpol == TypeInterpoly.cubic:
                alg_resampling = gdal.GRA_Cubic
            if self.__method_interpol == TypeInterpoly.bilinear:
                alg_resampling = gdal.GRA_Bilinear
            if self.__method_interpol == TypeInterpoly.cubicspline:
                alg_resampling = gdal.GRA_CubicSpline

            ds_out = gdal.Warp(self.__out_path, conteiner_vrt.getVrtFiles, format=out_format, dstSRS=self.__out_cs_wkt,
                               resampleAlg=alg_resampling, creationOptions=options_create,
                               callback=self.__process_callback)
            if ds_out is not None:
                ds_out.FlushCache()
            else:
                print("Not create out Image")
            if ds_out is not None and self.__out_options['tile']:
                count_over = ds_out.GetRasterBand(1).GetOverviewCount()

                levels = calcLevels(ds_out.RasterXSize, ds_out.RasterYSize)
                if count_over != len(levels):
                    self.updateFormatProgressBar.emit("Построение пирамиды " + ".. %p%")
                    self.updateProcess.emit(0)
                    ds_out.BuildOverviews("AVERAGE", overviewlist=levels, callback=self.__process_callback)

        except Exception as ex:
            self.__cancelProcess = True
            print(ex)
        ds_out = None
        conteiner_vrt.close()
        if self.__cancelProcess:
            self.finished.emit(0)
            return
        self.finished.emit(100)
        return

