import math
import operator
import struct
from math import fabs

from osgeo import ogr, gdal
#from osgeo import gdal_array
def applyGeoTransform( inX, inY, geoTransform,levels ):
    '''Apply a geotransform to coordinates.
    @param inX             Input coordinate (double)
    @param inY             Input coordinate (double)
    @param geoTransform    Input geotransform (six doubles)
    @return outX, outY     Output coordinates (two doubles)
    '''
    outX = geoTransform[ 0 ] + inX * geoTransform[ 1 ] + inY * geoTransform[ 2 ]
    outY = geoTransform[ 3 ] + inX * geoTransform[ 4 ] + inY * geoTransform[ 5 ]
    return outX, outY
def invertGeoTransform( geoTransform ):
    '''Invert standard 3x2 set of geotransform coefficients.
    @param geoTransform        Input GeoTransform (six doubles - unaltered)
    @return outGeoTransform    Output GeoTransform ( six doubles - updated )
                               on success, None if the equation is uninvertable
    '''
    # we assume a 3rd row that is [ 1 0 0 ]
    # compute determinate
    det = geoTransform[ 1 ] * geoTransform[ 5 ] - geoTransform[ 2 ] * geoTransform[ 4 ]

    if abs( det ) < 0.000000000000001:
        return

    invDet = 1.0 / det

    # compute adjoint and divide by determinate
    outGeoTransform = [ 0, 0, 0, 0, 0, 0 ]
    outGeoTransform[ 1 ] = geoTransform[ 5 ] * invDet
    outGeoTransform[ 4 ] = -geoTransform[ 4 ] * invDet

    outGeoTransform[ 2 ] = -geoTransform[ 2 ] * invDet
    outGeoTransform[ 5 ] = geoTransform[ 1 ] * invDet

    outGeoTransform[ 0 ] = ( geoTransform[ 2 ] * geoTransform[ 3 ] - geoTransform[ 0 ] * geoTransform[ 5 ] ) * invDet
    outGeoTransform[ 3 ] = ( -geoTransform[ 1 ] * geoTransform[ 3 ] + geoTransform[ 0 ] * geoTransform[ 4 ] ) * invDet

    return
def mapToPixel( mX, mY, geoTransform ):
    '''Convert map coordinates to pixel coordinates.
    @param mX              Input map X coordinate (double)
    @param mY              Input map Y coordinate (double)
    @param geoTransform    Input geotransform (six doubles)
    @return pX, pY         Output coordinates (two doubles)
    '''
    if geoTransform[ 2 ] + geoTransform[ 4 ] == 0:
        pX = ( mX - geoTransform[ 0 ] ) / geoTransform[ 1 ]
        pY = ( mY - geoTransform[ 3 ] ) / geoTransform[ 5 ]
    else:
        pX, pY = applyGeoTransform( mX, mY, invertGeoTransform( geoTransform ) )
    #return int( pX + 0.5 ), int( pY + 0.5 )
    #see http://gis-lab.info/forum/viewtopic.php?f=3&t=8242&st=0&sk=t&sd=a&start=10000#p137632
    return int(math.floor(pX)), int(math.floor(pY))

def findInetrVals(levels,value):


    for i in range(1,len(levels),1):
        if value>=levels[i-1] and value<levels[i]:
            return levels[i-1],levels[i]
    return None,None

def prepare(layer_bound,layer_contour,area_pix,band_source_grid,geo_tarnsform,levels):
    count_bound=layer_bound.GetFeatureCount()
    geo_bound=None
    if count_bound>1:
        ''' Объединение в один'''
        pass
    else:
        ft=layer_bound.GetFeature(0)
        if ft is None:
            ft=layer_bound.GetFeature(1)
        geo_bound=ft.GetGeometryRef().Clone()
        ft=None
    count_countr=layer_contour.GetFeatureCount()
    for i in range(count_countr+1):
        ft=layer_contour.GetFeature(i)
        if ft is None:
            continue
        geo=ft.GetGeometryRef()
        if geo is None:
            ft=None
            continue
        geo_bound=geo_bound.Difference(geo)
    jkl=0
    ''' По идее должны получить регионы которых нет в контурах '''
    count_poly=geo_bound.GetGeometryCount()
    list_out_polygon=[]
    for i in range(count_poly+1):
        temp_poly=geo_bound.GetGeometryRef(i)
        if temp_poly is None:
            continue
        #is_valid=temp_poly.IsValid()
        area=temp_poly.Area()
        if area>area_pix:
            list_out_polygon.append(temp_poly)
        jkl=0
    if len(list_out_polygon)==0:
        return
    feature_def=layer_contour.GetLayerDefn()
    for geo_obj in list_out_polygon:
        centroid=geo_obj.Centroid()
        x=centroid.GetX()
        y=centroid.GetY()
        xp,yp=mapToPixel(x,y,geo_tarnsform)
        #data = band_source_grid.ReadAsArray(xp, yp, 1, 1)
        data=band_source_grid.ReadRaster(xp, yp, 1, 1,1,1,buf_type=gdal.GDT_Float32)
        value = struct.unpack('<f',  data[0:4])
        min_level,max_level=findInetrVals(levels,value[0])
        if max_level is None and min_level is None:
            continue
        new_ft=ogr.Feature(feature_def)
        new_ft.SetGeometry(geo_obj)
        new_ft.SetField("level_min",min_level)
        new_ft.SetField("level_max",max_level)
        layer_contour.CreateFeature(new_ft)
        geo_obj=None
        new_ft=None
