import math
import os
from pathlib import Path

from PySide2 import QtCore, QtWidgets, QtGui
from PySide2.QtCore import QFile
from PySide2.QtGui import QDoubleValidator, QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QLineEdit, QTableWidgetItem
#from axipy import Style, StyledButton
from axipy import StyledButton, Style


def createFloatTabWidget(value):
    item_value=QTableWidgetItem(str(value))
    item_value.setTextAlignment(QtCore.Qt.AlignRight)
    return item_value

def createLineEditLevel(pre_min,value,next_max):
    ln_edit_level=QLineEdit()
    ln_edit_level.setFrame(False)
    str_val=str(value)
    ln_edit_level.setAlignment(QtCore.Qt.AlignRight)
    ln_edit_level.setText(str_val)
    db_validator_val=QDoubleValidator(ln_edit_level)
    if pre_min is not None:
        db_validator_val.setBottom(pre_min)
    if next_max is not None:
        db_validator_val.setTop(next_max)
    ln_edit_level.setValidator(db_validator_val)
    ln_edit_level.setCursorPosition(0)
    return ln_edit_level
class StyledItemDelegate(QtWidgets.QStyledItemDelegate):
    def createEditor(self, parent, option, index):
        editor = super(StyledItemDelegate, self).createEditor( parent, option, index )
        if isinstance(editor, QtWidgets.QLineEdit):
            validator = QtGui.QDoubleValidator( editor, notation=QtGui.QDoubleValidator.StandardNotation )
            editor.setValidator(validator)
            return editor


class CountourLevelWidget(QWidget):
    def __init__(self,params_init,parent=None):
        self.__startView=True
        self.__parentWin=parent
        self.__digramView=None
        super(CountourLevelWidget, self).__init__(parent)
        self.load_ui()
        self.window.tablPolygon.setColumnCount(3)
        self.window.tablPolygon.setColumnWidth(2,30)
        #self.window.tablPolygon.setHorizontalHeaderLabels(["Min", "Max", "Header 3"])
        #name_col1=QTableWidgetItem("Min")
        #self.window.tablPolygon.setHorizontalHeaderItem(0,name_col1)
        curent_path=str(Path(__file__).parent)
        path_icon=os.path.join(curent_path,"icons")
        path_add_png=os.path.join(path_icon,"add.png")
        path_del_png=os.path.join(path_icon,"del.png")
        icon_add=QIcon(path_add_png)
        icon_del=QIcon(path_del_png)
        self.window.pb_add_interval.setIcon(icon_add)
        self.window.pb_del_row.setIcon(icon_del)
        self.__params_init=None
        if params_init is not None:
            ''' Инициализируем таблицу уровней'''
            self.__params_init=params_init
            self.initLevels2(params_init)
        self.window.tablPolygon.cellChanged.connect(self.__changeItem)
        self.window.tablPolygon.itemSelectionChanged.connect(self.__selectItems)
        self.window.pb_del_row.clicked.connect(self.__del_row)
        self.window.pb_add_interval.clicked.connect(self.__add_row)

    def updateIntervals(self,params_init):
        self.__params_init=params_init
        self.initLevels2(params_init)
    def load_ui(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "ContourPolygons.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window  = loader.load(ui_file,self.__parentWin)
        ui_file.close()
    def initLevels(self,params_init):
        min_value=params_init['min']
        max_value=params_init['max']
        count=params_init['count']
        step=(max_value-min_value)/count
        self.window.tablPolygon.clear()
        self.window.tablPolygon.setRowCount(count)
        for i in range(count):
            min_level=min_value+i*step
            max_value=min_level+step
            if max_value>params_init['max']:
                max_value=params_init['max']
            pre_min=None
            if i>0:
                pre_min=min_level-step
            next_max=None
            if i<(count-1):
                next_max=max_value+step
            self.__addRow(i,min_level,max_value,pre_min,next_max)
    def setIntervals(self,levels):
        count=len(levels)
        self.window.tablPolygon.clearContents()
        self.window.tablPolygon.setRowCount(count)
        for i,level in enumerate(levels):
            item_low=createFloatTabWidget(level['min'])


            item_max=createFloatTabWidget(level['max'])
            self.window.tablPolygon.setItem(i,0, item_low)
            self.window.tablPolygon.setItem(i,1,item_max)
            style=Style.from_mapinfo(level['mi_style'])
            widget_style= StyledButton(style)
            #self.window.tablPolygon.setItem(i,2,QTableWidgetItem(widget_style))
            self.window.tablPolygon.setCellWidget(i,2,widget_style)
    def initLevels2(self,params_init):
        self.__delegate=StyledItemDelegate(self.window.tablPolygon)
        self.window.tablPolygon.setItemDelegate(self.__delegate)
        min_value=params_init['min']
        max_value=params_init['max']
        count=None
        try:
            count=params_init['count']
        except:
            pass
        if count is None:
            step=params_init['step']
            count=math.ceil((max_value-min_value)/step)
        else:
            step=(max_value-min_value)/count
        self.window.tablPolygon.clearContents()
        self.window.tablPolygon.setRowCount(count)
        mi_poly_def_style=Style.from_mapinfo("Pen (1, 2, 0) Brush (1, 16777215, 16777215)")
        for i in range(count):
            min_level=min_value+i*step
            max_value=min_level+step
            '''
            if max_value>params_init['max']:
                max_value=params_init['max']
            '''
            pre_min=None
            if i>0:
                pre_min=min_level-step
            next_max=None
            if i<(count-1):
                next_max=max_value+step
            #self.__addRow(i,min_level,max_value,pre_min,next_max)
            #item_low=QTableWidgetItem(str(min_level))
            item_low=createFloatTabWidget(min_level)
            #item_low.setTextAlignment(QtCore.Qt.AlignRight)
            item_max=createFloatTabWidget(max_value)
            #item_max.setTextAlignment(QtCore.Qt.AlignRight)
            self.window.tablPolygon.setItem(i,0, item_low)
            self.window.tablPolygon.setItem(i,1,item_max)
            widget_style= StyledButton(mi_poly_def_style)
            #self.window.tablPolygon.setItem(i,2,QTableWidgetItem(widget_style))
            self.window.tablPolygon.setCellWidget(i,2,widget_style)
        if max_value < params_init['max']:
            i=count
            item_low=createFloatTabWidget( max_value)

            item_max=createFloatTabWidget(params_init['max'])

            self.window.tablPolygon.setItem(i,0, item_low)
            self.window.tablPolygon.setItem(i,1,item_max)
            widget_style= StyledButton(mi_poly_def_style)
        #self.window.tablPolygon.setItem(i,2,QTableWidgetItem(widget_style))
            self.window.tablPolygon.setCellWidget(i,2,widget_style)
    def __addRow(self,id_row,min,max,pre_min,next_max):
        ln_edit_min=createLineEditLevel(pre_min,min,next_max)
        ln_edit_max=createLineEditLevel(min,max,next_max)
        self.window.tablPolygon.setCellWidget(id_row,0,ln_edit_min)
        self.window.tablPolygon.setCellWidget(id_row,1,ln_edit_max)
    def __changeItem(self,row,col):
        widget_change=self.window.tablPolygon.cellWidget(row,col)
        if widget_change is None:
            return
        if col==2:
            return
        value_txt=widget_change.text()
        value=float(value_txt)
        if row==0 and col==0:
            ''' Изменен минимальный уровень'''
            if value<self.__params_init['min']:
                ''' Значение меньше заданного '''
                item_low=createFloatTabWidget(self.__params_init['min'])
                #item_low.setTextAlignment(QtCore.Qt.AlignRight)
                self.window.tablPolygon.setItem(row,col,item_low)
                return
        if row==self.window.tablPolygon.rowCount()-1 and col==1:
            ''' Изменен максимальный уровень '''
            if value>self.__params_init['max']:
                item=createFloatTabWidget(self.__params_init['max'])
                #item_low.setTextAlignment(QtCore.Qt.AlignRight)
                self.window.tablPolygon.setItem(row,col,item)
                return
        if col==0:
            ''' Изменен минимум интервала '''

            pred_item_max=self.window.tablPolygon.item(row-1,1).text()
            pred_item_min=self.window.tablPolygon.item(row-1,0).text()
            prev_min=float(pred_item_min)
            prev_max=float(pred_item_max)
            if value<=prev_min:
                ''' Новое значение меньше нижнего уровня предудыщего'''
                item=createFloatTabWidget(prev_max)
                #item_low.setTextAlignment(QtCore.Qt.AlignRight)
                self.window.tablPolygon.setItem(row,col,item)
                return
            if value<prev_max:
                ''' Изменяем предыдущий max на уровень min текущего'''
                #item=createFloatTabWidget(value)
                #item_low.setTextAlignment(QtCore.Qt.AlignRight)
                self.window.tablPolygon.item(row-1,1).setText(value_txt)
                return
        if col==1:
            ''' Изменяется верхний интервал'''
            next_item_max=self.window.tablPolygon.item(row+1,1).text()
            next_item_min=self.window.tablPolygon.item(row+1,0).text()
            next_min=float(next_item_min)
            next_max=float(next_item_max)
            if value>next_min and value<next_max:
                self.window.tablPolygon.item(row+1,0).setText(value_txt)
                return
            self.window.tablPolygon.item(row,1).setText(next_item_min)
            return
    def __selectItems(self):
        selectedItems=self.window.tablPolygon.selectedItems()
        if selectedItems is None or len(selectedItems)==0:
            self.window.pb_add_interval.setEnabled(False)
            self.window.pb_del_row.setEnabled(False)
        else:
            if self.window.tablPolygon.rowCount()<=2:
                self.window.pb_del_row.setEnabled(False)
            else:
                self.window.pb_del_row.setEnabled(True)
            self.window.pb_add_interval.setEnabled(True)

    def __del_row(self):
        selectedItems=self.window.tablPolygon.selectedItems()
        row_id=selectedItems[0].row()
        min_curent_str=self.window.tablPolygon.item(row_id,0).text()
        max_curent_str=self.window.tablPolygon.item(row_id,1).text()
        if row_id==0:
            ''' Удаляется первый интервал '''
            self.window.tablPolygon.item(row_id+1,0).setText(min_curent_str)
        if row_id==self.window.tablPolygon.rowCount()-1:
            ''' Удаляем последний интервал'''
            self.window.tablPolygon.item(row_id-1,1).setText(max_curent_str)
        if row_id>0 and row_id<self.window.tablPolygon.rowCount()-1:
            self.window.tablPolygon.item(row_id-1,1).setText(min_curent_str)
            self.window.tablPolygon.item(row_id+1,0).setText(max_curent_str)
        self.window.tablPolygon.removeRow(row_id)

    def __add_row(self):
        selectedItems=self.window.tablPolygon.selectedItems()
        row_id=selectedItems[0].row()
        min_curent_str=self.window.tablPolygon.item(row_id,0).text()
        max_curent_str=self.window.tablPolygon.item(row_id,1).text()

        min_curent=float(min_curent_str)
        max_curent=float(max_curent_str)
        delta=max_curent-min_curent
        new_value=min_curent+delta/2
        self.window.tablPolygon.item(row_id,1).setText(str(new_value))
        self.window.tablPolygon.insertRow(row_id+1)
        #self.window.tablPolygon.item(row_id,1).setText("0.0")
        #self.window.tablPolygon.item(row_id,0).setText("0.0")
        item_min=createFloatTabWidget(new_value)
        #item_max.setTextAlignment(QtCore.Qt.AlignRight)
        self.window.tablPolygon.setItem(row_id+1,0, item_min)
        item_max=createFloatTabWidget(max_curent)
        self.window.tablPolygon.setItem(row_id+1,1,item_max)
    def getLevels(self):
        list_levels=[]
        count=self.window.tablPolygon.rowCount()
        for i in range(count):
            str_min=self.window.tablPolygon.item(i, 0).text()
            str_max=self.window.tablPolygon.item(i, 1).text()
            mi_style=None
            widget_style=self.window.tablPolygon.cellWidget(i,2)
            if widget_style is not None:
                mi_style= widget_style.style().to_mapinfo()
            list_levels.append({'min':float(str_min),"max":float(str_max),'mi_style':mi_style})
        min=list_levels[count-1]['min']
        max=list_levels[count-1]['max']
        style=list_levels[count-1]['mi_style']
        delta=max-min
        list_levels.append({'min':list_levels[count-1]['max'],"max":list_levels[count-1]['max']+delta,'mi_style':style})
        return list_levels











