import os
import time
from pathlib import Path

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication
from axipy import Layer, provider_manager, TabFile
from osgeo import osr

from BoundTable.toolprocessing.utils import getTableBounds
from RasterGridTool.Test.Grid.ui.DataSourceWidget import DlgSource
from RasterGridTool.Test.Grid.ui.GridSize import DlgGridSize
from RasterGridTool.Test.Grid.ui.dlgOutFileGrid import WidgetOutFile
from RasterGridTool.Test.Grid.ui.dlgParamIdw import ParamIdw
from RasterGridTool.Test.Grid.ui.process_tool.build_grid import CreateGrid,  createGrid
def getBoundTable(table):
    layer_tab=Layer.create(table)
    bound=layer_tab.bounds
    return bound
class DlgFromIDW(QWidget):
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        self.__parent=parent
        super(DlgFromIDW, self).__init__(parent)
        self.load_ui()
        self.__gridsize_widget=DlgGridSize()
        self.__datasource=DlgSource()
        self.__paramIdw=ParamIdw()
        self.__dlgSaveGrid=WidgetOutFile()
        self.window.vl_datasource.addWidget(self.__datasource.window)
        self.window.vrt_params.addWidget(self.__paramIdw.window)
        self.window.vrt_filesave.addWidget(self.__dlgSaveGrid.window)
        #self.window.name_select_table.setText(self.__datasource.curentTable)
        self.__change_source(self.__datasource.curentTable)
        self.__datasource.updateTable.connect(self.__change_source)
        self.__datasource.updateField.connect(self.__change_source_field)
        self.window.vrl_sizegrid.addWidget(self.__gridsize_widget.window)
        self.window.pb_run.clicked.connect(self.__thread_run)
        self.window.pb_cancel.clicked.connect(self.__cancel)
        self.__processGrid=CreateGrid(self)
        self.__processGrid.updateProcess.connect(self.__update_progress)
        self.__processGrid.finished.connect(self.__end_process)
        #name_select_table
    def load_ui(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "widgetIdw.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window  = loader.load(ui_file,self.__parent)
        ui_file.close()
    def __change_source(self,name_table):
        #self.window.name_select_table.setText(name_table)
        #bound,bound_rec_geo=getTableBounds(axipy.da.data_manager.find(name_table),True)
        bound_rec_geo=getBoundTable(axipy.da.data_manager.find(name_table))
        self.__gridsize_widget.setBound(bound_rec_geo.xmin,bound_rec_geo.ymin,bound_rec_geo.xmax,bound_rec_geo.ymax)
        name_field=self.__datasource.curentField
        out_file=self.__createDefOutPath(name_table,name_field)
        self.__dlgSaveGrid.setDefaultFileName(out_file)
    def __change_source_field(self,name_field):
        name_tab=self.__datasource.curentTable
        new_out_path=self.__createDefOutPath(name_tab,name_field)
        self.__dlgSaveGrid.setDefaultFileName(new_out_path)
    def __createDefOutPath(self,name_tab,name_field):
        tab_source=axipy.da.data_manager.find(name_tab)
        file_name=tab_source.properties['fileName']
        path_=Path(file_name)
        folder=str(path_.parent)
        name_file=str(path_.stem)
        out_file_with_ext=name_file+"_"+name_field

        return os.path.join(folder,out_file_with_ext)
    def __run(self):
        name_method=self.__paramIdw.nameMethod
        string_format="Интерполяция "+name_method+".. %p%"
        #self.window.progress_bar.setFormat('Read .. %p%')
        self.window.pr_bar.setFormat(string_format)
        self.window.pr_bar.setRange(0,100)
        param_algorithm=self.__paramIdw.optionsGridCreate
        path_out_file=self.__dlgSaveGrid.nameOutFile
        name_table=self.__datasource.curentTable
        table_source=axipy.da.data_manager.find(name_table)
        path_source=table_source.properties['tabFile']
        name_field=self.__datasource.curentField
        bound_grid=self.__gridsize_widget.bound
        nx=self.__gridsize_widget.Nx
        ny=self.__gridsize_widget.Ny
        self.window.pb_cancel.setEnabled(True)
        self.window.pb_run.setEnabled(False)
        try:
            #cls_build=createGrid()
            #self.__processGrid.run(path_out_file,path_source,name_field,bound_grid,param_algorithm,nx,ny)
            #buildGrid(self,path_out_file,path_source,name_field,bound_grid,param_algorithm,nx,ny)
            cs_proj4=table_source.coordsystem.proj
            source_srs = osr.SpatialReference()
            source_srs.ImportFromProj4(cs_proj4)
            #cls_build.buildGrid(self,path_out_file,path_source,name_field,bound_grid,param_algorithm,nx,ny,source_srs)
            self.__processGrid.run(path_out_file,path_source,name_field,bound_grid,param_algorithm,nx,ny,source_srs)
        except Exception as ex:
            pass
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
    def __thread_run(self):
        self.window.pb_cancel.setEnabled(True)
        self.window.pb_run.setEnabled(False)
        name_method=self.__paramIdw.nameMethod
        string_format="Интерполяция "+name_method+".. %p%"
        #self.window.progress_bar.setFormat('Read .. %p%')
        self.window.pr_bar.setFormat(string_format)
        self.window.pr_bar.setRange(0,100)
        param_algorithm=self.__paramIdw.optionsGridCreate
        path_out_file=self.__dlgSaveGrid.nameOutFile
        name_table=self.__datasource.curentTable
        table_source=axipy.da.data_manager.find(name_table)
        path_source=table_source.properties['tabFile']
        name_field=self.__datasource.curentField
        bound_grid=self.__gridsize_widget.bound
        nx=self.__gridsize_widget.Nx
        ny=self.__gridsize_widget.Ny

        cs_proj4=table_source.coordsystem.proj
        source_srs = osr.SpatialReference()
        source_srs.ImportFromProj4(cs_proj4)
        self.__processGrid.setParam(path_out_file,path_source,name_field,bound_grid,param_algorithm,nx,ny,source_srs)
        self.thread = QThread()
        self.__processGrid.moveToThread(self.thread)
        self.__processGrid.run()
    def __end_process(self,value):
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
        if value==100:
            ''' ok '''
            path_out=self.__processGrid.pathOutGrid
            name_tab=str(Path(path_out).name)
            '''
            definition = { 'src': path_out,
                           'dataobject': name_tab,
                           'provider': 'GdalDataProvider' }
            tab_grid=provider_manager.open(definition)
            '''
            tab_grid=provider_manager.gdal.open(path_out)
            tab = TabFile()
            out_file_name=os.path.join(str(Path(path_out).parent),str(Path(path_out).name)+".tab")
            tab.generate_tab(tab_grid, out_file_name)
            tab_grid.close()
            tab_grid=provider_manager.openfile(out_file_name)
    def __cancel(self):
        self.cancelProcess.emit()
        self.__processGrid.setCancel()
    def __update_progress(self,value):
        self.window.pr_bar.setValue(value)
        #print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    @property
    def widget(self):
        return self.window

