import os
import tempfile

import axipy
from axipy import Attribute, CoordSystem, Schema, provider_manager, Geometry, Feature, Polygon, MultiPolygon, Style
from osgeo import ogr, gdal, osr

from RasterGridTool.toolprocessing.ui.contour.tool_smothing.Chaikins_smoth import Object


def copyOgrSchemaToAxioma(featDef,cs):
    '''
    Формирование Axioma schema из LayerDef Ogr
    :param featDef: LayerDef Ogr
    :return: Axioma Schema
    '''
    axiSchema=[]
    for i in range(featDef.GetFieldCount()):
        field_def=featDef.GetFieldDefn(i)
        type_field=field_def.GetType()
        name_field=field_def.GetName()
        axi_field_def=None
        if type_field==ogr.OFTInteger:
            axi_field_def=Attribute.integer(name_field)
        if type_field==ogr.OFTReal:
            axi_field_def=Attribute.double(name_field)
        if axi_field_def is not None:
            axiSchema.append(axi_field_def)
    return Schema(axiSchema,coordsystem=cs)

def copyOgrLayerToAxiTab(layer,path_out_tab,intervals,callback=None):
    srs = layer.GetSpatialRef()
    proj4=srs.ExportToProj4()
    axi_cs=CoordSystem.from_proj(proj4)
    featDef = layer.GetLayerDefn()
    count_field=featDef.GetFieldCount()
    schema_axi=copyOgrSchemaToAxioma(featDef,axi_cs)
    axiTable=provider_manager.tab.create_open(path_out_tab, schema_axi)
    count_row=layer.GetFeatureCount()+1
    if callback is not None:
        i_proc_curent=0
    for i in range(count_row):
        if callback is not None:
            cur_proc=(i/count_row)*100
            if int(cur_proc)>i_proc_curent:
                i_proc_curent=int(cur_proc)
                isCancell=callback(i_proc_curent)
                if isCancell==0:
                    break
        ft=layer.GetFeature(i)
        if ft is None:
            continue
        geometry=ft.GetGeometryRef()
        axi_geometry=Geometry.from_wkb(geometry.ExportToWkb(),coordsystem=axi_cs)
        axi_att={}
        for id_field in range(count_field):
            value_field=ft.GetField(id_field)
            name=featDef.GetFieldDefn(id_field).GetName()
            axi_att[name]=value_field
        interval=findInterval(intervals,axi_att['level_min'],axi_att['level_max'])
        ft_axi=None
        if interval is not None:
            axi_att['level_min']=interval['min']
            axi_att['level_max']=interval['max']
            str_mi_style=interval["mi_style"]
            style= Style.from_mapinfo(str_mi_style)

            ft_axi=Feature(axi_att,geometry=axi_geometry,style=style)
        if ft_axi is not None:
            axiTable.insert([ft_axi])
    axiTable.commit()
    axiTable.close()
def findInterval(intarvals,z_min,z_max):
    for index,interval in enumerate(intarvals):
        #if z_min<interval['max'] and z_min>=interval['min']:
        if interval['min']<z_max and interval['min']>=z_min:
            return interval
    return None

def smothingPolygon(axi_polygon,smothing_poly_coef):
    points=axi_polygon.points
    obj = Object(listPntToTuple(points))
    Smoothed_obj = obj.Smooth_by_Chaikin(number_of_refinements = smothing_poly_coef)
    out_poly=Polygon(Smoothed_obj,cs=axi_polygon.coordsystem)
    hole_source=axi_polygon.holes
    if hole_source is None or len(hole_source)==0:
        return out_poly
    for hole in hole_source:
        obj = Object(listPntToTuple(hole))
        Smoothed_hole = obj.Smooth_by_Chaikin(number_of_refinements = smothing_poly_coef)
        out_poly.holes.append(Smoothed_hole)
    return out_poly
def smothingMultiPolygon(axi_polygon,smothing_poly_coef):
    out_poly=MultiPolygon(cs=axi_polygon.coordsystem)
    for poly in axi_polygon:
        poly_smothing=smothingPolygon(poly,smothing_poly_coef)
        out_poly.append(poly_smothing)
    return out_poly
def copyOgrLayerToAxiTabSmothing(layer,path_out_tab,coefSmothing,intervals,callback=None):
    srs = layer.GetSpatialRef()
    proj4=srs.ExportToProj4()
    axi_cs=CoordSystem.from_proj(proj4)
    featDef = layer.GetLayerDefn()
    count_field=featDef.GetFieldCount()
    schema_axi=copyOgrSchemaToAxioma(featDef,axi_cs)
    axiTable=provider_manager.tab.create_open(path_out_tab, schema_axi)
    count_row=layer.GetFeatureCount()+1
    if callback is not None:
        i_proc_curent=0
    for i in range(count_row):
        if callback is not None:
            cur_proc=(i/count_row)*100
            if int(cur_proc)>i_proc_curent:
                i_proc_curent=int(cur_proc)

                isCancell=callback(i_proc_curent)
                if isCancell==0:
                    break
        ft=layer.GetFeature(i)
        if ft is None:
            continue
        geometry=ft.GetGeometryRef()
        if geometry is None:
            continue
        axi_att={}
        for id_field in range(count_field):
            value_field=ft.GetField(id_field)
            name=featDef.GetFieldDefn(id_field).GetName()
            axi_att[name]=value_field
        axi_geometry=Geometry.from_wkb(geometry.ExportToWkb(),coordsystem=axi_cs)
        print(axi_geometry.name)
        smoth_geom_axi=None
        if isinstance(axi_geometry,axipy.da.Polygon):
            ''' Сглаживаем полигон '''
            smoth_geom_axi=smothingPolygon(axi_geometry,coefSmothing)
        if isinstance(axi_geometry,axipy.da.MultiPolygon):
            smoth_geom_axi=smothingMultiPolygon(axi_geometry,coefSmothing)
        if smoth_geom_axi is not None:
            interval=findInterval(intervals,axi_att['level_min'],axi_att['level_max'])
            ft_axi=None
            if interval is not None:
                axi_att['level_min']=interval['min']
                axi_att['level_max']=interval['max']
                str_mi_style=interval["mi_style"]
                style= Style.from_mapinfo(str_mi_style)
                ft_axi=Feature(axi_att,geometry=smoth_geom_axi,style=style)
            if ft_axi is not None:
                axiTable.insert([ft_axi])
        '''
        count_geometry=geometry.GetGeometryCount()
        list_points=[]
        for i in range(count_geometry):
            geo_i=geometry.GetGeometryRef(i)
            points=geo_i.GetPoints()
            print("Source count:"+str(len(points)))
            obj = Object(points)
            Smoothed_obj = obj.Smooth_by_Chaikin(number_of_refinements = 1.5)
            print("Smoth count:"+str(len(Smoothed_obj)))
            list_points.append(Smoothed_obj)
            geo_i=None
        try:
            smoth_geom=createGeometry(1,list_points)
            axi_geometry=Geometry.from_wkb(smoth_geom.ExportToWkb())
        except:
            smoth_geom=None
            axi_geometry=None
            ft_axi=Feature(axi_att,geometry=axi_geometry)
            axiTable.insert([ft_axi])
            '''
    axiTable.commit()
    axiTable.close()
def createLineRing(points):
    outRing = ogr.Geometry(ogr.wkbLinearRing)
    for pnt in points:
        outRing.AddPoint(pnt[0],pnt[1])
    return outRing
def createGeometry(type_geometry,points):
    if type_geometry==0:
        pass
    if type_geometry==1:
        ''' Polygon '''
        poly = ogr.Geometry(ogr.wkbPolygon)
        for pnts in points:
            outRing = createLineRing(pnts)
            poly.AddGeometry(outRing)
        return poly
def listPntToTuple(list_point):
    list_uple_xy=[]
    for pnt in list_point:
        list_uple_xy.append((pnt.x,pnt.y))
    return list_uple_xy
class tempQueryTab:
    def __init__(self,cs_wkt):
        self.__cs_wkt=cs_wkt
        self.__name_layer="data_grid"
        self.__create(cs_wkt)
    def __create(self,cs_wkt):
        temp_name=next(tempfile._get_candidate_names())
        self.__path_mem_shp=os.path.join("/vsimem/",temp_name+".shp")
        driver_shp=ogr.GetDriverByName('ESRI Shapefile')
        self.__ds = driver_shp.CreateDataSource(self.__path_mem_shp)
        if self.__ds is None:
            return False
        srs =  osr.SpatialReference()
        srs.ImportFromWkt(cs_wkt)
        self.__layer = self.__ds.CreateLayer(self.__name_layer, srs, ogr.wkbUnknown)
        idField = ogr.FieldDefn("id", ogr.OFTInteger)
        self.__layer.CreateField(idField)
        zField = ogr.FieldDefn("z", ogr.OFTReal)
        self.__layer.CreateField(zField)
        self.__featureDefn=self.__layer.GetLayerDefn()
        return True
    @property
    def pathTemp(self):
        return self.__path_mem_shp
    @property
    def nameFieldZ(self):
        return "z"
    def copy(self,axi_table,name_field_z,call_back=None):
        count=axi_table.count()
        for i,ft in enumerate(axi_table.items()):
            if call_back is not None:
                ret_cl=call_back(float(i)/float(count))
                if ret_cl==0:
                    self.__layer=None
                    self.__ds=None
                    return
            if ft is None or ft.geometry is None:
                continue
            ogr_ft=ogr.Feature(self.__featureDefn)
            geometry=ogr.CreateGeometryFromWkb(ft.geometry.wkb)



            ogr_ft.SetGeometry(geometry)
            ogr_ft.SetField("z", ft[name_field_z])
            ogr_ft.SetField("id", i)
            self.__layer.CreateFeature(ogr_ft)
            geometry=None
            ogr_ft=None
        if call_back is not None:
            ret_cl=call_back(100)
        self.__layer=None
        self.__ds=None
        return
    def close(self):
        self.__layer=None
        self.__ds=None
        gdal.Unlink(self.__path_mem_shp)






