import operator
import tempfile
from math import fabs
from pathlib import Path

from PySide2.QtCore import QObject, Signal
import time

from osgeo import gdal, ogr, osr

#from RasterGridTool.Test.Grid.ui.process_tool.DataSourceContour import MemVectorOgr
#from RasterGridTool.Test.Grid.ui.process_tool.addTools import copyOgrLayerToAxiTabSmothing, copyOgrLayerToAxiTab
from RasterGridTool.toolprocessing.ui.contour.utils.PrepareContour import prepare
from RasterGridTool.toolprocessing.ui.createGrid.process_tool.DataSourceContour import MemVectorOgr
from RasterGridTool.toolprocessing.ui.createGrid.process_tool.addTools import copyOgrLayerToAxiTabSmothing, \
    copyOgrLayerToAxiTab


class CreateContourProcess(QObject):
    updateProcess = Signal(float)
    finished = Signal(int)
    def __init__(self,parent_class):
        self.__erorr_message=None
        super(CreateContourProcess, self).__init__()
        self.__parent=parent_class
    def __process_callback(self,complete, message=None, user_data=None):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(complete*100)
        time.sleep(0.001)
        #self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def __process_callback_save(self,complete, message=None, user_data=None):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(complete)
        time.sleep(0.001)
        #self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def setCancel(self):
        self.__cancelProcess=True
    def setParametrs(self,path_out,levels,source_grid_info,smothing=None):
        self.__path_out=path_out
        self.__levels=levels
        self.__source_path=source_grid_info['path']
        self.__source_band=source_grid_info['band']
        self.__smothing=smothing
        ''' Создаем список интервалов '''
        self.__intervals=[]
        count=len(levels)
        for level in levels:
            self.__intervals.append(level['min'])
        self.__intervals.append(levels[count-1]['max'])
    def run_old(self):
        self.__tab=[0]
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess=False
        '''
        Открываем исходный растр ( grid)
        '''
        ds_source=gdal.Open(self.__source_path)
        if ds_source is None:
            self.__cancelProcess=True
            self.__erorr_message="Ошибка открытия исходного растра "
            return
        grid_band=ds_source.GetRasterBand(self.__source_band)
        noDataValue=grid_band.GetNoDataValue()
        source_src=ds_source.GetProjectionRef()
        ''' Создание таблицы для контуров '''
        extention_out_file=Path(self.__path_out).suffix
        out_drv_name="MapInfo File"
        if extention_out_file.lower()==".tab":
            out_drv_name="MapInfo File"
        else:
            if extention_out_file.lower()==".shp":
                out_drv_name="ESRI Shapefile"
        ogr_ds_contour = ogr.GetDriverByName(out_drv_name).CreateDataSource(self.__path_out)
        if ogr_ds_contour is None:
            ds_source=None
            self.__cancelProcess=True
            self.__erorr_message="Ошибка создания векторного исчточника"
            return
        out_srs= osr.SpatialReference()
        out_srs.ImportFromProj4(source_src)
        name=Path(self.__path_out).stem
        ogr_lyr = ogr_ds_contour.CreateLayer(name,out_srs, geom_type=ogr.wkbMultiPolygon)
        if ogr_lyr is None:
            ogr_ds_contour=None
            ds_source=None
            self.__cancelProcess=True
            self.__erorr_message="Ошибка создания векторного источника CreateLayer"
            return
        ''' Создаем структуру выходной таблицы ( слоя )'''
        field_defn = ogr.FieldDefn('id', ogr.OFTInteger)
        ogr_lyr.CreateField(field_defn)
        field_defn = ogr.FieldDefn('level_min', ogr.OFTReal)
        ogr_lyr.CreateField(field_defn)
        field_defn = ogr.FieldDefn('level_max', ogr.OFTReal)
        ogr_lyr.CreateField(field_defn)
        fix_interval_options="FIXED_LEVELS="
        converted_list = [str(element) for element in self.__intervals]
        fix_interval_options = fix_interval_options+",".join(converted_list)
        options_countour=[]
        options_countour.append(fix_interval_options)
        options_countour.append('ID_FIELD=0')
        options_countour.append('ELEV_FIELD_MIN=1')
        options_countour.append('ELEV_FIELD_MAX=2')
        options_countour.append('POLYGONIZE=YES')
        options_countour.append('NODATA='+str(noDataValue))
        try:
            gdal.ContourGenerateEx(grid_band, ogr_lyr, options =options_countour,callback=self.__process_callback)
    #gdal.ContourGenerate(curent_band, 0, 0, list_interval, 0, 0, ogr_lyr, 0, 1)
        except Exception as ex:
            print(ex)

        ogr_lyr=None
        ogr_ds_contour=None
        ds_source=None
        ogr_ds_contour=None
        if self.__cancelProcess:

            self.updateProcess.emit(0)
            self.finished.emit(0)
        else:

            self.updateProcess.emit(100)
            self.finished.emit(100)
        if self.__cancelProcess:
            return False
        return True
    def run(self):
        self.__status_process=False
        self.__tab=[0]
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.setFormatprogressBar("Создание изолиний .. %p%")
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess=False
        '''
        Открываем исходный растр ( grid)
        '''
        ds_source=gdal.Open(self.__source_path)
        if ds_source is None:
            self.__cancelProcess=True
            self.__erorr_message="Ошибка открытия исходного растра "
            return
        grid_band=ds_source.GetRasterBand(self.__source_band)
        noDataValue=grid_band.GetNoDataValue()
        source_src=ds_source.GetProjectionRef()
        out_srs= osr.SpatialReference()
        out_srs.ImportFromProj4(source_src)
        ''' Создание таблицы для контуров '''
        name=Path(self.__path_out).stem
        mem_drv=MemVectorOgr(name)
        mem_layer=mem_drv.create(name,out_srs)
        extention_out_file=Path(self.__path_out).suffix

        if mem_layer is None:
            ogr_ds_contour=None
            ds_source=None
            self.__cancelProcess=True
            self.__erorr_message="Ошибка создания векторного источника CreateLayer"
            return

        fix_interval_options="FIXED_LEVELS="
        converted_list = [str(element) for element in self.__intervals]
        fix_interval_options = fix_interval_options+",".join(converted_list)
        options_countour=[]
        options_countour.append(fix_interval_options)
        options_countour.append('ID_FIELD=0')
        options_countour.append('ELEV_FIELD_MIN=1')
        options_countour.append('ELEV_FIELD_MAX=2')
        options_countour.append('POLYGONIZE=YES')
        str_noData=str(noDataValue)
        if str_noData is None:
            str_noData="65535"
        options_countour.append('NODATA='+str_noData)
        try:
            gdal.ContourGenerateEx(grid_band, mem_layer, options =options_countour,callback=self.__process_callback)
        #gdal.ContourGenerate(curent_band, 0, 0, list_interval, 0, 0, ogr_lyr, 0, 1)
        except Exception as ex:
            print(ex)
        if mem_layer.GetFeatureCount()>0:
            count_ft=mem_layer.GetFeatureCount()
            for i in range(count_ft+1):
                ft=mem_layer.GetFeature(i)
                if ft is None:
                    continue
                min_v=ft.GetField(1)
                max_v=ft.GetField(2)
                if max_v<min_v:
                    ft.SetField("level_min",self.__intervals[0])
                    mem_layer.SetFeature(ft)
                ft=None
                jkl=0
        grid_band=None
        ds_source=None
        self.updateProcess.emit(100)
        self.updateProcess.emit(0)
        if self.__cancelProcess:
            self.finished.emit(100)

            return False
        if self.__smothing is not None:
            self.__parent.setFormatprogressBar("Сохранение и сглаживание .. %p%")
            copyOgrLayerToAxiTabSmothing(mem_layer,self.__path_out,self.__smothing,self.__levels,self.__process_callback_save)
        else:
            self.__parent.setFormatprogressBar("Сохранение  .. %p%")
            copyOgrLayerToAxiTab(mem_layer,self.__path_out,self.__levels,self.__process_callback_save)
        self.updateProcess.emit(100)
        mem_drv.close()
        ogr_lyr=None
        ogr_ds_contour=None
        ds_source=None
        ogr_ds_contour=None
        if self.__cancelProcess:

            self.updateProcess.emit(0)
            self.finished.emit(0)
        else:

            self.updateProcess.emit(100)
            self.finished.emit(100)
        if self.__cancelProcess:

            return False

        return True
    def __isCancel(self):
        self.__cancelProcess=True







