import multiprocessing
import time
from pathlib import Path

import numpy as np
from PySide2 import QtCore
from PySide2.QtCore import QObject, Signal
from axipy import CoordSystem

from osgeo import gdal

from RasterGridTool.toolprocessing.ui.clipRaster.tool.rasterTool import getOutFormatByExtantion, \
    createRasterRegistrationTab
from RasterGridTool.toolprocessing.ui.createGrid.process_tool.addTools import tempQueryTab
def getGPC(nx,ny,gt):
    gt_point=[]
    x_s=gt[0]
    w_x=gt[1]*(nx-1)
    y_s=gt[3]
    w_y=gt[5]*(ny-1)
    p1=gdal.GCP(x_s,y_s+w_y,0.0,0.0,0.0,'point1','pt1')
    p2=gdal.GCP(x_s+w_x,y_s+w_y,0.0,nx,0.0,'point2','pt2')
    p3=gdal.GCP(x_s+w_x,y_s,0,nx,ny,'point3','pt3')
    p4=gdal.GCP(x_s,y_s+w_y,0,0,ny,'point4','pt4')
    #p1=gdal.GCP(x_s,y_s,0.0,0.0,0.0,'point1','pt1')
    #p2=gdal.GCP(x_s+w_x,y_s,0.0,nx,0.0,'point2','pt2')
    #p3=gdal.GCP(x_s+w_x,y_s+w_y,0,nx,ny,'point3','pt3')
    #p4=gdal.GCP(x_s,y_s+w_y,0,0,ny,'point4','pt4')
    gt_point.append(p1)
    gt_point.append(p2)
    gt_point.append(p3)
    gt_point.append(p4)

    new_gt=[gt[0],gt[1],gt[2],y_s+w_y,gt[4],-gt[5]]
    #new_gt=[gt[0],gt[1],gt[2],y_s,gt[4],gt[5]]
    return gt_point,new_gt
def reverseArray(source_array):
    nx,ny=source_array.shape
    array_result=np.arange(nx*ny,dtype=np.float32).reshape(nx,ny)
    for j in range(nx):
        j_r=nx-1-j
        array_result[j,:]=source_array[j_r,:]
    return array_result
def MemToGpkg(mem_ds,path_out_gpkg,nodata):
    gt=mem_ds.GetGeoTransform()
    outDrv = gdal.GetDriverByName('GPKG')
    nx=mem_ds.RasterXSize
    ny=mem_ds.RasterYSize
    gpc,gt_new=getGPC(nx,ny,gt)
    #nodata=mem_ds.GetRasterBand(1).GetNoDataValue()
    out = outDrv.Create(path_out_gpkg, nx, ny, 1, gdal.GDT_UInt16,options=['BLOCKXSIZE=256', 'BLOCKYSIZE=256','TILE_FORMAT=PNG','ZLEVEL=1'])
    #out = outDrv.Create(path_out_gpkg, nx, ny, 1, gdal.GDT_Float32,options=['BLOCKXSIZE=256', 'BLOCKYSIZE=256'])
    out.SetProjection(mem_ds.GetProjection())
    out.GetRasterBand(1).SetNoDataValue(nodata)
    #temp_gpc=mem_ds.GetGCPs()
    #out.SetGCPs(gpc,'')
    #out.SetGeoTransform(gt_new)
    out.SetGeoTransform(gt)
    #out.AddBand(gdal.GDT_UInt16)
    data_grd=mem_ds.GetRasterBand(1).ReadAsArray()
    #data_grd_rev=reverseArray(data_grd)
    out.GetRasterBand(1).WriteArray( data_grd)
    del data_grd
    #del data_grd_rev
    mem_ds=None
    out.GetRasterBand(1).FlushCache()
    out.FlushCache()
    out=None

class createGrid:

    def process_callback(self,complete, message, user_data):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
    #if self.__cancelProcess:
    #    return 0
    #self.updateProcess.emit(complete*100)
        self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def buildGrid(self,parent_clss,path_out,path_source,name_field,bound_out,param_method,nx,ny,srs_out,type_out=gdal.GDT_Float32,noData=-99999):
        self.__parent=parent_clss
        tab=[0]
        ext_out=str(Path(path_out).suffix)
        out_format='GTiff'
        if ext_out.lower()=='.gpkg':
            out_format="MEM"
            ds_out=gdal.Grid('',path_source,zfield=name_field,outputBounds=bound_out,width=nx,height=ny,outputType=type_out,noData=noData,algorithm=param_method,format=out_format,outputSRS=srs_out,callback=self.process_callback,callback_data = tab)
            if ds_out is None:
                return False
            drv_gpkg=gdal.GetDriverByName( "GPKG" )
            out_ds = drv_gpkg.CreateCopy( path_out, ds_out, 0 )
            if out_ds is not None:
                out_ds=None
                ds_out=None
                return True
        else:
            print(param_method)
            ds_out=gdal.Grid(path_out,path_source,zfield=name_field,outputBounds=bound_out,width=nx,height=ny,outputType=type_out,noData=noData,algorithm=param_method,format=out_format,outputSRS=srs_out,callback=self.process_callback,callback_data = tab)
            if ds_out is not None:
                ds_out=None
                return True
        return False
        #ds_out=gdal.Grid(path_out,path_source,algorithm='invdist:max_points=150:min_points=1:radius=1800',zfield='PHLR_ABS', format='GTiff', width=1712/6, height=819/6,outputType = gdal.GDT_Float32,callback=self.process_callback,callback_data = tab)

class CreateGrid(QObject):
    updateProcess = Signal(float)
    finished = Signal(int)
    def __init__(self,parent_class):
        super(CreateGrid, self).__init__()
        self.__parent=parent_class
    def __process_callback(self,complete, message, user_data):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        self.updateProcess.emit(complete*100)
        time.sleep(0.001)
        #self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def setCancel(self):
        self.__cancelProcess=True
    def setParam(self,path_out,path_source,name_field,bound_out,param_method,nx,ny,srs_out,type_out=gdal.GDT_Float32,noData=-99999):
        self.__path_out=path_out
        self.__path_source=path_source
        self.__name_field=name_field
        self.__bound_out=bound_out
        self.__param_method=param_method
        self.__nx=nx
        self.__ny=ny
        self.__srs_out=srs_out
        self.__type_out=type_out
        self.__noData=noData
    def run(self):
        self.__path_out_tab=None
        self.__tab=[0]
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess=False
        source_path=self.__path_source
        cls_temp=None

        if isinstance(self.__path_source,dict):
            ''' исходные данные не из tab файла'''
            cls_temp=self.__path_source['cls_temp']
            base_tab=self.__path_source['table']

            cls_temp.copy(base_tab,self.__name_field)
            source_path=cls_temp.pathTemp
            self.__name_field=cls_temp.nameFieldZ
        ds_out=None
        try:
            out_format='GTiff'
            #count_proc=multiprocessing.cpu_count()
            #gdal.SetConfigOption('GDAL_NUM_THREADS', str(count_proc/2))
            print(self.__param_method)
            out_format_temp=getOutFormatByExtantion(Path(self.__path_out).suffix)
            if out_format_temp=="GPKG":
                out_format="MEM"
                #out_format="GPKG"
                ds_out=gdal.Grid('',source_path,zfield=self.__name_field,outputBounds=self.__bound_out,width=self.__nx,height=self.__ny,outputType=gdal.GDT_UInt16,noData=self.__noData,algorithm=self.__param_method,format=out_format,outputSRS=self.__srs_out,callback=self.__process_callback)
                MemToGpkg(ds_out,self.__path_out,self.__noData)
            else:
                ds_out=gdal.Grid(self.__path_out,source_path,zfield=self.__name_field,outputBounds=self.__bound_out,width=self.__nx,height=self.__ny,outputType=self.__type_out,noData=self.__noData,algorithm=self.__param_method,format=out_format,outputSRS=self.__srs_out,callback=self.__process_callback)
                if ds_out is not None:
                    ds_out.GetRasterBand(1).FlushCache()
                    ds_out.GetRasterBand(1).SetNoDataValue(self.__noData)
        except Exception as ex:
            ds_out=None
        if cls_temp is not None:
            cls_temp.close()
        #gdal.SetConfigOption('GDAL_NUM_THREADS', 'ALL_CPUS')
        if self.__cancelProcess:
            ds_out.FlushCache()
            ds_out==None
            self.updateProcess.emit(0)
            self.finished.emit(0)
        else:
            if ds_out is not None:
                path_dest = Path(self.__path_out)
                folder = path_dest.parent
                name = str(path_dest.stem)
                self.__path_out_tab = str(folder.joinpath(name + ".tab"))
                cs_axi=CoordSystem.from_wkt(self.__srs_out.ExportToWkt())
                createRasterRegistrationTab(ds_out, self.__path_out_tab, cs_axi)
                ds_out.GetRasterBand(1).FlushCache()
                ds_out.FlushCache()
                ds_out=None
            self.updateProcess.emit(100)
            self.finished.emit(100)
        if self.__cancelProcess:
            return False
        #self.finished.emit()
        if ds_out is not None:
            ds_out=None
            return True
        return False
    @property
    def pathOutGrid(self):
        return self.__path_out_tab
    def run_old(self,path_out,path_source,name_field,bound_out,param_method,nx,ny,srs_out,type_out=gdal.GDT_Float32,noData=65535):
        self.__tab=[0]
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess=False
        try:
            '''
            path_source="E:\\Support\\GridTest\\relief200_gk7.tab"
            path_out="E:\\Support\\GridTest\\relief200_gk7_grd_demo.tif"
            #ds_out=gdal.Grid(path_out,path_source,zfield=name_field,outputBounds=bound_out,width=nx,height=ny,outputType=type_out,noData=noData,algorithm=param_method,callback=self.__process_callback,callback_data = self.__tab)
            ds_out=gdal.Grid(path_out,path_source,algorithm='invdist:max_points=150:min_points=1:radius=1800',zfield='PHLR_ABS', format='GTiff', width=1712/6, height=819/6,outputType = gdal.GDT_Float32,callback=self.__process_callback,callback_data = self.__tab)
            #ds_out=gdal.Grid(path_out,path_source,algorithm='invdist:max_points=150:min_points=1:radius=1800',zfield=name_field, format='GTiff', width=nx, height=ny,outputType = gdal.GDT_Float32,callback=self.__process_callback,callback_data = self.__tab)
            '''
            print(param_method)
            out_format='GTiff'
            ds_out=gdal.Grid(path_out,path_source,zfield=name_field,outputBounds=bound_out,width=nx,height=ny,outputType=type_out,noData=noData,algorithm=param_method,format=out_format,outputSRS=srs_out,callback=self.__process_callback,callback_data = self.__tab)


        except Exception as ex:
            pass
        self.updateProcess.emit(100)
        self.finished.emit()
        #self.finished.emit()
        if ds_out is not None:
            ds_out=None
            return True
        return False


    def __isCancel(self):
        self.__cancelProcess=True