import os
import time
from pathlib import Path

import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication
from axipy import Layer, provider_manager, TabFile, Settings, mainwindow
from axipy.app import Notifications
from osgeo import osr


from RasterGridTool.toolprocessing.ui.addTools.mapTool import addRasterTabLayerUpTab
from RasterGridTool.toolprocessing.ui.addTools.utils import RasterIsOpen
from RasterGridTool.toolprocessing.ui.createGrid.DataSourceWidget import DlgSource
from RasterGridTool.toolprocessing.ui.createGrid.GridSize import DlgGridSize
from RasterGridTool.toolprocessing.ui.createGrid.dlgOutFileGrid import WidgetOutFile
from RasterGridTool.toolprocessing.ui.createGrid.dlgParamIdw import ParamIdw
from RasterGridTool.toolprocessing.ui.createGrid.formIdwNn import DlgIdwNn
from RasterGridTool.toolprocessing.ui.createGrid.formNearest import DlgNearest
from RasterGridTool.toolprocessing.ui.createGrid.formTin import DlgTin
from RasterGridTool.toolprocessing.ui.createGrid.process_tool.addTools import tempQueryTab
from RasterGridTool.toolprocessing.ui.createGrid.process_tool.build_grid import CreateGrid
def getBoundFirstObject(tab_source):

    for ft in tab_source.items():

        if ft.geometry.type==axipy.GeometryType.Polygon or ft.geometry.type==axipy.GeometryType.MultiPolygon:
            rect_bound= ft.geometry.boundary()
            return rect_bound
        return None
def clearLayout(layout):
    while layout.count():
        child = layout.takeAt(0)
        if child.widget():
            child.widget().deleteLater()
def getBoundTable(table):
    if table is None:
        return None
    layer_tab=Layer.create(table)
    bound=layer_tab.get_bounds()
    return bound

class FromCreateGrid(QWidget):
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        #print("Init Form Create Grid")
        self.__parent=parent
        super(FromCreateGrid, self).__init__(parent)
        self.load_ui()
        self.__initListMethod()
        self.__initcmbMethod()
        self.window.cmb_method.setCurrentIndex(0)
        self.__change_method_name()
        self.window.cmb_method.currentIndexChanged.connect(self.__change_method_name)
        self.__gridsize_widget=DlgGridSize()
        self.__datasource=DlgSource()

        self.__dlgSaveGrid=WidgetOutFile()
        self.window.vl_datasource.addWidget(self.__datasource.window)
        self.window.vrl_sizegrid.addWidget(self.__gridsize_widget.window)
        self.window.vrt_filesave.addWidget(self.__dlgSaveGrid.window)
        self.__datasource.updateTable.connect(self.__change_source)
        self.__datasource.updateField.connect(self.__change_source_field)
        self.__change_source(self.__datasource.curentTable)
        self.window.pb_run.clicked.connect(self.__thread_run)
        self.window.pb_cancel.clicked.connect(self.__cancel)
        self.__processGrid=CreateGrid(self)
        self.__processGrid.updateProcess.connect(self.__update_progress)
        self.__processGrid.finished.connect(self.__end_process)
        axipy.gui.selection_manager.changed.connect(self.__change_select)
        self.__change_select()
    def load_ui(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "widgetCreateGrid.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window  = loader.load(ui_file,self.__parent)
        ui_file.close()
    def __initListMethod(self):
        self.__method_grid=[]
        self.__method_grid.append({'name':"Idw","cls":'ParamIdw'})
        self.__method_grid.append({'name':"IdwNn","cls":'DlgIdwNn'})
        self.__method_grid.append({'name':"Nearest","cls":'DlgNearest'})
        self.__method_grid.append({'name':"Tin","cls":'DlgTin'})

    def __initcmbMethod(self):
        self.window.cmb_method.clear()
        for item_method in self.__method_grid:
            self.window.cmb_method.addItem(item_method['name'])

    def __change_method_name(self):
        index_method=self.window.cmb_method.currentIndex()
        #widget_method=self.__method_grid[index_method]["cls"]
        name_cls=self.__method_grid[index_method]['name']

        if name_cls=="Idw":
            self.__widget_method=ParamIdw()
        if name_cls=="IdwNn":
            self.__widget_method=DlgIdwNn()
        if name_cls=="Nearest":
            self.__widget_method=DlgNearest()
        if name_cls=="Tin":
            self.__widget_method=DlgTin()
        '''
        item = self.window.vrt_params.itemAt(0)
        count_widget=self.window.vrt_params.count()
        if item is not None:
            #self.window.vrt_params.removeItem(item)
            widget = item.widget()
            widget.deleteLater()
        #if index_method<2:
        '''
        clearLayout(self.window.vrt_params)

        try:
            self.window.vrt_params.addWidget(self.__widget_method.window)
        except Exception as ex:
            jkl=0
        #self.window.vrt_params.update()
        return
    def __change_source(self,name_table):
        #self.window.name_select_table.setText(name_table)
        #bound,bound_rec_geo=getTableBounds(axipy.da.data_manager.find(name_table),True)
        bound_rec_geo=getBoundTable(axipy.da.data_manager.find(name_table))
        if bound_rec_geo is None:
            return
        self.__gridsize_widget.setBound(bound_rec_geo.xmin,bound_rec_geo.ymin,bound_rec_geo.xmax,bound_rec_geo.ymax,axipy.da.data_manager.find(name_table).coordsystem)
        name_field=self.__datasource.curentField
        out_file=self.__createDefOutPath(name_table,name_field)
        self.__dlgSaveGrid.setDefaultFileName(out_file)
    def __change_source_field(self,name_field):
        name_tab=self.__datasource.curentTable
        new_out_path=self.__createDefOutPath(name_tab,name_field)
        self.__dlgSaveGrid.setDefaultFileName(new_out_path)
    def __createDefOutPath(self,name_tab,name_field):
        tab_source=axipy.da.data_manager.find(name_tab)
        try:
            file_name=tab_source.properties['fileName']
            path_=Path(file_name)
            folder=str(path_.parent)
            name_file=str(path_.stem)
        except:
            folder=Settings.value(Settings.LastOpenPath)
            name_file=name_tab
        out_file_with_ext=name_file+"_"+name_field

        return os.path.join(folder,out_file_with_ext)
    def setFormatprogressBar(self,str_format):
        self.window.pr_bar.setFormat(str_format)
    def __thread_run(self):
        path_out_file=self.__dlgSaveGrid.nameOutFile
        tabExist=RasterIsOpen(path_out_file)
        if tabExist is not None:
            nameClose=tabExist.name
            tabExist.close()
            Notifications.push('Создание регулярного грида', 'Закрыта т.к создается новый грид с таким же именем '+nameClose, Notifications.Warning)
        self.window.pb_cancel.setEnabled(True)
        self.window.pb_run.setEnabled(False)
        index_method=self.window.cmb_method.currentIndex()
        widget_method=self.__widget_method
        name_method=widget_method.nameMethod
        string_format="Интерполяция "+name_method+".. %p%"
        #self.window.progress_bar.setFormat('Read .. %p%')
        self.window.pr_bar.setFormat(string_format)
        self.window.pr_bar.setRange(0,100)
        param_algorithm=widget_method.optionsGridCreate

        name_table=self.__datasource.curentTable

        table_source=axipy.da.data_manager.find(name_table)
        name_field=self.__datasource.curentField
        try:
            path_source=table_source.properties['fileName']
        except:
            ''' Возможно это запрос'''

            path_source={'cls_temp':tempQueryTab(table_source.coordsystem.wkt),'table':table_source}

        bound_grid=self.__gridsize_widget.bound
        nx=self.__gridsize_widget.Nx
        ny=self.__gridsize_widget.Ny

        cs_proj4=table_source.coordsystem.proj
        cs_wkt=table_source.coordsystem.wkt
        source_srs = osr.SpatialReference()
        #source_srs.ImportFromProj4(cs_proj4)
        source_srs.ImportFromWkt(cs_wkt)
        self.__processGrid.setParam(path_out_file,path_source,name_field,bound_grid,param_algorithm,nx,ny,source_srs)
        self.thread = QThread()
        self.__processGrid.moveToThread(self.thread)
        self.__processGrid.run()
    def __end_process(self,value):
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(True)
        if value==100:
            ''' ok '''
            self.window.pr_bar.setValue(0)
            path_out=self.__processGrid.pathOutGrid
            '''
            name_tab=str(Path(path_out).name)
           
            definition = { 'src': path_out,
                           'dataobject': name_tab,
                           'provider': 'GdalDataProvider' }
            tab_grid=provider_manager.open(definition)
            
            tab_grid=provider_manager.gdal.open(path_out)
            tab = TabFile()
            out_file_name=os.path.join(str(Path(path_out).parent),str(Path(path_out).stem)+".tab")
            if tab_grid is None:
                Notifications.push('Создание регулярного грида', 'Произошла ошибка', Notifications.Critical)
                return
            tab.generate_tab(tab_grid, out_file_name)
            tab_grid.close()
            tab_grid=provider_manager.openfile(out_file_name)
            name_tab_source=self.__datasource.curentTable
            tab_source=axipy.da.data_manager.find(name_tab_source)
            addRasterTabLayerUpTab(tab_source,tab_grid)
            '''
            tab_grid=provider_manager.openfile(path_out)
            Notifications.push('Создание регулярного грида', 'Успешно создан '+tab_grid.name, Notifications.Success)
            layer = Layer.create(tab_grid)
            mainwindow.add_layer_interactive(layer)
        else:
            Notifications.push('Создание регулярного грида', 'Произошла ошибка', Notifications.Critical)
    def __cancel(self):
        self.cancelProcess.emit()
        self.__processGrid.setCancel()
    def __update_progress(self,value):
        self.window.pr_bar.setValue(value)
        #print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def updateTableList(self,list_open_table):
        self.__datasource.updateListTableOpen(list_open_table)
    def addTable(self,name_tab):
        self.__datasource.addTab(name_tab)
    def removeTable(self,name_table):
        return self.__datasource.removeTable(name_table)
    def __change_select(self):
        rect_bound=None
        if axipy.gui.selection_manager.count==1:
            tab_sel=axipy.gui.selection_manager.get_as_table()
            rect_bound=getBoundFirstObject(tab_sel)
        if rect_bound is None:
            return
        self.__gridsize_widget.set_bound_selection(rect_bound)



    @property
    def allTabelNames(self):
        return self.__datasource.allTableNames
    @property
    def widget(self):
        return self.window


