import os
from math import sqrt

import axipy

from PySide2.QtCore import QFile
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget

from axipy import Table


from .DlgVariogramDirection import WidgetVariogRamDirection
from .DlgViewVariogram import ViewVariogram
from .calcTool.addTools import workerLoadData
from .calcTool.utils import getTableBounds, factoryVariogramDataForPlot


def listNumericField(table:Table):
    shema_tab=table.schema
    list_numeric_fields=[]
    for att in shema_tab:
        if att.type_string=='int' or att.type_string=='double':
            list_numeric_fields.append(att.name)
    return list_numeric_fields
class DlgCalcVariogram(QWidget):
    __max_count_interval=50

    def __init__(self,parent=None):
        self.__loadData=None
        self.__parentWin=parent
        super(DlgCalcVariogram, self).__init__(parent)
        self.load_ui()
        self.__directionVario=WidgetVariogRamDirection()
        self.window.vl_dir_variogram.addWidget(self.__directionVario.window)
        self.window.cm_tables.currentIndexChanged.connect(self.__change_tab)
        self.window.pb_run.clicked.connect(self.__run)
        self.window.pb_cancel.clicked.connect(self.__cancel)
        self.updateSourceTables()
    def load_ui(self):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), "FormCalcVariogram.ui")
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window  = loader.load(ui_file,self.__parentWin)
        ui_file.close()
    def updateSourceTables(self):
        self.window.cm_tables.clear()
        tables=axipy.da.data_manager.tables
        if len(tables)==0:
            return
        for i,tab in enumerate(tables):
            numeric_fields=listNumericField(tab)
            if len(numeric_fields)==0:
                continue
            self.window.cm_tables.addItem(tab.name)
            if i==0:
                self.__updateListFields(tab,numeric_fields)
        self.window.cm_tables.setCurrentIndex(0)
    def __updateListFields(self,table,list_fields=None):
        self.window.cm_fields.clear()
        temp_list_fields=list_fields
        if temp_list_fields is None:
            temp_list_fields=listNumericField(table)
        for name in temp_list_fields:
            self.window.cm_fields.addItem(name)
        self.window.cm_fields.setCurrentIndex(0)
    def __change_tab(self,index):
        if self.window.cm_tables.count()==0:
            return
        select_table_name=self.window.cm_tables.currentText()
        #tables=axipy.da.data_manager.tables
        #filter_tab=filter(lambda table: table.name == select_table_name, tables)
        #select_table=list(filter_tab)
        select_table=axipy.da.data_manager.find(select_table_name)
        if select_table is None:
            return
        bound_cs,bound_obj=getTableBounds(select_table,True)
        decart_dist=sqrt((bound_obj.xmax-bound_obj.xmin)*(bound_obj.xmax-bound_obj.xmin)+(bound_obj.ymax-bound_obj.ymin)*(bound_obj.ymax-bound_obj.ymin))
        str_max_dist="{:.2f}".format(decart_dist)
        self.window.lb_max_dist.setText(str_max_dist)
        self.__step=decart_dist/self.__max_count_interval
        str_step="{:.3f}".format(self.__step)
        self.window.ln_step.setText(str_step)
    def __run(self):
        max_distance=float(self.window.lb_max_dist.text())
        direction=self.__directionVario.variogramDirections()
        step_calc=float(self.window.ln_step.text())
        select_table_name=self.window.cm_tables.currentText()

        select_table=axipy.da.data_manager.find(select_table_name)
        if self.__loadData is None:
            self.__loadData=workerLoadData()
            self.__loadData.progressMade.connect(self.update_progress)
            self.__loadData.finished.connect(self.on_finished)
            self.__loadData.setFormatProgress.connect(self.updateFormat)


        name_field_z=self.window.cm_fields.currentText()
        self.__loadData.setPropertyForWork(select_table,name_field_z,direction,max_distance,step_calc)
        self.window.progress_bar.setRange(0,select_table.count())
        self.__curentWorker=self.__loadData
        self.window.progress_bar.setFormat('Read .. %p%')
        self.window.pb_run.setEnabled(False)
        self.window.pb_cancel.setEnabled(True)
        self.__loadData.start()
    def update_progress(self, n_done):
        self.window.progress_bar.setValue(n_done)
    def on_finished(self, done):
        '''
        self.__ui.pb_run.setEnabled(True)
        self.__ui.pb_cancel.setEnabled(False)
        '''
        #self.window.progress_bar.setValue(0)
        self.window.pb_run.setEnabled(True)
        self.window.pb_cancel.setEnabled(False)
        if not self.__loadData.okCalcVario:
            return
        result_vario,disp=self.__loadData.variogram
        step=float(self.window.ln_step.text())
        vario_data=factoryVariogramDataForPlot(step,disp,result_vario)

        main_plot = ViewVariogram(vario_data,axipy.app.mainwindow.qt_object())
        main_plot.show()
    def updateFormat(self,new_format,new_max_progerss):
        self.window.progress_bar.setFormat(new_format)
        if new_max_progerss is not None:
            self.window.progress_bar.setRange(0,new_max_progerss)
    def __process_cancel(self):
        #print("Test")
        self.__curentWorker.terminate()
    def __cancel(self):
        #self.worker.cancel(True)
        print("Cancel")
        self.__curentWorker.terminate()
    def addTab(self,name_tab):
        self.window.cm_tables.addItem(name_tab)
    def removeTable(self,name_tab):
        index_tab_exist=self.window.cm_tables.findText(name_tab)
        if index_tab_exist>=0:
            self.window.cm_tables.removeItem(index_tab_exist)
        return self.window.cm_tables.count()
    def show_x(self):
        self.window.show()
    @property
    def widget(self):
        return self.window
