import time

from PySide2 import QtCore
from axipy import Table, Geometry, Point, MultiPoint, LineString, Polygon, Line, MultiLineString, GeometryCollection

from .VariogramCalulator import DirectionVario
from .VariogramCalulator import VariogrammBuilder
from rtree import index
from math import sqrt
class RtreePoints:
    def __init__(self):
        p = index.Property()
        p.dimension = 2
        self.__idx3d = index.Index(properties=p)
        self.__coords=[]
        self.__id=-1
    def add(self,points):
        for pt in points:
            coord=(pt[0],pt[1])
            nearest = list(self.__idx3d.nearest(coord, 1))
            if len(nearest)==0:
                self.__id=self.__id+1
                self.__idx3d.add(self.__id, coord)
                self.__coords.append(pt)
            else:
                id_out = nearest[0]
                '''
                if id_out!=id:
                    id=id+1
                    idx3d.add(id, coord)
                    coords.append((x,y,z_value))
                    continue
                '''
                try:
                    dx=self.__coords[id_out][0]-coord[0]
                    dy=self.__coords[id_out][1]-coord[1]
                except:
                    jkl=0
                dist=sqrt(dx*dx+dy*dy)
                if dist<=0.0:
                    z_aver=(self.__coords[id_out][2]+pt[2])/2.0
                    self.__coords[id_out]=(self.__coords[id_out][0],self.__coords[id_out][1],z_aver)
                else:
                    self.__id=self.__id+1
                    self.__idx3d.add(self.__id, coord)
                    self.__coords.append(pt)

    @property
    def points(self):
        return self.__coords
def getXYZFromLineString(geo:LineString,z):
    data_xyz=[]
    pts=geo.points
    for pt in pts:
        data_xyz.append((pt.x,pt.y,z))
    return data_xyz
def geometryToXYZ(geo:Geometry,z):
    if geo is None:
        return None
    if isinstance(geo,Point):
        return [(geo.x,geo.y,z)]
    if isinstance(geo,MultiPoint):
        data_points=[]
        for pt in geo:
            data_points.append([(pt.x,pt.y,z)])
        return data_points
    if isinstance(geo,LineString):
        return getXYZFromLineString(geo,z)
    if isinstance(geo,Line):
        geo_lineString=geo.to_linestring()
        return getXYZFromLineString(geo_lineString,z)
    if isinstance(geo,Polygon):
        geo_lineString=geo.to_linestring()
        if isinstance(geo_lineString,MultiLineString):
            data_points=[]
            for geo_polyline in geo_lineString:
                data_point= getXYZFromLineString(geo_polyline,z)
                del data_point[-1]
                data_points.extend(data_point)
            return data_points
        data_points= getXYZFromLineString(geo_lineString,z)
        del data_points[-1]
        return data_points
    if isinstance(geo,GeometryCollection):
        data_points=[]
        for geo_item in geo:
            data_points.extend(geometryToXYZ(geo_item,z))
        return data_points
    return None




class workerLoadData(QtCore.QThread):
    progressMade = QtCore.Signal(int)
    finished = QtCore.Signal(int)
    interrupted = QtCore.Signal()
    setFormatProgress=QtCore.Signal(str,int)
    __isCancel=False
    def __init__(self):
        self.__last_table_name=None
        super(workerLoadData, self).__init__()
        self.__cls_progresBar=None
    def cancel(self,isCancel):
        self.__isCancel=isCancel
    def terminate(self):
        self.__isCancel=True
    def setPropertyForWork(self,data_source:Table,name_parm_z,directions,max_distance,step):
        self.__table=data_source
        self.__name_param_value=name_parm_z
        self.__directions=directions
        self.__max_distance=max_distance
        self.__step=step
    def run(self):
        ''' Чтение данных в пямять '''
        ''' Првоверям данные уже были загружены
        Признак что данные уже были загружены:
        - имя таблицы
        - поле с данными 
        - число загруженных данных 
        
        '''
        self.__isBreakCalcVariogram=True
        self.__isCancel=False
        needLoadData=True
        if self.__last_table_name is not None:
            if self.__last_table_name==self.__table.name and self.__last_field==self.__name_param_value:
                needLoadData=False
        if needLoadData:
            index=0
            points_rtree=RtreePoints()
            self.__data_xyz=[]
            for ft in self.__table.items():
                time.sleep(0.001)
                self.progressMade.emit(index + 1)
                index=index+1
                if self.__isCancel:
                    break
                z_value=ft[self.__name_param_value]
                geo=ft.geometry
                if geo is None or z_value is None:
                    continue
                #self.__data_xyz.extend(geometryToXYZ(geo,z_value))
                points_rtree.add(geometryToXYZ(geo,z_value))
            if not self.__isCancel:
                ''' Операция не была прервана
                запоминаем :
                - имя таблицы 
                - имя поля 
                - число записей
                '''
                self.__data_xyz=points_rtree.points
                self.__last_table_name=self.__table.name
                self.__last_field=self.__name_param_value
                self.__last_count_rec=len(self.__data_xyz)
        if not self.__isCancel:
            ''' Расчитываем вариограмму'''
            list_dir_variogram=[]
            for directions in self.__directions:
                list_dir_variogram.append(DirectionVario(directions['direction'],directions["width"]/2.0))

            varioBuilder=VariogrammBuilder()
            varioBuilder.setParametrs(self.__data_xyz,list_dir_variogram,self.__max_distance,self.__step)

            count_block,size_block=varioBuilder.pre_start()
            self.setFormatProgress.emit('Calc vario .. %p%',count_block)
            self.progressMade.emit(0)
            time.sleep(0.001)
            self.__isBreakCalcVariogram=False
            for id_block in range(count_block):
                time.sleep(0.001)
                self.progressMade.emit(id_block + 1)
                if self.__isCancel:
                    self.__isBreakCalcVariogram=True
                    break
                varioBuilder.runBlock(id_block)
            self.__variogram_result=varioBuilder.result
            self.__disp=varioBuilder.dispersion
            index=count_block
        #self.__isCancel=False
        self.finished.emit(index)
    @property
    def okCalcVario(self):
        return not self.__isBreakCalcVariogram
    @property
    def variogram(self):
        return self.__variogram_result,self.__disp



