import axipy
from axipy import Table, Polygon, Rect, Point, CoordSystem

class VariogramsData:
    def __init__(self,step,dispersion):
        self.__step=step
        self.__dispersion=dispersion
        self.__list_direction_data=[]
    def addDirectionData(self,angle,width,data):
        self.__list_direction_data.append({'direction':angle,'width':width,'data':data})
    @property
    def step(self):
        return self.__step
    @property
    def dispersion(self):
        return self.__dispersion
    @property
    def count(self):
        return len(self.__list_direction_data)
    def getData(self,index):
        return self.__list_direction_data[index]
    @property
    def dataVariogram(self):
        return self.__list_direction_data
    def prepareData(self):
        '''
        Выравниваем по длине
        :return:
        '''
        min_size=0
        for i,item_data in enumerate(self.__list_direction_data):
            if i ==0:
                min_size=len(item_data['data'])
                continue
            if len(item_data['data'])<min_size:
                min_size=len(item_data['data'])
        for item_data in self.__list_direction_data:
            if len(item_data['data'])==min_size:
                continue
            count_remove=len(item_data['data'])-min_size
            data_list=item_data['data']
            del data_list[-count_remove:]

def factoryVariogramDataForPlot(step,disp,result_vario):
    cls_data_for_polt=VariogramsData(step,disp)

    for var_dir in result_vario:
        angleDirection,width=var_dir.parmDirection
        cls_data_for_polt.addDirectionData(angleDirection,width,var_dir.getResultVarioInDirection())
    cls_data_for_polt.prepareData()
    return cls_data_for_polt
class DoubleRect:
    __xmin=None
    __ymin=None
    __xmax=None
    __ymax=None
    def __init__(self,cs:CoordSystem,xmin=None,ymin=None,xmax=None,ymaх=None):
        self.__cs=cs
        self.__xmin=xmin
        self.__ymin=ymin
        self.__xmax=xmax
        self.__ymax=ymaх
    @property
    def xmin(self):
        return self.__xmin
    @property
    def ymin(self):
        return self.__ymin
    @property
    def xmax(self):
        return self.__xmax
    @property
    def ymax(self):
        return self.__ymax
    def merge(self,rect:Rect ):
        if rect.xmin<self.__xmin:
            self.__xmin=rect.xmin
        if rect.xmax>self.__xmax:
            self.__xmax=rect.xmax
        if rect.ymin<self.__ymin:
            self.__ymin=rect.ymin
        if rect.ymax>self.__ymax:
            self.__ymax=rect.ymax
    @property
    def coordsystem(self):
        return self.__cs
    def merge(self,rect):
        self.__merge_point(rect.xmin,rect.ymin)
        self.__merge_point(rect.xmin,rect.ymax)
        self.__merge_point(rect.xmax,rect.ymax)
        self.__merge_point(rect.xmax,rect.ymin)
    def mergePoint(self,point:Point):
        if self.__xmin is None or self.__xmax is None or self.__ymin is None or self.__ymax is None:
            self.__xmin=point.x
            self.__ymin=point.y
            self.__xmax=point.x
            self.__ymax=point.y
            return
        if point.x<self.__xmin:
            self.__xmin=point.x
        if point.x>self.__xmax:
            self.__xmax=point.x
        if point.y<self.__ymin:
            self.__ymin=point.y
        if point.y>self.__ymax:
            self.__ymax=point.y
        return
    def __merge_point(self,x,y):
        if x<self.__xmin:
            self.__xmin=x
        if x>self.__xmax:
            self.__xmax=x
        if y<self.__ymin:
            self.__ymin=y
        if y>self.__ymax:
            self.__ymax=y
        return
    def extendOnProcent(self,value_proc_ext):
        self.__xmin,self.__xmax=self.__calcExtend(value_proc_ext,self.__xmin,self.__xmax)
        self.__ymin,self.__ymax=self.__calcExtend(value_proc_ext,self.__ymin,self.__ymax)
    def __calcExtend(self,value_proc_ext,val_min,val_max):
        dx=val_max-val_min
        val_dx_proc=dx/100
        d_proc=val_dx_proc*value_proc_ext
        return val_min-d_proc,val_max+d_proc
    def reproject(self,new_cs):
        poly=Polygon.from_rect(Rect(self.__xmin,self.__ymin,self.__xmax,self.__ymax),self.__cs)
        poly_rep=poly.reproject(new_cs)
        bound=poly_rep.bounds
        return DoubleRect(new_cs,bound.xmin,bound.ymin,bound.xmax,bound.ymax)
    @property
    def clone(self):
        return DoubleRect(self.__cs,self.__xmin,self.__ymin,self.__xmax,self.__ymax)
    @property
    def boundsStr(self):
        str_bounds="Bounds ("+coordToString(self.__xmin,self.__cs.lat_lon)
        str_bounds=str_bounds+","+coordToString(self.__ymin,self.__cs.lat_lon)+") "
        str_bounds=str_bounds+"("+coordToString(self.__xmax,self.__cs.lat_lon)+","

        str_bounds=str_bounds+coordToString(self.__ymax,self.__cs.lat_lon)+")"
        return str_bounds

'''
Получения границ таблицы ( границы проекции и геометрии)
'''
def getTableBounds(table:Table,isGeometryBound=False):
    if table is None:
        return None,None
    if table.coordsystem is None:
        return None,None
    bound_cs=table.coordsystem.rect
    bound_rec_cs=DoubleRect(table.coordsystem,bound_cs.xmin,bound_cs.ymin,bound_cs.xmax,bound_cs.ymax)
    if not isGeometryBound:
        return bound_rec_cs,None
    ''' Select bound geometry from table'''
    sql="Select Max(MbrMaxX(FromAxiGeo(obj))) as xmax,Max(MbrMaxY(FromAxiGeo(obj))) as ymax,Min(MbrMinX(FromAxiGeo(obj))) as xmin,Min(MbrMinY(FromAxiGeo(obj))) as ymin From "+table.name
    tab_sel=axipy.da.data_manager.query(sql)
    fts=tab_sel.items()
    for ft in fts:
        xmin=ft['xmin']
        xmax=ft['xmax']
        ymin=ft['ymin']
        ymax=ft['ymax']
    bound_rec_geo=DoubleRect(table.coordsystem,xmin,ymin,xmax,ymax)
    tab_sel.close()
    return bound_rec_cs,bound_rec_geo
''' Форматирование координат '''
def coordToString(value,typeCsLatLon=True):
    if typeCsLatLon:
        return f'{value:0.6f}'
    else:
        return f'{value:0.2f}'