import os
import time
from pathlib import Path

import axipy
from PySide2.QtCore import Signal, QFile, QThread
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication
from axipy import data_manager

from RasterGridTool.toolprocessing.Utils.helper_observer import existOneNumberField
from RasterGridTool.toolprocessing.ui.viewshade.tools.process_viewshade import ProcessViewShade


class FormViewShade(QWidget):
    cancelProcess = Signal()
    name_observer = 'GridAxi'
    def __init__(self, parent=None):
        self.__parent = parent
        super(FormViewShade, self).__init__(parent)
        self.__parentWin=parent
        self.load_ui('FromViewShade_5.ui')
        self.ui.cmb_vectot_tab.currentIndexChanged.connect(self.__change_table)
        self.ui.cmb_rasters.currentIndexChanged.connect(self.__change_raster)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.pb_cancel.clicked.connect(self.__cancel)
        #self.__initliatTab()

    def load_ui(self, name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.ui = loader.load(ui_file, self.__parentWin)

        ui_file.close()
    def __initliatTab(self):
        self.ui.cmb_vectot_tab.clear()
        for tab in data_manager.tables:
            existNuberField=existOneNumberField(tab)
            if not existNuberField:
                continue
            self.ui.cmb_vectot_tab.addItem(tab.name)
        self.ui.cmb_vectot_tab.setCurrentIndex(0)
    def __initGridData(self):

        grid_observer=axipy.ObserverManager.get(self.name_observer)
        list_grid=grid_observer.value
        if not list_grid:
            return
        self.ui.cmb_rasters.clear()
        for name in list(list_grid.keys()):
            self.ui.cmb_rasters.addItem(name)
        self.ui.cmb_rasters.setCurrentIndex(0)
    def __change_table(self):
        if self.ui.cmb_vectot_tab.count()==0:
            return
        name_sel_tab=self.ui.cmb_vectot_tab.currentText()
        self.ui.cmb_index_field.clear()
        self.ui.cmb_field_height.clear()
        tab_sel=data_manager.find(name_sel_tab)
        for att in tab_sel.schema:
            self.ui.cmb_index_field.addItem(att.name)
            if att.type_string == "int" or att.type_string == 'double':
                self.ui.cmb_field_height.addItem(att.name)
        self.ui.cmb_index_field.setCurrentIndex(0)
        self.ui.cmb_field_height.setCurrentIndex(0)
        self.__updateOutPath()
    def __change_raster(self):
        self.__updateOutPath()

    def __updateOutPath(self):
        name_sel_tab = self.ui.cmb_vectot_tab.currentText()
        tab_sel = data_manager.find(name_sel_tab)
        file_name=tab_sel.properties['fileName']
        folder_path=Path(file_name).parent
        name_sel_tab = self.ui.cmb_rasters.currentText()
        path_out_tab=str(folder_path.joinpath(name_sel_tab+"_view_shade.tab"))
        self.ui.ln_viewshade_tab.setText(path_out_tab)
        self.ui.ln_viewshade_tab.setCursorPosition(0)
    def __end_process(self,value):
        self.ui.pb_cancel.setEnabled(False)
        self.ui.pb_run.setEnabled(True)
        self.__process.updateProcess.disconnect()
        self.__process.finished.disconnect()
    def __run(self):
        params={}
        params['source_tab']=self.ui.cmb_vectot_tab.currentText()
        name_raster=self.ui.cmb_rasters.currentText()
        grid_observer = axipy.ObserverManager.get(self.name_observer)
        list_grid = grid_observer.value
        axiGrid=list_grid[name_raster]
        params['source_raster']=axiGrid
        params['key_field']=self.ui.cmb_index_field.currentText()
        params['field_height']=self.ui.cmb_field_height.currentText()
        params['path_out_tab']=self.ui.ln_viewshade_tab.text()
        self.__process = ProcessViewShade(self)
        self.__process.setParams(params)
        self.__process.updateProcess.connect(self.__update_progress)
        self.__process.finished.connect(self.__end_process)
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        self.ui.pb_cancel.setEnabled(True)
        self.ui.pb_run.setEnabled(False)
        self.__process.run()
    def __update_progress(self,value_main):

        self.ui.prog_bar.setValue(value_main)

            # print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def setFormatprogressBar(self, str_format_main):
        self.ui.prog_bar.setFormat(str_format_main)

        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __cancel(self):
        self.__process_status=False
        self.cancelProcess.emit()
        self.__process.setCancel()
    @property
    def widget(self):
        return self.ui

    def show(self):
        # self.ui.showMaximized()
        # print("Show Form")
        self.__initliatTab()
        self.__initGridData()
        self.ui.show()
    def updateSource(self):
        self.__initliatTab()
        self.__initGridData()
