from PySide2.QtCore import QObject, Signal
from axipy import data_manager, TypeSqlDialect, Schema, Attribute, provider_manager, Geometry, Feature, Style, \
    Notifications
from osgeo import gdal, osr, ogr
import time

def create_viewshade_point(x,y,height_point,source_raster_band,сs_source,fnCancel,callback_progress):
    #noDataValue=source_raster_band.GetNoDataValue()
    callback_progress(0,None,-1)
    out_ds = gdal.ViewshedGenerate(
        srcBand=source_raster_band,
        driverName='MEM',
        targetRasterName='',
        creationOptions=[],
        observerX=x,
        observerY=y,
        observerHeight=height_point,
        targetHeight=0,
        visibleVal=255,
        invisibleVal=0,
        outOfRangeVal=0,
        noDataVal=0,
        dfCurvCoeff=0.85714,
        mode=2,
        maxDistance=0,
        callback=callback_progress
    )
    if fnCancel():
        out_ds=None
        return None
    if out_ds is None:
        return None

    out_srs = osr.SpatialReference()
    out_srs.ImportFromWkt(сs_source.wkt)
    ''' Создаем векторный слой в памяти'''
    drv = ogr.GetDriverByName('MEMORY')
    dst_ds = drv.CreateDataSource('memData')
    dst_layername = "view_shade"
    dst_layer = dst_ds.CreateLayer(dst_layername, srs=out_srs)
    srcband = out_ds.GetRasterBand(1)
    mask = srcband.GetMaskBand()
    callback_progress(0, None, -1)
    gdal.Polygonize(srcband, mask, dst_layer, -1, [], callback=callback_progress)
    geo_collection = ogr.Geometry(ogr.wkbGeometryCollection)
    for feature in dst_layer:
        geom = feature.GetGeometryRef()
        geo_collection.AddGeometry(geom)
    srcband=None
    out_ds=None
    dst_layer=None
    dst_ds=None
    return geo_collection
class ProcessViewShade(QObject):
    updateProcess = Signal(float)
    finished = Signal(int)
    def __init__(self,parent_cls=None):
        super(ProcessViewShade, self).__init__()
        self.__parent_cls=parent_cls
    def setParams(self,params):
        self.__params=params
    def __process_callback(self,value_proc, message=None, dop_parm=None):
        #print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        if dop_parm==-1:
            self.__curent_procent=self.__curent_procent+self.__process_proc
            self.__process_proc=0

        else:
            self.__process_proc=value_proc*100/(2*self.__count_point)
        self.updateProcess.emit(self.__curent_procent+self.__process_proc)
        time.sleep(0.001)
        #self.__parent.window.pr_bar.setValue(complete*100)
        #print(complete*100)
        return 1
    def __simple_progress(self,value_proc,message=None,dop_parm=None):
        if dop_parm==-1:
            self.__curent_procent=self.__curent_procent+self.__process_proc
            self.__process_proc=0

        else:
            self.__process_proc=value_proc*100/(2*self.__count_point)
        #self.__curent_procent=(self.__curent_procent+value_proc/(2*self.__count_point))
        print(self.__curent_procent+self.__process_proc)
        #print(self.__curent_procent)
    def getCancel(self):
        return self.__cancelProcess
    def setCancel(self):
        self.__cancelProcess=True
    def run(self):
        self.__curent_procent=0
        self.__process_proc=0
        name_tab_source=self.__params['source_tab']
        name_key_field=self.__params['key_field']
        field_height=self.__params['field_height']
        source_raster=self.__params['source_raster']
        path_out_table=self.__params['path_out_tab']
        sql="select * from "+name_tab_source+' Where Str$(obj)="Point"'
        tab_query=data_manager.query_hidden(sql,TypeSqlDialect.axioma)
        bound_raster_cs_tab=source_raster.bound.reproject(tab_query.coordsystem)
        list_points=list(tab_query.items(bound_raster_cs_tab.bounds))
        self.__count_point=len(list_points)
        if self.__count_point==0:
            tab_query.close()
            return
        try:
            self.__parent_cls.disconnect(self.__isCancel)
        except:
            pass
        self.__parent_cls.setFormatprogressBar("Расчет зон видимости .. %p%")
        self.__parent_cls.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess = False
        att_key_field_source=tab_query.schema.by_name(name_key_field)
        schema_viewshade = Schema(
            att_key_field_source,

            coordsystem=source_raster.coordsystem
        )
        out_tab=provider_manager.createfile(path_out_table, schema_viewshade)
        ds_source=gdal.Open(source_raster.pathRaster)
        band_source=ds_source.GetRasterBand(source_raster.IdBand)
        inserts_ft=[]
        for ft in list_points:
            height=ft[field_height]
            key_value=ft[name_key_field]
            #print("Point id="+str(key_value))
            geometry_point=ft.geometry.reproject(source_raster.coordsystem)
            #geo_view_shade=create_viewshade_point(geometry_point.x,geometry_point.y,height,band_source,source_raster.coordsystem,self.getCancel,self.__simple_progress)
            geo_view_shade = create_viewshade_point(geometry_point.x, geometry_point.y, height, band_source,
                                                    source_raster.coordsystem, self.getCancel, self.__process_callback)
            if self.__cancelProcess:
                break
            if geo_view_shade is None:
                continue
            geom_axi = Geometry.from_wkb(geo_view_shade.ExportToWkb(), source_raster.coordsystem)
            geo_view_shade=None
            ft = Feature({name_key_field: key_value}, geometry=geom_axi, style=Style.for_geometry(geom_axi))
            #out_tab.insert([ft])
            inserts_ft.append(ft)
        if len(inserts_ft)>0:
            out_tab.insert(inserts_ft)
            Notifications.push('Расчет зон видимостей', 'Создана таблица зон видимостей '+out_tab.name, Notifications.Success)
        else:
            out_tab.close()
        band_source=None
        ds_source=None
        self.updateProcess.emit(100)
        self.finished.emit(100)
    def __isCancel(self):
        self.__cancelProcess=True








